# include "g2def.h"
  
/* copyright 1984-1988 David Fotland */
 * Permission is granted to use this information in any public domain
 * or personal use only Go program.  If source is distributed, please
 * retain this notice.
 */

 
/* This is an early versio nof the Joseki library from the Cosmos
 * Go playing program.  It contains all the moves from 38 basic Joseki
 * (about 1700 moves) and most of Ishida vol I.
 *
 *  one entry per move - more, next, xy, flags.
 *  int more is index of next sibling - 0 if none
 *  int next is index of first child - 0 if none
 *  int X,Y relative to nearest corner 4 bits each in a byte (X is msbs)
 * flags is bit (0 is msb, 7 is lsb):
 *     1- symetric joseki (only one of symetric children given)
 *     2- color reverse (children are opposite color)
 *     3 - color 
 *     color is 1 if this move is same color as first move in corner
 *     4-7 type is type of joseki move this is
 *	URGT - urgent move
 *	BAD - bad move, but here so can see response
 *	COMP - Complicated continuation, to be avoided
 *	NORM - normal joseki pattern
 *	THFR - this corner only has stone on 3-4 point
 *	FRFR - this corner only has stone on 4-4 point
 *	SHIM - this corner has a 3-4 point based shimari in it
 *	FOLL - follow up move after joseki for later
 *	TRIK - trick move, doesn't work, but use if behind
 *	IGNR - ignore.  This move is not joseki, but following moves may be
 */

struct jos_struct {
	int more;     /* pointer sibling */
	int next;     /* pointer for child */
	char xy;      /* 4 bits x, 4 bits y */
	char flag;   /* 0x80 change jreflect for continuation
			 0x40 make symetric
			 0x20 color reverse
			 0x10 Move is same color as first move in corner
			 0-15 type
			 */
	};


struct jos_struct jlib[NUMJOSEKIS] = {
{     0,  3613, 0*16+ 0, 0x0 +NORM },	/* dummy entry */
{     0,   280, 4*16+ 3, 0x10+THFR },	/* 1  3-4 point */
{     1,  1892, 4*16+ 4, 0x10+FRFR },	/* 2  4-4 point */
{     0,  1439, 6*16+ 3, 0x0 +NORM },	/* 3  4-4 nights approach */
{     3,  1466, 6*16+ 4, 0x0 +NORM },	/* 4  4-4 1 pt high approach */
{     0,  1101, 4*16+ 6, 0x10+URGT },	/* 5  nights approach, one point jump */
{     4,  1894, 3*16+ 3, 0x0 +BAD  },	/* 6  4-4 3-3 invasion */
{     0,  1839, 4*16+ 5, 0x0 +NORM },	/* 7  3-4 one pt high approach */
{     7,     0, 3*16+ 5, 0x10+SHIM },	/* 8  3-4 nights nove shimari */
{     0,    10, 6*16+ 3, 0x10+NORM },	/* 9  4-4 high approach attatch underneath */
{     0,    11, 7*16+ 3, 0x0 +NORM },	/* 10  hane outside */
{     0,     0, 5*16+ 3, 0x10+NORM },	/* 11 extend */
{     6,     0, 3*16+ 3, 0x10+BAD  },	/* 12 4-4 3-3 shimari */
{     9,     0, 5*16+ 6, 0x10+URGT },	/* 13 4-4 high approach extend */
{    12,     0, 4*16+ 3, 0x10+BAD  },	/* 14 4-4 4-3 shimari */
{     0,   169, 3*16+ 5, 0x10+URGT },	/* 15 3-4 high approach hit under */
{     0,    17, 3*16+ 6, 0x0 +URGT },	/* 16 hane outside */
{     0,   136, 3*16+ 4, 0x10+URGT },	/* 17 pull back */
{     8,  2220, 3*16+ 5, 0x0 +NORM },	/* 18  3-4 nights move kakari */
{     0,    20, 4*16+ 3, 0x10+URGT },	/* 19 4-4 3-3 block */
{     0,    21, 3*16+ 4, 0x0 +URGT },	/* 20 4-4 3-3 block push */
{     0,    22, 3*16+ 5, 0x10+URGT },	/* 21 4-4 3-3 block push hane */
{     0,   725, 2*16+ 5, 0x0 +URGT },	/* 22 4-4 ... hane hane */
{     0,   704, 3*16+ 6, 0x10+URGT },	/* 23 4-4 ... hane hane extend */
{    18,     0, 3*16+ 6, 0x10+SHIM },	/* 24 3-4 large nights move shimari */
{    24,     0, 4*16+ 5, 0x10+SHIM },	/* 25 3-4 1 pt high shimari */
{    14,     0, 3*16+ 2, 0x10+BAD  },	/* 26 4-4 3-2 point shimari (no longer used) */
{     5,     0, 4*16+ 3, 0x10+BAD  },	/* 27 4-4 nights approach, block corner */
{     0,   935, 4*16+ 2, 0x0 +NORM },	/* 28 4-4 nights one pt jmp slide */
{     0,  3159, 3*16+ 7, 0x10+NORM },	/* 29 3-4 nights approach, pincher 1 low */
{    29,  3571, 3*16+ 8, 0x10+NORM },	/* 30 3-4 nights approach, pincher 2 low */
{    30,   601, 3*16+ 9, 0x10+NORM },	/* 31 3-4 nights approach, pincher 3 low */
{    31,   510, 4*16+ 7, 0x10+NORM },	/* 32 3-4 nights approach, pincher 1 high */
{    32,   435, 4*16+ 8, 0x10+NORM },	/* 33 3-4 nights approach, pincher 2 high */
{    33,  2380, 7*16+ 3, 0x10+BAD  },	/* 34 3-4 nights approach, 2 pt jump */
{    34,  2214, 5*16+ 4, 0x10+NORM },	/* 35 3-4 nights approach, kosumi */
{    35,  1527, 5*16+ 4, 0x0 +NORM },	/* 36 3-4 nights approach, tenuki */
{     0,   461, 5*16+ 5, 0x0 +NORM },	/* 37 1 low pincer, 1 pt jump */
{     0,   475, 6*16+ 4, 0x10+NORM },	/* 38 1 low pincer resonse to 38 */
{     0,   471, 4*16+ 8, 0x0 +URGT },	/* 39 1 low pincer response to 39 */
{     2,   124, 3*16+ 3, 0x10+NORM },	/* 40 3-3 in empty corner */
{    40,  1769, 4*16+ 5, 0x10+BAD  },	/* 41 5-4 in empty (for 9 line contest) */
{    41,  1736, 3*16+ 5, 0x10+BAD  },	/* 42 5-3 start (for 9 line contest) */
{    26,   800,10*16+ 4, 0x10+IGNR },	/* 43 handicap point start */
{     0,  1415, 6*16+ 3, 0x0 +NORM },	/* 44 standard approach move */
{     0,    46, 5*16+ 3, 0x10+NORM },	/* 45 diagonal attach joseki*/
{     0,    47, 6*16+ 4, 0x0 +NORM },	/* 46 */
{     0,    48, 4*16+ 6, 0x10+URGT },	/* 47 */
{     0,    49, 8*16+ 4, 0x0 +NORM },	/* 48 */
{     0,    50, 9*16+ 3, 0x10+NORM },	/* 49 */
{     0,   942, 8*16+ 3, 0x0 +NORM },	/* 50 */
{    45,    52, 6*16+ 4, 0x10+NORM },	/* 51 attach on top */
{     0,    58, 7*16+ 4, 0x0 +NORM },	/* 52 */
{     0,    54, 6*16+ 5, 0x10+NORM },	/* 53 attach and extend joseki */
{     0,    55, 5*16+ 3, 0x0 +NORM },	/* 54 */
{     0,    56, 4*16+ 3, 0x10+NORM },	/* 55 */
{     0,    57, 8*16+ 3, 0x0 +NORM },	/* 56 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 57 */
{    53,    59, 5*16+ 3, 0x10+NORM },	/* 58 attach and block joseki */
{     0,    60, 6*16+ 5, 0x0 +NORM },	/* 59 */
{     0,    61, 5*16+ 4, 0x10+URGT },	/* 60 */
{     0,    62, 6*16+ 2, 0x0 +NORM },	/* 61 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 62 */
{    51,     0, 4*16+ 3, 0x10+BAD  },	/* 63 knights approach, block corner is bad */
{    44,   710, 4*16+10, 0x50+IGNR },	/* 64 */
{    64,  1447, 3*16+ 6, 0x0 +NORM },	/* 65 other side approach move */
{     0,  1889, 6*16+ 3, 0x0 +NORM },	/* 66 after extend both ways, if comes in,*/
{     0,    76, 6*16+ 4, 0x10+NORM },	/* 67 1 pt jump*/
{    67,     0, 3*16+ 4, 0x10+BAD  },	/* 68 don't just block corner*/
{    68,    70, 3*16+ 8, 0x10+NORM },	/* 69 pincer */
{     0,    71, 3*16+ 3, 0x0 +NORM },	/* 70 enter corner */
{     0,    72, 4*16+ 3, 0x10+NORM },	/* 71 */
{     0,  1262, 3*16+ 4, 0x0 +NORM },	/* 72 */
{     0,  1277, 4*16+ 5, 0x10+NORM },	/* 73 */
{     0,    75, 2*16+ 6, 0x0 +NORM },	/* 74 */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 75 */
{     0,    77, 2*16+ 4, 0x0 +NORM },	/* 76 slide into corner */
{     0,    78, 3*16+ 3, 0x10+NORM },	/* 77 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 78 */
{     0,    80, 3*16+ 3, 0x10+NORM },	/* 79 block other slide */
{     0,   911, 9*16+ 3, 0x0 +NORM },	/* 80 */
{    15,    82, 6*16+ 4, 0x10+NORM },	/* 81 knights resonse to high kakari 3-4*/
{     0,    83, 3*16+ 3, 0x0 +URGT },	/* 82 */
{     0,    84, 3*16+ 2, 0x10+URGT },	/* 83 */
{     0,    86, 3*16+ 4, 0x0 +URGT },	/* 84 */
{     0,    87, 2*16+ 2, 0x10+NORM },	/* 85 */
{    85,    88, 5*16+ 2, 0x10+NORM },	/* 86 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 87 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 88 */
{    27,    90, 5*16+ 3, 0x10+NORM },	/* 89 4-4 knights, kosumi */
{     0,    91, 6*16+ 4, 0x0 +NORM },	/* 90 */
{     0,    92, 4*16+ 6, 0x10+NORM },	/* 91 */
{     0,     0,10*16+ 3, 0x0 +NORM },	/* 92 */
{    89,  1201, 6*16+ 4, 0x10+NORM },	/* 93 4-4 knights, attach */
{     0,   100, 7*16+ 4, 0x0 +NORM },	/* 94 */
{     0,    96, 6*16+ 5, 0x10+NORM },	/* 95 and extend */
{     0,    97, 5*16+ 3, 0x0 +NORM },	/* 96 */
{     0,    98, 4*16+ 3, 0x10+NORM },	/* 97 */
{     0,    99,10*16+ 3, 0x0 +NORM },	/* 98 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 99 */
{    95,   101, 5*16+ 3, 0x10+NORM },	/* 100 and block */
{     0,   102, 6*16+ 5, 0x0 +NORM },	/* 101 */
{     0,  1178, 5*16+ 4, 0x10+NORM },	/* 102 */
{     0,  1181, 6*16+ 2, 0x0 +NORM },	/* 103 and descend*/
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 104 */
{     0,     0, 4*16+ 5, 0x10+SHIM },	/* 105  make a shimari here are all the josekis from 38 basic josekis.  first 3-3*/
{   105,   811, 4*16+ 6, 0x10+SHIM },	/* 106 */
{   106,   112, 4*16+ 4, 0x0 +NORM },	/* 107 kakari */
{     0,   109, 4*16+ 3, 0x10+URGT },	/* 108 */
{     0,   115, 5*16+ 4, 0x0 +URGT },	/* 109 */
{     0,   111, 2*16+ 5, 0x10+NORM },	/* 110 */
{     0,   114, 4*16+ 7, 0x0 +NORM },	/* 111 */
{   108,     0, 3*16+ 4, 0x0 +NORM },	/* 112 block after tenuki */
{     0,     0, 5*16+ 3, 0x0 +NORM },	/* 113 */
{   113,     0, 6*16+ 2, 0x10+NORM },	/* 114 */
{   110,   116, 3*16+ 4, 0x10+NORM },	/* 115 another variation */
{     0,   117, 4*16+ 6, 0x0 +NORM },	/* 116 */
{     0,   118, 3*16+ 5, 0x10+NORM },	/* 117 */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 118 */
{   107,   132, 4*16+ 6, 0x0 +NORM },	/* 119 kakari from side */
{     0,   121, 6*16+ 3, 0x10+NORM },	/* 120 */
{     0,     0, 4*16+ 9, 0x0 +NORM },	/* 121 */
{   120,     0, 6*16+ 4, 0x10+NORM },	/* 122 */
{   122,     0, 5*16+ 4, 0x10+NORM },	/* 123 */
{   119,   125, 4*16+ 5, 0x0 +NORM },	/* 124 another kakari from side */
{     0,   126, 5*16+ 4, 0x10+NORM },	/* 125 */
{     0,   127, 5*16+ 5, 0x0 +NORM },	/* 126 */
{     0,   128, 6*16+ 4, 0x10+NORM },	/* 127 */
{     0,   129, 3*16+ 4, 0x0 +NORM },	/* 128 */
{     0,   130, 2*16+ 3, 0x10+NORM },	/* 129 */
{     0,   131, 6*16+ 5, 0x0 +NORM },	/* 130 */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 131 */
{   123,     0, 5*16+ 4, 0x0 +NORM },	/* 132 */
{     0,   142, 4*16+ 7, 0x0 +NORM },	/* 133 complete basic joseki 3 Chapter 2  section 3  3-4 high kakari */
{     0,   135, 6*16+ 3, 0x10+NORM },	/* 134 */
{     0,   153, 3*16+11, 0x0 +NORM },	/* 135 */
{   133,   140, 4*16+ 6, 0x0 +NORM },	/* 136 solid connection */
{     0,   139, 6*16+ 3, 0x10+NORM },	/* 137 */
{     0,     0, 4*16+10, 0x0 +NORM },	/* 138 */
{   138,     0, 3*16+10, 0x0 +NORM },	/* 139 */
{   137,   141, 5*16+ 4, 0x10+NORM },	/* 140 */
{     0,     0, 4*16+10, 0x0 +NORM },	/* 141 */
{   134,   143, 5*16+ 6, 0x10+NORM },	/* 142  peep */
{     0,   144, 4*16+ 6, 0x0 +NORM },	/* 143 */
{     0,     0, 5*16+ 4, 0x10+URGT },	/* 144 */
{    25,   146, 4*16+10, 0x10+BAD  },	/* 145 chinese style */
{     0,   147, 4*16+ 5, 0x0 +NORM },	/* 146 */
{     0,   148, 3*16+ 5, 0x10+NORM },	/* 147 */
{     0,   149, 3*16+ 6, 0x0 +NORM },	/* 148 */
{     0,   150, 3*16+ 4, 0x10+NORM },	/* 149 */
{     0,   151, 6*16+ 6, 0x0 +NORM },	/* 150 */
{     0,   152, 6*16+ 4, 0x10+NORM },	/* 151 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 152 */
{     0,   158, 3*16+ 8, 0x10+COMP },	/* 153 invasion */
{     0,   155, 4*16+ 8, 0x0 +BAD  },	/* 154 */
{     0,   156, 3*16+ 9, 0x10+NORM },	/* 155 */
{     0,   157, 4*16+ 9, 0x0 +NORM },	/* 156 */
{     0,     0, 2*16+ 6, 0x10+NORM },	/* 157 */
{   154,   159, 2*16+ 7, 0x0 +URGT },	/* 158 */
{     0,   164, 3*16+10, 0x10+COMP },	/* 159 */
{     0,   161, 2*16+10, 0x0 +NORM },	/* 160 */
{     0,   162, 2*16+ 9, 0x10+NORM },	/* 161 */
{     0,   163, 4*16+10, 0x0 +NORM },	/* 162 */
{     0,     0, 4*16+ 9, 0x10+NORM },	/* 163 */
{   160,   165, 4*16+11, 0x0 +NORM },	/* 164 */
{     0,   166, 5*16+ 9, 0x10+NORM },	/* 165 */
{     0,   167, 5*16+ 4, 0x0 +NORM },	/* 166 */
{     0,   168, 5*16+ 3, 0x10+NORM },	/* 167 */
{     0,     0, 7*16+ 6, 0x0 +NORM },	/* 168 */
{    16,   170, 4*16+ 4, 0x0 +COMP },	/* 169 section 4 avalache joseki */
{     0,   171, 3*16+ 4, 0x10+URGT },	/* 170 */
{     0,   227, 5*16+ 3, 0x0 +URGT },	/* 171 */
{     0,   173, 3*16+ 3, 0x10+NORM },	/* 172 simple variation */
{     0,   174, 4*16+ 6, 0x0 +NORM },	/* 173 */
{     0,   176, 3*16+ 7, 0x10+NORM },	/* 174 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 175 */
{   175,   178, 5*16+ 8, 0x0 +NORM },	/* 176 */
{     0,     0, 3*16+ 8, 0x10+NORM },	/* 177 */
{   177,     0, 3*16+ 8, 0x0 +NORM },	/* 178 */
{   172,   180, 4*16+ 6, 0x10+NORM },	/* 179 small avalanche */
{     0,   181, 3*16+ 3, 0x0 +NORM },	/* 180 */
{     0,   182, 4*16+ 2, 0x10+NORM },	/* 181 */
{     0,   183, 2*16+ 3, 0x0 +NORM },	/* 182 */
{     0,   207, 5*16+ 2, 0x10+NORM },	/* 183 */
{     0,   199, 6*16+ 4, 0x0 +NORM },	/* 184 */
{     0,   186, 6*16+ 5, 0x10+NORM },	/* 185 */
{     0,   187, 7*16+ 4, 0x0 +NORM },	/* 186 */
{     0,   188, 5*16+ 6, 0x10+NORM },	/* 187 */
{     0,   189, 5*16+ 4, 0x0 +NORM },	/* 188 */
{     0,   190, 2*16+ 4, 0x10+NORM },	/* 189 */
{     0,   191, 3*16+ 6, 0x0 +NORM },	/* 190 */
{     0,   192, 2*16+ 6, 0x10+NORM },	/* 191 */
{     0,   193, 3*16+ 7, 0x0 +NORM },	/* 192 */
{     0,   194, 2*16+ 2, 0x10+NORM },	/* 193 */
{     0,   195, 1*16+ 4, 0x0 +NORM },	/* 194 */
{     0,   196, 2*16+ 5, 0x10+NORM },	/* 195 */
{     0,   197, 2*16+ 7, 0x0 +NORM },	/* 196 */
{     0,   198, 3*16+ 2, 0x10+NORM },	/* 197 */
{     0,     0, 3*16+10, 0x0 +NORM },	/* 198 */
{   185,   200, 6*16+ 3, 0x10+NORM },	/* 199 another small avalanche variation */
{     0,   201, 3*16+ 6, 0x0 +NORM },	/* 200 */
{     0,   202, 5*16+ 4, 0x10+NORM },	/* 201 */
{     0,   203, 5*16+ 5, 0x0 +NORM },	/* 202 */
{     0,   204, 6*16+ 5, 0x10+NORM },	/* 203 */
{     0,   205, 5*16+ 6, 0x0 +NORM },	/* 204 */
{     0,   206, 7*16+ 4, 0x10+NORM },	/* 205 */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 206 */
{   184,   215, 6*16+ 3, 0x0 +COMP },	/* 207 another small avalanche variation */
{     0,   209, 6*16+ 2, 0x10+NORM },	/* 208 */
{     0,   210, 3*16+ 6, 0x0 +NORM },	/* 209 */
{     0,   211, 5*16+ 5, 0x10+NORM },	/* 210 */
{     0,   212, 5*16+ 4, 0x0 +NORM },	/* 211 */
{     0,   213, 7*16+ 3, 0x10+NORM },	/* 212 */
{     0,   214, 6*16+ 4, 0x0 +NORM },	/* 213 */
{     0,     0, 6*16+ 5, 0x10+NORM },	/* 214 */
{   208,   216, 5*16+ 6, 0x10+NORM },	/* 215 */
{     0,   217, 6*16+ 5, 0x0 +NORM },	/* 216 */
{     0,   218, 2*16+ 4, 0x10+NORM },	/* 217 */
{     0,   219, 3*16+ 6, 0x0 +NORM },	/* 218 */
{     0,   220, 2*16+ 6, 0x10+NORM },	/* 219 */
{     0,   221, 3*16+ 7, 0x0 +NORM },	/* 220 */
{     0,   222, 2*16+ 2, 0x10+NORM },	/* 221 */
{     0,     0, 6*16+ 6, 0x0 +NORM },	/* 222 */
{   179,   224, 3*16+ 6, 0x10+NORM },	/* 223  Big avalanche joseki */
{     0,   225, 4*16+ 2, 0x0 +NORM },	/* 224 */
{     0,   226, 3*16+ 3, 0x10+NORM },	/* 225 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 226 */
{   223,   228, 5*16+ 2, 0x10+NORM },	/* 227 double hane variation */
{     0,   229, 4*16+ 2, 0x0 +NORM },	/* 228 */
{     0,   230, 3*16+ 3, 0x10+NORM },	/* 229 */
{     0,   231, 6*16+ 2, 0x0 +NORM },	/* 230 */
{     0,   232, 4*16+ 6, 0x10+NORM },	/* 231 */
{     0,   234, 6*16+ 4, 0x0 +NORM },	/* 232 */
{     0,     0, 5*16+ 6, 0x10+NORM },	/* 233 */
{   233,     0, 3*16+ 2, 0x10+NORM },	/* 234 */
{    81,   239, 5*16+ 5, 0x10+NORM },	/* 235 section 5 3-4 high kakari, outside tsuke*/
{     0,   237, 5*16+ 6, 0x0 +NORM },	/* 236 */
{     0,   238, 5*16+ 4, 0x10+NORM },	/* 237 */
{     0,   256, 4*16+ 7, 0x0 +NORM },	/* 238 */
{   236,     0, 3*16+ 3, 0x0 +COMP },	/* 239 Start of complicated var, rest not here */
{     0,   241, 3*16+ 5, 0x10+NORM },	/* 240 */
{     0,   242, 3*16+ 6, 0x0 +NORM },	/* 241 */
{     0,   243, 2*16+ 5, 0x10+NORM },	/* 242 */
{     0,   244, 7*16+ 6, 0x0 +NORM },	/* 243 */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 244 */
{   240,   246, 6*16+ 6, 0x10+NORM },	/* 245 second variation */
{     0,   248, 6*16+ 7, 0x0 +NORM },	/* 246 */
{     0,     0, 7*16+ 6, 0x10+NORM },	/* 247 */
{   247,   249, 7*16+ 7, 0x10+NORM },	/* 248 */
{     0,   250, 7*16+ 6, 0x0 +NORM },	/* 249 */
{     0,   251, 6*16+ 5, 0x10+NORM },	/* 250 */
{     0,   252, 6*16+ 8, 0x0 +NORM },	/* 251 */
{     0,   253, 7*16+ 8, 0x10+NORM },	/* 252 */
{     0,   254, 7*16+ 9, 0x0 +NORM },	/* 253 */
{     0,   255, 8*16+ 6, 0x10+NORM },	/* 254 */
{     0,     0, 6*16+10, 0x10+NORM },	/* 255 */
{   245,   257, 3*16+ 3, 0x0 +NORM },	/* 256 take corner after tenuki */
{     0,   258, 3*16+ 4, 0x10+NORM },	/* 257 */
{     0,   259, 4*16+ 4, 0x0 +NORM },	/* 258 */
{     0,   260, 3*16+ 5, 0x10+NORM },	/* 259 */
{     0,   261, 4*16+ 6, 0x0 +NORM },	/* 260 */
{     0,   262, 2*16+ 3, 0x10+NORM },	/* 261 */
{     0,   263, 5*16+ 3, 0x0 +NORM },	/* 262 */
{     0,   267, 3*16+ 2, 0x10+NORM },	/* 263 */
{     0,   265, 4*16+ 2, 0x0 +NORM },	/* 264 */
{     0,   266, 3*16+ 3, 0x10+NORM },	/* 265 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 266 */
{   264,   268, 6*16+ 5, 0x0 +NORM },	/* 267 */
{     0,   269, 6*16+ 4, 0x10+NORM },	/* 268 */
{     0,   270, 7*16+ 4, 0x0 +NORM },	/* 269 */
{     0,   271, 6*16+ 3, 0x10+NORM },	/* 270 */
{     0,   272, 7*16+ 3, 0x0 +NORM },	/* 271 */
{     0,   273, 5*16+ 2, 0x10+NORM },	/* 272 */
{     0,   274, 6*16+ 2, 0x0 +NORM },	/* 273 */
{     0,   275, 5*16+ 3, 0x10+NORM },	/* 274 */
{     0,     0, 7*16+ 5, 0x0 +NORM },	/* 275 */
{     0,  1940, 3*16+ 9, 0x0 +NORM },	/* 276 section 7  3-4 1 ptr low kakari, kosumi */
{   276,  1919, 4*16+ 7, 0x0 +NORM },	/* 277 */
{   277,  2121, 4*16+ 6, 0x10+NORM },	/* 278 */
{     0,  2113, 3*16+ 7, 0x0 +NORM },	/* 279 */
{   145,   680, 3*16+ 6, 0x0 +BAD  },	/* 280 section 8 3-4 large knights kakari */
{     0,   282, 3*16+ 4, 0x10+NORM },	/* 281 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 282 */
{     0,   287, 6*16+ 5, 0x0 +NORM },	/* 283 chapter 3 squeeze plays section 9  low kakari, 2 pt high pincer, 2 pt jump*/
{     0,   353, 7*16+ 3, 0x10+NORM },	/* 284 */
{   284,   286, 4*16+11, 0x10+BAD  },	/* 285 */
{     0,     0, 6*16+ 3, 0x0 +NORM },	/* 286 */
{   285,   288, 5*16+ 5, 0x10+COMP },	/* 287 intercept immediately */
{     0,   289, 5*16+ 6, 0x0 +NORM },	/* 288 */
{     0,   291, 5*16+ 4, 0x10+NORM },	/* 289 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 290 */
{   290,   292, 3*16+ 7, 0x0 +NORM },	/* 291 */
{     0,   293, 6*16+ 6, 0x10+NORM },	/* 292 */
{     0,   294, 5*16+ 7, 0x0 +NORM },	/* 293 */
{     0,   295, 7*16+ 5, 0x10+NORM },	/* 294 */
{     0,   296, 6*16+ 7, 0x0 +NORM },	/* 295 */
{     0,     0, 7*16+ 6, 0x10+NORM },	/* 296 */
{     0,   307, 4*16+11, 0x0 +NORM },	/* 297 attack from other side */
{   297,   307, 4*16+10, 0x0 +NORM },	/* 298 */
{   298,   307, 3*16+10, 0x0 +NORM },	/* 299 */
{     0,   301, 6*16+ 7, 0x10+NORM },	/* 300 */
{     0,   302, 8*16+ 5, 0x0 +BAD  },	/* 301 */
{     0,   303, 4*16+ 5, 0x10+NORM },	/* 302 */
{     0,   304, 4*16+ 6, 0x0 +NORM },	/* 303 */
{     0,   305, 5*16+ 5, 0x10+NORM },	/* 304 */
{     0,   306, 5*16+ 6, 0x0 +NORM },	/* 305 */
{     0,     0, 6*16+ 6, 0x10+NORM },	/* 306 */
{   300,   308, 5*16+ 4, 0x0 +NORM },	/* 307 standard defence from attack */
{     0,   309, 5*16+ 3, 0x10+NORM },	/* 308 */
{     0,   310, 8*16+ 4, 0x0 +NORM },	/* 309 */
{     0,   311, 7*16+ 4, 0x10+NORM },	/* 310 */
{     0,   312, 7*16+ 5, 0x0 +NORM },	/* 311 */
{     0,   313, 8*16+ 5, 0x10+NORM },	/* 312 */
{     0,   314, 8*16+ 6, 0x0 +NORM },	/* 313 */
{     0,   315, 9*16+ 5, 0x10+NORM },	/* 314 */
{     0,     0, 7*16+ 7, 0x0 +NORM },	/* 315 */
{   299,   328, 8*16+ 4, 0x0 +NORM },	/* 316  push before attacking */
{     0,   318, 8*16+ 3, 0x10+NORM },	/* 317 */
{     0,   319, 9*16+ 4, 0x0 +NORM },	/* 318 */
{     0,   320, 9*16+ 3, 0x10+NORM },	/* 319 */
{     0,   327,10*16+ 4, 0x0 +NORM },	/* 320 */
{     0,   322,11*16+ 3, 0x10+NORM },	/* 321 */
{     0,   323, 5*16+ 4, 0x0 +NORM },	/* 322 */
{     0,   326, 5*16+ 3, 0x10+NORM },	/* 323 */
{     0,     0, 4*16+11, 0x0 +NORM },	/* 324 */
{   324,     0, 4*16+10, 0x0 +NORM },	/* 325 */
{   325,     0, 3*16+10, 0x10+NORM },	/* 326 */
{   321,     0, 4*16+11, 0x10+BAD  },	/* 327 */
{   317,   329, 7*16+ 4, 0x10+COMP },	/* 328 push thru and cut */
{     0,   330, 7*16+ 5, 0x0 +NORM },	/* 329 */
{     0,   331, 8*16+ 5, 0x10+NORM },	/* 330 */
{     0,   338,10*16+ 4, 0x0 +NORM },	/* 331 */
{     0,   333, 8*16+ 6, 0x10+NORM },	/* 332 */
{     0,   334, 8*16+ 3, 0x0 +URGT },	/* 333 */
{     0,   335, 8*16+ 2, 0x10+NORM },	/* 334 */
{     0,   336, 9*16+ 2, 0x0 +NORM },	/* 335 */
{     0,   337, 7*16+ 2, 0x10+NORM },	/* 336 */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 337 */
{   332,     0, 9*16+ 4, 0x10+BAD  },	/* 338 */
{   316,   340, 3*16+ 3, 0x0 +NORM },	/* 339  live in corner */
{     0,   341, 3*16+ 2, 0x10+NORM },	/* 340 */
{     0,   346, 2*16+ 3, 0x0 +NORM },	/* 341 */
{     0,   343, 2*16+ 2, 0x10+NORM },	/* 342 */
{     0,   344, 4*16+ 4, 0x0 +NORM },	/* 343 */
{     0,   345, 5*16+ 3, 0x10+NORM },	/* 344 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 345 */
{   342,   347, 5*16+ 5, 0x10+NORM },	/* 346 variation */
{     0,   348, 4*16+ 2, 0x0 +NORM },	/* 347 */
{     0,   349, 5*16+ 3, 0x10+NORM },	/* 348 */
{     0,   350, 5*16+ 2, 0x0 +NORM },	/* 349 */
{     0,   351, 6*16+ 2, 0x10+NORM },	/* 350 */
{     0,   352, 2*16+ 2, 0x0 +NORM },	/* 351 */
{     0,     0, 5*16+ 6, 0x10+NORM },	/* 352 */
{   339,   354, 3*16+ 4, 0x10+NORM },	/* 353 after tenuki, how to attack */
{     0,   355, 4*16+ 5, 0x0 +NORM },	/* 354 */
{     0,   356, 5*16+ 4, 0x10+NORM },	/* 355 */
{     0,   357, 5*16+ 5, 0x0 +NORM },	/* 356 */
{     0,   358, 8*16+ 4, 0x10+NORM },	/* 357 */
{     0,     0, 8*16+ 7, 0x0 +COMP },	/* 358 */
{   283,   375, 5*16+ 4, 0x0 +COMP },	/* 359 section 10 2 pt high pincer, keima */
{     0,   361, 5*16+ 3, 0x10+NORM },	/* 360 */
{     0,   370, 6*16+ 4, 0x0 +NORM },	/* 361 */
{     0,   363, 7*16+ 3, 0x10+NORM },	/* 362 */
{     0,   364, 4*16+11, 0x0 +NORM },	/* 363 */
{     0,   365, 5*16+ 6, 0x10+URGT },	/* 364 */
{     0,   366, 4*16+ 4, 0x0 +NORM },	/* 365 */
{     0,   367, 3*16+ 3, 0x10+NORM },	/* 366 */
{     0,   368, 7*16+ 4, 0x0 +NORM },	/* 367 */
{     0,   369, 8*16+ 3, 0x10+NORM },	/* 368 */
{     0,     0, 7*16+ 6, 0x0 +NORM },	/* 369 */
{   362,   371, 6*16+ 3, 0x10+NORM },	/* 370 variation */
{     0,   372, 7*16+ 4, 0x0 +NORM },	/* 371 */
{     0,   373, 3*16+12, 0x10+NORM },	/* 372 */
{     0,   374, 7*16+ 3, 0x0 +NORM },	/* 373 */
{     0,     0, 2*16+ 4, 0x10+NORM },	/* 374 */
{   360,   376, 4*16+ 4, 0x10+NORM },	/* 375 push thru and cut */
{     0,   377, 4*16+ 5, 0x0 +NORM },	/* 376 */
{     0,   378, 5*16+ 5, 0x10+NORM },	/* 377 */
{     0,   386, 4*16+ 7, 0x0 +NORM },	/* 378 */
{     0,   380, 5*16+ 6, 0x10+NORM },	/* 379 */
{     0,   381, 5*16+ 7, 0x0 +NORM },	/* 380 */
{     0,   382, 6*16+ 4, 0x10+NORM },	/* 381 */
{     0,     0, 4*16+10, 0x0 +NORM },	/* 382 */
{   379,   384, 6*16+ 5, 0x10+NORM },	/* 383 variation */
{     0,   385, 5*16+ 7, 0x0 +NORM },	/* 384 */
{     0,     0, 7*16+ 3, 0x10+NORM },	/* 385 */
{   383,   387, 5*16+ 8, 0x10+BAD  },	/* 386 */
{     0,   388, 6*16+ 5, 0x0 +NORM },	/* 387 */
{     0,   389, 5*16+ 6, 0x10+NORM },	/* 388 */
{     0,     0, 5*16+ 7, 0x0 +NORM },	/* 389 */
{   359,   394, 4*16+ 6, 0x0 +NORM },	/* 390 section 11 2 pt high pincer, kosumi */
{     0,   392, 6*16+ 4, 0x10+NORM },	/* 391 */
{     0,   393, 2*16+ 3, 0x0 +NORM },	/* 392 */
{     0,     0,10*16+ 3, 0x10+URGT },	/* 393 */
{   391,   395, 3*16+ 4, 0x10+NORM },	/* 394  variation */
{     0,   396, 2*16+ 4, 0x0 +NORM },	/* 395 */
{     0,   397, 2*16+ 3, 0x10+NORM },	/* 396 */
{     0,   398, 4*16+ 4, 0x0 +NORM },	/* 397 */
{     0,   399, 3*16+ 3, 0x10+NORM },	/* 398 */
{     0,   400, 5*16+ 4, 0x0 +NORM },	/* 399 */
{     0,     0, 6*16+ 3, 0x10+NORM },	/* 400 */
{   390,   418, 4*16+ 4, 0x0 +NORM },	/* 401 section 12 2 pt high pincer, diagonal tsuke */
{     0,   403, 5*16+ 3, 0x10+NORM },	/* 402 */
{     0,   404, 3*16+ 3, 0x0 +NORM },	/* 403 */
{     0,   405, 3*16+ 2, 0x10+NORM },	/* 404 */
{     0,   407, 2*16+ 3, 0x0 +NORM },	/* 405 */
{     0,   416, 7*16+ 4, 0x10+NORM },	/* 406 */
{   406,   408, 8*16+ 3, 0x10+NORM },	/* 407 */
{     0,   409, 5*16+ 4, 0x0 +NORM },	/* 408 */
{     0,   410, 6*16+ 4, 0x10+NORM },	/* 409 */
{     0,   411, 6*16+ 5, 0x0 +NORM },	/* 410 */
{     0,   412, 7*16+ 4, 0x10+NORM },	/* 411 */
{     0,     0, 4*16+10, 0x0 +NORM },	/* 412 */
{     0,   415, 2*16+ 2, 0x10+NORM },	/* 413 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 414 */
{   414,     0, 2*16+ 8, 0x0 +NORM },	/* 415 */
{   413,   417, 3*16+ 7, 0x10+NORM },	/* 416 */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 417 */
{   402,   423, 3*16+ 3, 0x10+COMP },	/* 418 answer tsuke at 3-3 messy*/
{     0,   420, 6*16+ 4, 0x0 +NORM },	/* 419 */
{     0,   421, 5*16+ 3, 0x10+NORM },	/* 420 */
{     0,   422, 6*16+ 6, 0x0 +NORM },	/* 421 */
{     0,     0, 7*16+ 3, 0x10+NORM },	/* 422 */
{   419,   424, 5*16+ 3, 0x0 +COMP },	/* 423 variation */
{     0,   425, 5*16+ 4, 0x10+NORM },	/* 424 */
{     0,   428, 5*16+ 2, 0x0 +NORM },	/* 425 */
{     0,   429, 4*16+ 6, 0x10+NORM },	/* 426 */
{   426,     0, 5*16+ 5, 0x10+NORM },	/* 427 */
{   427,     0, 6*16+ 4, 0x10+NORM },	/* 428 */
{     0,   430, 2*16+ 3, 0x0 +NORM },	/* 429 */
{     0,   431, 3*16+ 4, 0x10+NORM },	/* 430 */
{     0,   432, 4*16+ 5, 0x0 +NORM },	/* 431 */
{     0,   433, 2*16+ 4, 0x10+NORM },	/* 432 */
{     0,     0, 6*16+ 5, 0x0 +NORM },	/* 433 there's more... */
{   401,   436, 3*16+ 4, 0x10+NORM },	/* 434 section 13 2 pt high pincer, tenuki */
{   434,   446, 4*16+ 5, 0x10+NORM },	/* 435 */
{     0,   437, 4*16+ 5, 0x0 +NORM },	/* 436 */
{     0,   438, 6*16+ 4, 0x10+NORM },	/* 437 */
{     0,   439, 5*16+ 6, 0x0 +NORM },	/* 438 */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 439 */
{     0,   441, 3*16+ 4, 0x0 +NORM },	/* 440 */
{     0,   442, 4*16+ 4, 0x10+NORM },	/* 441 */
{     0,   443, 3*16+ 3, 0x0 +NORM },	/* 442 */
{     0,   444, 3*16+ 6, 0x10+NORM },	/* 443 */
{     0,   445, 2*16+ 6, 0x0 +NORM },	/* 444 */
{     0,     0, 2*16+ 7, 0x10+NORM },	/* 445 */
{   440,   455, 4*16+ 4, 0x0 +COMP },	/* 446 wedge variation */
{     0,   448, 5*16+ 4, 0x10+NORM },	/* 447 */
{     0,   449, 3*16+ 4, 0x0 +NORM },	/* 448 */
{     0,   450, 3*16+ 3, 0x10+NORM },	/* 449 */
{     0,   451, 4*16+ 7, 0x0 +URGT },	/* 450 */
{     0,   452, 5*16+ 8, 0x10+NORM },	/* 451 */
{     0,   453, 5*16+ 5, 0x0 +NORM },	/* 452 */
{     0,   454, 6*16+ 4, 0x10+NORM },	/* 453 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 454 */
{   447,   456, 3*16+ 4, 0x10+NORM },	/* 455 atari underneath */
{     0,   457, 5*16+ 4, 0x0 +NORM },	/* 456 */
{     0,   458, 3*16+ 3, 0x10+NORM },	/* 457 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 458 */
{    39,   460, 3*16+ 9, 0x0 +COMP },	/* 459 section 14 1 pt low pincer */
{     0,     0, 4*16+ 8, 0x10+NORM },	/* 460 */
{    38,   462, 7*16+ 3, 0x10+NORM },	/* 461 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 462 */
{     0,  2580, 3*16+ 4, 0x10+NORM },	/* 463 */
{     0,  2690, 2*16+ 5, 0x0 +NORM },	/* 464 */
{     0,   466, 3*16+ 8, 0x10+NORM },	/* 465 */
{     0,   467, 4*16+ 9, 0x0 +NORM },	/* 466 */
{     0,   468, 4*16+ 7, 0x10+NORM },	/* 467 */
{     0,   469, 5*16+ 7, 0x0 +NORM },	/* 468 */
{     0,   470, 3*16+ 9, 0x10+NORM },	/* 469 */
{     0,  2595, 4*16+10, 0x0 +NORM },	/* 470 */
{   463,   472, 4*16+ 7, 0x10+COMP },	/* 471 variation */
{     0,   473, 5*16+ 7, 0x0 +NORM },	/* 472 */
{     0,  2671, 5*16+ 8, 0x10+NORM },	/* 473 */
{     0,  2682, 3*16+ 8, 0x0 +NORM },	/* 474 */
{   459,   476, 8*16+ 4, 0x0 +COMP },	/* 475 counter pincer */
{     0,   477, 6*16+ 5, 0x10+NORM },	/* 476 */
{     0,   478, 4*16+ 7, 0x0 +NORM },	/* 477 */
{     0,   479, 3*16+ 8, 0x10+NORM },	/* 478 */
{     0,  2719, 6*16+ 6, 0x0 +NORM },	/* 479 */
{     0,  2693, 7*16+ 6, 0x10+NORM },	/* 480 */
{     0,   482, 6*16+ 7, 0x0 +NORM },	/* 481 */
{     0,  2692, 8*16+ 5, 0x10+NORM },	/* 482 */
{     0,   484,10*16+ 4, 0x0 +NORM },	/* 483 */
{     0,   485, 3*16+ 4, 0x10+NORM },	/* 484 */
{     0,   486, 3*16+ 6, 0x0 +NORM },	/* 485 */
{     0,     0, 3*16+11, 0x10+NORM },	/* 486 */
{    37,   488, 4*16+ 7, 0x0 +NORM },	/* 487  tsuke on top of pincer stone*/
{     0,  2849, 4*16+ 8, 0x10+NORM },	/* 488 */
{     0,  2748, 4*16+ 6, 0x0 +NORM },	/* 489 */
{     0,   491, 7*16+ 3, 0x10+NORM },	/* 490 */
{     0,   492, 3*16+ 8, 0x0 +NORM },	/* 491 */
{     0,   493, 3*16+ 9, 0x10+NORM },	/* 492 */
{     0,   494, 2*16+ 8, 0x0 +NORM },	/* 493 */
{     0,  2739, 4*16+ 9, 0x10+NORM },	/* 494 */
{     0,     0, 2*16+ 7, 0x0 +NORM },	/* 495 */
{   490,   497, 3*16+ 8, 0x10+NORM },	/* 496 variation */
{     0,   498, 7*16+ 4, 0x0 +NORM },	/* 497 */
{     0,   499, 3*16+ 4, 0x10+NORM },	/* 498 */
{     0,   500, 2*16+ 4, 0x0 +NORM },	/* 499 */
{     0,   501, 2*16+ 3, 0x10+NORM },	/* 500 */
{     0,   502, 4*16+ 4, 0x0 +NORM },	/* 501 */
{     0,   503, 3*16+ 3, 0x10+NORM },	/* 502 */
{     0,   504, 5*16+ 4, 0x0 +NORM },	/* 503 */
{     0,   505, 6*16+ 2, 0x10+NORM },	/* 504 */
{     0,     0, 8*16+ 3, 0x0 +NORM },	/* 505 */
{     0,   557, 3*16+ 3, 0x0 +NORM },	/* 506 section 15 1 pt high pincer */
{   506,     0, 4*16+ 4, 0x0 +NORM },	/* 507 */
{   507,   511, 5*16+ 5, 0x0 +NORM },	/* 508 */
{   508,   528, 5*16+ 6, 0x0 +NORM },	/* 509 */
{   509,   549, 3*16+ 7, 0x0 +NORM },	/* 510 */
{     0,   512, 6*16+ 4, 0x10+NORM },	/* 511 one point jump */
{     0,   513, 4*16+ 9, 0x0 +NORM },	/* 512 */
{     0,   514, 6*16+ 7, 0x10+NORM },	/* 513 */
{     0,   515, 3*16+ 7, 0x0 +NORM },	/* 514 */
{     0,   516, 4*16+ 6, 0x10+NORM },	/* 515 */
{     0,   517, 3*16+ 6, 0x0 +NORM },	/* 516 */
{     0,   518, 4*16+ 5, 0x10+NORM },	/* 517 */
{     0,   520, 3*16+ 4, 0x0 +NORM },	/* 518 */
{     0,     0, 4*16+ 4, 0x10+NORM },	/* 519 */
{   519,   521, 3*16+ 3, 0x10+NORM },	/* 520 variation */
{     0,   522, 4*16+ 4, 0x0 +NORM },	/* 521 */
{     0,   523, 5*16+ 4, 0x10+NORM },	/* 522 */
{     0,   524, 4*16+ 8, 0x0 +NORM },	/* 523 */
{     0,     0, 6*16+ 5, 0x10+NORM },	/* 524 */
{     0,     0, 7*16+ 3, 0x10+NORM },	/* 525 keima */
{   525,   527, 4*16+ 6, 0x10+BAD  },	/* 526 */
{     0,     0, 5*16+ 4, 0x0 +NORM },	/* 527 */
{   526,   533, 6*16+ 4, 0x10+NORM },	/* 528 */
{     0,   530, 5*16+ 7, 0x0 +NORM },	/* 529 */
{     0,   531, 4*16+ 8, 0x10+NORM },	/* 530 */
{     0,   532, 4*16+ 6, 0x0 +NORM },	/* 531 */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 532 */
{   529,   538, 3*16+ 3, 0x0 +NORM },	/* 533 */
{     0,   535, 3*16+ 2, 0x10+NORM },	/* 534 */
{     0,   536, 4*16+ 4, 0x0 +NORM },	/* 535 */
{     0,   537, 5*16+ 3, 0x10+NORM },	/* 536 */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 537 */
{   534,   539, 4*16+ 4, 0x10+NORM },	/* 538 variation*/
{     0,   540, 3*16+ 4, 0x0 +NORM },	/* 539 */
{     0,   541, 4*16+ 6, 0x10+NORM },	/* 540 */
{     0,   542, 4*16+ 5, 0x0 +NORM },	/* 541 */
{     0,   543, 5*16+ 5, 0x10+NORM },	/* 542 */
{     0,   544, 6*16+ 6, 0x0 +NORM },	/* 543 */
{     0,   545, 2*16+ 6, 0x10+NORM },	/* 544 */
{     0,   546, 3*16+ 6, 0x0 +NORM },	/* 545 */
{     0,   547, 3*16+ 7, 0x10+NORM },	/* 546 */
{     0,   548, 2*16+ 5, 0x0 +NORM },	/* 547 */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 548 */
{     0,   550, 4*16+ 6, 0x10+NORM },	/* 549 tsuke after pincer */
{     0,   551, 3*16+ 6, 0x0 +NORM },	/* 550 */
{     0,   552, 4*16+ 5, 0x10+NORM },	/* 551 */
{     0,   553, 3*16+ 4, 0x0 +NORM },	/* 552 */
{     0,   554, 4*16+ 4, 0x10+NORM },	/* 553 */
{     0,   555, 4*16+ 8, 0x0 +NORM },	/* 554 */
{     0,   556, 3*16+ 3, 0x10+NORM },	/* 555 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 556 */
{     0,   558, 4*16+ 4, 0x10+NORM },	/* 557 pincer, then tsuke at 3-3 */
{     0,   559, 3*16+ 4, 0x0 +NORM },	/* 558 */
{     0,   560, 4*16+ 5, 0x10+NORM },	/* 559 */
{     0,   561, 3*16+ 6, 0x0 +NORM },	/* 560 */
{     0,   562, 4*16+ 6, 0x10+NORM },	/* 561 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 562 */
{     0,   564, 6*16+ 5, 0x0 +NORM },	/* 563 two point jump section 16 3 point low pincer */
{     0,   565, 7*16+ 3, 0x10+NORM },	/* 564 */
{     0,     0, 3*16+11, 0x0 +NORM },	/* 565 */
{   563,   567, 4*16+ 4, 0x0 +NORM },	/* 566 diagonal tsuke */
{     0,   568, 5*16+ 3, 0x10+NORM },	/* 567 */
{     0,   569, 3*16+ 3, 0x0 +NORM },	/* 568 */
{     0,   570, 3*16+ 2, 0x10+NORM },	/* 569 */
{     0,   571, 2*16+ 3, 0x0 +NORM },	/* 570 */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 571 */
{   566,   573, 6*16+ 4, 0x10+NORM },	/* 572 tenuki */
{     0,   574, 3*16+ 3, 0x0 +NORM },	/* 573 */
{     0,   575, 3*16+ 2, 0x10+NORM },	/* 574 */
{     0,   578, 4*16+ 4, 0x0 +NORM },	/* 575 */
{     0,   577, 5*16+ 4, 0x10+NORM },	/* 576 */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 577 */
{   576,   579, 2*16+ 3, 0x10+NORM },	/* 578 variation */
{     0,   580, 5*16+ 3, 0x0 +NORM },	/* 579 */
{     0,   581, 3*16+ 4, 0x10+NORM },	/* 580 */
{     0,   582, 4*16+ 2, 0x0 +NORM },	/* 581 */
{     0,   583, 3*16+ 3, 0x10+NORM },	/* 582 */
{     0,   584, 5*16+ 4, 0x0 +NORM },	/* 583 */
{     0,   585, 4*16+ 5, 0x10+NORM },	/* 584 */
{     0,   586, 5*16+ 5, 0x0 +NORM },	/* 585 */
{     0,   587, 4*16+ 6, 0x10+NORM },	/* 586 */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 587 */
{   572,   594, 5*16+ 4, 0x10+NORM },	/* 588 diagonal move after tenuki*/
{     0,   590, 4*16+ 8, 0x0 +NORM },	/* 589 */
{     0,   591, 4*16+ 9, 0x10+NORM },	/* 590 */
{     0,   592, 5*16+ 8, 0x0 +NORM },	/* 591 */
{     0,   593, 3*16+ 8, 0x10+NORM },	/* 592 */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 593 */
{   589,   595, 2*16+ 3, 0x0 +NORM },	/* 594 variation */
{     0,   596, 4*16+ 6, 0x10+NORM },	/* 595 */
{     0,   597, 3*16+ 6, 0x0 +NORM },	/* 596 */
{     0,   598, 4*16+ 7, 0x10+NORM },	/* 597 */
{     0,   599, 3*16+ 7, 0x0 +NORM },	/* 598 */
{     0,   600, 4*16+ 8, 0x10+NORM },	/* 599 */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 600 */
{   588,   602, 3*16+ 4, 0x10+NORM },	/* 601 diagonal tsuke after tenuki */
{     0,   603, 4*16+ 5, 0x0 +NORM },	/* 602 */
{     0,   604, 6*16+ 4, 0x10+NORM },	/* 603 */
{     0,   605, 5*16+ 9, 0x0 +NORM },	/* 604 */
{     0,   606, 4*16+11, 0x10+NORM },	/* 605 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 606 */
{   235,   608, 3*16+ 7, 0x10+NORM },	/* 607 section 17 1 pt high kakari, 1 pt low pincer */
{     0,   626, 3*16+ 3, 0x0 +NORM },	/* 608  running fight */
{     0,   610, 3*16+ 2, 0x10+NORM },	/* 609 */
{     0,   611, 3*16+ 4, 0x0 +NORM },	/* 610 */
{     0,   612, 5*16+ 2, 0x10+NORM },	/* 611 */
{     0,   619, 5*16+ 6, 0x0 +NORM },	/* 612 */
{     0,   614, 7*16+ 4, 0x10+NORM },	/* 613 */
{     0,   615, 4*16+ 8, 0x0 +NORM },	/* 614 */
{     0,   616, 3*16+ 9, 0x10+NORM },	/* 615 */
{     0,   617, 3*16+ 7, 0x0 +NORM },	/* 616 */
{     0,   618, 2*16+ 7, 0x10+NORM },	/* 617 */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 618 */
{   613,   620, 4*16+ 9, 0x10+NORM },	/* 619 variation */
{     0,   621, 6*16+ 3, 0x0 +NORM },	/* 620 */
{     0,   622, 7*16+ 3, 0x10+NORM },	/* 621 */
{     0,   623, 7*16+ 4, 0x0 +NORM },	/* 622 */
{     0,   624, 7*16+ 2, 0x10+NORM },	/* 623 */
{     0,   625, 8*16+ 4, 0x0 +NORM },	/* 624 */
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 625 */
{   609,   627, 3*16+ 4, 0x10+COMP },	/* 626 normal, staying connected variation */
{     0,   628, 4*16+ 4, 0x0 +NORM },	/* 627 */
{     0,   629, 3*16+ 5, 0x10+NORM },	/* 628 */
{     0,   650, 3*16+ 6, 0x0 +NORM },	/* 629 */
{     0,   643, 2*16+ 6, 0x10+NORM },	/* 630 atari from bottom */
{     0,   634, 5*16+ 3, 0x0 +NORM },	/* 631 */
{     0,   633, 3*16+ 6, 0x10+BAD  },	/* 632 */
{     0,     0, 4*16+ 2, 0x0 +NORM },	/* 633 */
{   632,   635, 4*16+ 2, 0x10+NORM },	/* 634 */
{     0,   636, 4*16+ 6, 0x0 +NORM },	/* 635 */
{     0,   637, 2*16+ 4, 0x10+NORM },	/* 636 */
{     0,   638, 4*16+ 7, 0x0 +NORM },	/* 637 */
{     0,   639, 3*16+ 8, 0x10+NORM },	/* 638 */
{     0,   640, 5*16+ 2, 0x0 +NORM },	/* 639 */
{     0,   642, 3*16+ 2, 0x10+NORM },	/* 640 */
{     0,     0, 4*16+ 9, 0x0 +NORM },	/* 641 */
{   641,     0,10*16+ 4, 0x0 +NORM },	/* 642 */
{   631,   644, 4*16+ 6, 0x0 +BAD  },	/* 643 just connect */
{     0,   645, 5*16+ 3, 0x10+NORM },	/* 644 */
{     0,   646, 2*16+ 7, 0x0 +NORM },	/* 645 */
{     0,   647, 2*16+ 5, 0x10+NORM },	/* 646 */
{     0,   648, 2*16+ 8, 0x0 +NORM },	/* 647 */
{     0,   649, 2*16+ 3, 0x10+NORM },	/* 648 */
{     0,     0, 4*16+ 8, 0x0 +NORM },	/* 649 */
{   630,   651, 4*16+ 6, 0x10+NORM },	/* 650 atari from above */
{     0,   652, 2*16+ 6, 0x0 +URGT },	/* 651 */
{     0,   653, 2*16+ 3, 0x10+URGT },	/* 652 */
{     0,   654, 3*16+ 2, 0x0 +URGT },	/* 653 */
{     0,   659, 2*16+ 5, 0x10+URGT },	/* 654 */
{     0,   656, 5*16+ 3, 0x0 +NORM },	/* 655 */
{     0,   657, 2*16+ 7, 0x10+NORM },	/* 656 */
{     0,   658, 5*16+ 6, 0x0 +NORM },	/* 657 */
{     0,     0, 4*16+ 7, 0x10+NORM },	/* 658 */
{   655,   663, 2*16+ 7, 0x0 +COMP },	/* 659 turn out variation */
{     0,   661, 4*16+ 7, 0x10+NORM },	/* 660 */
{     0,   662, 3*16+ 8, 0x0 +NORM },	/* 661 */
{     0,     0, 5*16+ 4, 0x10+NORM },	/* 662 */
{   660,   664, 5*16+ 3, 0x10+NORM },	/* 663 */
{     0,   665, 6*16+ 4, 0x0 +NORM },	/* 664 */
{     0,   666, 2*16+ 2, 0x10+NORM },	/* 665 */
{     0,   668, 3*16+ 8, 0x0 +NORM },	/* 666 */
{     0,     0, 5*16+ 4, 0x10+NORM },	/* 667 */
{   667,   669, 4*16+ 7, 0x10+COMP },	/* 668 */
{     0,   670, 4*16+ 8, 0x0 +NORM },	/* 669 */
{     0,   671, 6*16+ 7, 0x10+NORM },	/* 670 */
{     0,   672, 5*16+ 4, 0x0 +NORM },	/* 671 */
{     0,   673, 5*16+ 8, 0x10+NORM },	/* 672 */
{     0,   674, 1*16+ 5, 0x0 +NORM },	/* 673 */
{     0,   675, 2*16+ 4, 0x10+NORM },	/* 674 */
{     0,   676, 6*16+ 3, 0x0 +NORM },	/* 675 */
{     0,   677, 4*16+ 2, 0x10+NORM },	/* 676 */
{     0,   678, 5*16+ 2, 0x0 +NORM },	/* 677 */
{     0,   679, 3*16+ 1, 0x10+NORM },	/* 678 */
{     0,     0, 4*16+10, 0x0 +NORM },	/* 679 */
{   281,   681, 3*16+ 8, 0x10+NORM },	/* 680 section 18 large knights kakari, 1 pt low pincer */
{     0,   698, 3*16+ 3, 0x0 +NORM },	/* 681 */
{     0,   683, 3*16+ 4, 0x10+NORM },	/* 682 */
{     0,   692, 4*16+ 4, 0x0 +NORM },	/* 683 */
{     0,   685, 2*16+ 3, 0x10+NORM },	/* 684 */
{     0,   686, 3*16+ 2, 0x0 +NORM },	/* 685 */
{     0,   687, 4*16+ 5, 0x10+NORM },	/* 686 */
{     0,   688, 5*16+ 4, 0x0 +NORM },	/* 687 */
{     0,   689, 3*16+ 5, 0x10+NORM },	/* 688 */
{     0,   690, 2*16+ 2, 0x0 +NORM },	/* 689 */
{     0,   691, 4*16+ 6, 0x10+NORM },	/* 690 */
{     0,     0, 7*16+ 4, 0x0 +NORM },	/* 691 */
{   684,   693, 4*16+ 5, 0x10+BAD  },	/* 692 */
{     0,   694, 5*16+ 4, 0x0 +NORM },	/* 693 */
{     0,   695, 2*16+ 3, 0x10+NORM },	/* 694 */
{     0,   696, 3*16+ 5, 0x0 +NORM },	/* 695 */
{     0,   697, 3*16+ 2, 0x10+NORM },	/* 696 */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 697 */
{   682,   699, 3*16+ 2, 0x10+BAD  },	/* 698 */
{     0,   700, 3*16+ 4, 0x0 +NORM },	/* 699 */
{     0,   701, 5*16+ 2, 0x10+NORM },	/* 700 */
{     0,   702, 5*16+ 6, 0x0 +NORM },	/* 701 */
{     0,   703, 5*16+ 4, 0x10+NORM },	/* 702 */
{     0,     0, 5*16+ 8, 0x0 +NORM },	/* 703 */
{     0,   705, 2*16+ 6, 0x0 +NORM },	/* 704 finish basic variation chapter 4 4-4 point section 19 3-3 invasion */
{     0,   706, 3*16+ 7, 0x10+NORM },	/* 705 */
{     0,   707, 4*16+ 2, 0x0 +NORM },	/* 706 */
{     0,   708, 5*16+ 2, 0x10+NORM },	/* 707 */
{     0,  1895, 3*16+ 2, 0x0 +NORM },	/* 708 */
{    65,  1894, 3*16+ 3, 0x0 +BAD  },	/* 709 3-3 invasion after handicap point start */
{    66,  1894, 3*16+ 3, 0x0 +BAD  },	/* 710 3-3 invasion after two handicap points */
{   709,   746, 3*16+10, 0x10+IGNR },	/* 711 Low extension down side */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 712 3-3 after two extensions */
{   711,   747, 3*16+ 9, 0x10+IGNR },	/* 713 */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 714 3-3 after a different two extensions */
{   713,   748, 4*16+ 9, 0x10+IGNR },	/* 715 */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 716 3-3 after a different two extensions */
{    43,   802,10*16+ 3, 0x10+IGNR },	/* 717  */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 718 3-3 after single long extension */
{   718,   720, 3*16+10, 0x10+NORM },	/* 719 */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 720 3-3 after two long low extensions */
{   719,   722, 3*16+ 9, 0x10+NORM },	/* 721 */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 722 3-3 after different two long extensions */
{   721,   724, 4*16+ 9, 0x10+NORM },	/* 723 */
{     0,  1894, 3*16+ 3, 0x0 +NORM },	/* 724 3-3 aftyer another pair of extensions */
{    23,   736, 2*16+ 6, 0x10+NORM },	/* 725 double hane variation */
{     0,   727, 3*16+ 6, 0x0 +NORM },	/* 726 */
{     0,   728, 4*16+ 5, 0x10+NORM },	/* 727 */
{     0,   733, 2*16+ 7, 0x0 +NORM },	/* 728 */
{     0,   730, 4*16+ 6, 0x10+NORM },	/* 729 */
{     0,   731, 1*16+ 6, 0x0 +NORM },	/* 730 */
{     0,   732, 3*16+ 7, 0x10+NORM },	/* 731 */
{     0,     0, 2*16+ 6, 0x0 +NORM },	/* 732 */
{   729,   774, 2*16+ 4, 0x10+NORM },	/* 733 variation */
{     0,   735, 1*16+ 6, 0x0 +NORM },	/* 734 */
{     0,     0, 2*16+ 3, 0x10+NORM },	/* 735 */
{   726,   737, 4*16+ 2, 0x0 +COMP },	/* 736 variation */
{     0,   738, 5*16+ 2, 0x10+NORM },	/* 737 */
{     0,   739, 5*16+ 3, 0x0 +NORM },	/* 738 */
{     0,   740, 4*16+ 5, 0x10+NORM },	/* 739 */
{     0,   741, 6*16+ 2, 0x0 +NORM },	/* 740 */
{     0,   742, 3*16+ 2, 0x10+NORM },	/* 741 */
{     0,   743, 5*16+ 1, 0x0 +NORM },	/* 742 */
{     0,     0, 2*16+ 2, 0x10+NORM },	/* 743 */
{   715,   842, 3*16+ 7, 0x10+IGNR },	/* 744 shorter extension this way */
{   717,   842, 3*16+ 7, 0x10+IGNR },	/* 745 or just the extension */
{   712,   842, 3*16+ 7, 0x10+IGNR },	/* 746 or this one */
{   714,   842, 3*16+ 7, 0x10+IGNR },	/* 747 or this one*/
{   716,   842, 3*16+ 7, 0x10+IGNR },	/* 748 or this one*/
{     0,   750, 3*16+ 3, 0x0 +NORM },	/* 749 */
{     0,   751, 4*16+ 3, 0x10+NORM },	/* 750 block on long side */
{     0,   765, 3*16+ 4, 0x0 +NORM },	/* 751 */
{     0,   753, 3*16+ 2, 0x10+COMP },	/* 752 */
{     0,   754, 3*16+ 6, 0x0 +NORM },	/* 753 */
{     0,   755, 4*16+ 6, 0x10+NORM },	/* 754 */
{     0,   756, 4*16+ 5, 0x0 +NORM },	/* 755 */
{     0,   757, 5*16+ 5, 0x10+NORM },	/* 756 */
{     0,   758, 3*16+ 5, 0x0 +NORM },	/* 757 */
{     0,   759, 4*16+ 7, 0x10+NORM },	/* 758 */
{     0,   760, 2*16+ 2, 0x0 +NORM },	/* 759 */
{     0,   761, 4*16+ 2, 0x10+NORM },	/* 760 */
{     0,   762, 2*16+ 7, 0x0 +NORM },	/* 761 */
{     0,   763, 2*16+ 8, 0x10+NORM },	/* 762 */
{     0,   764, 1*16+ 6, 0x0 +NORM },	/* 763 */
{     0,     0, 3*16+ 9, 0x10+NORM },	/* 764 */
{   752,   766, 3*16+ 5, 0x0 +NORM },	/* 765 variation not in book, oh well */
{     0,     0, 2*16+ 6, 0x10+NORM },	/* 766 */
{    19,   768, 4*16+ 2, 0x0 +BAD  },	/* 767 hane variation */
{     0,   769, 5*16+ 2, 0x10+NORM },	/* 768 */
{     0,   770, 2*16+ 2, 0x0 +NORM },	/* 769 */
{     0,   771, 4*16+ 1, 0x10+NORM },	/* 770 */
{     0,   772, 3*16+ 1, 0x0 +NORM },	/* 771 */
{     0,     0, 3*16+ 2, 0x10+NORM },	/* 772 */
{   745,     0, 3*16+ 6, 0x10+IGNR },	/* 773 4-4 knight's move shimari */
{   734,   775, 3*16+ 3, 0x0 +NORM },	/* 774 3-3 invasion */
{     0,   776, 4*16+ 3, 0x10+NORM },	/* 775 */
{     0,   768, 4*16+ 2, 0x0 +NORM },	/* 776 */
{   744,   778, 3*16+ 6, 0x10+IGNR },	/* 777 knight's after handicap*/
{     0,   775, 3*16+ 3, 0x0 +NORM },	/* 778 */
{   723,   780, 3*16+ 6, 0x10+IGNR },	/* 779 knight's after long extension */
{     0,   775, 3*16+ 3, 0x0 +NORM },	/* 780 */
{   773,   782, 3*16+ 5, 0x10+IGNR },	/* 781 4-4 kosumi */
{     0,   783, 3*16+ 3, 0x0 +NORM },	/* 782 */
{     0,   784, 4*16+ 3, 0x10+NORM },	/* 783 */
{     0,   785, 4*16+ 2, 0x0 +NORM },	/* 784 */
{     0,   786, 5*16+ 2, 0x10+URGT },	/* 785 */
{     0,   787, 2*16+ 2, 0x0 +NORM },	/* 786 */
{     0,     0, 2*16+ 4, 0x10+URGT },	/* 787 */
{   777,   789, 3*16+ 5, 0x10+IGNR },	/* 788 kosumi after handicap */
{     0,   783, 3*16+ 3, 0x0 +NORM },	/* 789 */
{   779,   791, 3*16+ 5, 0x10+NORM },	/* 790 kosumi after long extension*/
{     0,   783, 3*16+ 3, 0x0 +NORM },	/* 791 */
{   781,   808, 4*16+ 6, 0x10+IGNR },	/* 792 4-4 one pt jump shimari */
{     0,   794, 3*16+ 3, 0x0 +NORM },	/* 793 */
{     0,   795, 4*16+ 3, 0x10+NORM },	/* 794 */
{     0,   804, 2*16+ 4, 0x0 +NORM },	/* 795 */
{     0,   797, 3*16+ 2, 0x10+NORM },	/* 796 */
{     0,   798, 2*16+ 2, 0x0 +NORM },	/* 797 */
{     0,   799, 4*16+ 2, 0x10+NORM },	/* 798 */
{     0,     0, 2*16+ 6, 0x0 +NORM },	/* 799 */
{   788,   818, 4*16+ 6, 0x10+IGNR },	/* 800 1 pt jump after handicap */
{     0,   794, 3*16+ 3, 0x0 +NORM },	/* 801 */
{   790,   826, 4*16+ 6, 0x10+IGNR },	/* 802 */
{     0,   794, 3*16+ 3, 0x0 +NORM },	/* 803 */
{   796,   805, 2*16+ 6, 0x10+NORM },	/* 804 */
{     0,   806, 4*16+ 2, 0x0 +NORM },	/* 805 */
{     0,   807, 5*16+ 2, 0x10+NORM },	/* 806 */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 807 */
{   793,   849, 4*16+ 3, 0x0 +NORM },	/* 808 section 20 attach at 3-4 point */
{     0,   838, 3*16+ 3, 0x10+NORM },	/* 809 */
{     0,   828, 5*16+ 4, 0x0 +NORM },	/* 810 */
{     0,   812, 4*16+ 3, 0x0 +NORM },	/* 811 (transposition from 3-3 joseki)*/
{     0,   813, 4*16+ 4, 0x10+NORM },	/* 812 */
{     0,   828, 5*16+ 4, 0x0 +NORM },	/* 813 */
{     0,   815, 4*16+ 5, 0x10+NORM },	/* 814 */
{     0,   816, 5*16+ 3, 0x0 +NORM },	/* 815 */
{     0,   817, 3*16+ 2, 0x10+NORM },	/* 816 */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 817 */
{   801,   819, 4*16+ 3, 0x0 +NORM },	/* 818 attach with black extension in place */
{     0,   820, 3*16+ 3, 0x10+NORM },	/* 819 */
{     0,   821, 5*16+ 4, 0x0 +NORM },	/* 820 */
{     0,   822, 4*16+ 5, 0x10+NORM },	/* 821 */
{     0,   824, 6*16+ 3, 0x0 +NORM },	/* 822 */
{     0,   825, 4*16+ 2, 0x10+NORM },	/* 823 */
{   823,     0, 4*16+ 2, 0x0 +NORM },	/* 824 */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 825 */
{   803,   827, 4*16+ 3, 0x0 +NORM },	/* 826 different extension */
{     0,   820, 3*16+ 3, 0x10+NORM },	/* 827 */
{   814,   829, 5*16+ 3, 0x10+NORM },	/* 828 crosscut */
{     0,   830, 4*16+ 2, 0x0 +NORM },	/* 829 */
{     0,   831, 5*16+ 2, 0x10+NORM },	/* 830 */
{     0,   832, 4*16+ 5, 0x0 +NORM },	/* 831 */
{     0,   833, 3*16+ 4, 0x10+NORM },	/* 832 */
{     0,   834, 6*16+ 4, 0x0 +NORM },	/* 833 */
{     0,   835, 3*16+ 5, 0x10+NORM },	/* 834 */
{     0,   836, 6*16+ 2, 0x0 +NORM },	/* 835 */
{     0,   837, 3*16+ 2, 0x10+NORM },	/* 836 */
{     0,     0, 6*16+ 3, 0x0 +NORM },	/* 837 */
{   810,   839, 5*16+ 3, 0x0 +BAD  },	/* 838 a bad variation! */
{     0,   840, 3*16+ 2, 0x10+NORM },	/* 839 */
{     0,   841, 5*16+ 4, 0x0 +NORM },	/* 840 */
{     0,     0, 5*16+ 5, 0x10+NORM },	/* 841 */
{   749,   885, 4*16+ 3, 0x0 +NORM },	/* 842 attach with big knights shimari */
{     0,   844, 3*16+ 3, 0x10+NORM },	/* 843 */
{     0,   845, 5*16+ 3, 0x0 +NORM },	/* 844 */
{     0,   846, 3*16+ 2, 0x10+NORM },	/* 845 */
{     0,   847, 5*16+ 4, 0x0 +NORM },	/* 846 */
{     0,   848, 4*16+ 5, 0x10+NORM },	/* 847 */
{     0,     0, 8*16+ 3, 0x0 +NORM },	/* 848 */
{   809,   855, 5*16+ 3, 0x10+NORM },	/* 849 hane on the outside */
{     0,   851, 3*16+ 4, 0x0 +NORM },	/* 850 */
{     0,   852, 4*16+ 5, 0x10+NORM },	/* 851 */
{     0,   853, 3*16+ 2, 0x0 +NORM },	/* 852 */
{     0,   854, 5*16+ 2, 0x10+NORM },	/* 853 */
{     0,     0, 2*16+ 6, 0x0 +NORM },	/* 854 */
{   850,   866, 5*16+ 2, 0x0 +COMP },	/* 855 other counter hane*/
{     0,   857, 5*16+ 4, 0x10+NORM },	/* 856 */
{     0,   858, 4*16+ 2, 0x0 +NORM },	/* 857 */
{     0,   859, 6*16+ 2, 0x10+NORM },	/* 858 */
{     0,   860, 3*16+ 4, 0x0 +NORM },	/* 859 */
{     0,   861, 3*16+ 5, 0x10+NORM },	/* 860 */
{     0,   862, 2*16+ 3, 0x0 +NORM },	/* 861 */
{     0,   863, 2*16+ 4, 0x10+NORM },	/* 862 */
{     0,   864, 3*16+ 3, 0x0 +NORM },	/* 863 */
{     0,   865, 2*16+ 5, 0x10+NORM },	/* 864 */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 865 */
{   856,   867, 3*16+ 3, 0x10+COMP },	/* 866 Complicated version of this counter hane*/
{     0,   868, 4*16+ 2, 0x0 +NORM },	/* 867 */
{     0,   869, 6*16+ 3, 0x10+NORM },	/* 868 */
{     0,   870, 3*16+ 2, 0x0 +NORM },	/* 869 */
{     0,   871, 3*16+ 5, 0x10+NORM },	/* 870 */
{     0,   872, 2*16+ 4, 0x0 +NORM },	/* 871 */
{     0,   873, 2*16+ 3, 0x10+NORM },	/* 872 */
{     0,   874, 5*16+ 4, 0x0 +NORM },	/* 873 */
{     0,   875, 4*16+ 5, 0x10+NORM },	/* 874 */
{     0,   876, 2*16+ 2, 0x0 +NORM },	/* 875 */
{     0,   877, 2*16+ 5, 0x10+NORM },	/* 876 */
{     0,   878, 3*16+ 2, 0x0 +NORM },	/* 877 */
{     0,   884, 1*16+ 4, 0x10+NORM },	/* 878 */
{     0,   880, 7*16+ 3, 0x0 +COMP },	/* 879 */
{     0,   881, 6*16+ 2, 0x10+NORM },	/* 880 */
{     0,   882, 6*16+ 4, 0x0 +NORM },	/* 881 */
{     0,   883, 7*16+ 2, 0x10+NORM },	/* 882 */
{     0,     0, 8*16+ 3, 0x0 +NORM },	/* 883 */
{   879,     0, 6*16+ 2, 0x0 +NORM },	/* 884 */
{   843,   891, 5*16+ 3, 0x10+NORM },	/* 885 outside hane after attach large knight's*/
{     0,   887, 3*16+ 3, 0x0 +NORM },	/* 886 */
{     0,   888, 5*16+ 2, 0x10+NORM },	/* 887 */
{     0,   889, 2*16+ 5, 0x0 +NORM },	/* 888 */
{     0,   890, 4*16+ 6, 0x10+NORM },	/* 889 */
{     0,     0, 2*16+ 2, 0x0 +URGT },	/* 890 */
{   886,   892, 5*16+ 2, 0x0 +NORM },	/* 891 counter hane */
{     0,   893, 5*16+ 4, 0x10+NORM },	/* 892 */
{     0,   858, 4*16+ 2, 0x0 +NORM },	/* 893 */
{    93,  1055, 3*16+ 6, 0x10+NORM },	/* 894 knights response to kakari chapter 5 4-4 point, kakari */
{     0,   896, 4*16+ 2, 0x0 +NORM },	/* 895 slide*/
{     0,   897, 3*16+ 3, 0x10+NORM },	/* 896 block*/
{     0,   908, 9*16+ 3, 0x0 +NORM },	/* 897 extend*/
{     0,   912, 3*16+10, 0x10+NORM },	/* 898 */
{     0,   900, 8*16+ 4, 0x10+NORM },	/* 899 */
{     0,   901, 8*16+ 3, 0x0 +NORM },	/* 900 */
{     0,   902, 6*16+ 4, 0x10+NORM },	/* 901 */
{     0,   903, 7*16+ 4, 0x0 +NORM },	/* 902 */
{     0,   904, 7*16+ 5, 0x10+NORM },	/* 903 */
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 904 */
{     0,   900, 8*16+ 4, 0x10+NORM },	/* 905 */
{   898,   907, 8*16+ 3, 0x0 +NORM },	/* 906 attack*/
{     0,     0, 3*16+ 2, 0x10+URGT },	/* 907 */
{   905,   928, 8*16+ 3, 0x0 +NORM },	/* 908 */
{     0,     0, 3*16+ 2, 0x10+URGT },	/* 909 */
{   906,   900, 8*16+ 4, 0x10+NORM },	/* 910 */
{   910,   916, 3*16+ 2, 0x0 +NORM },	/* 911 white gets point in corner*/
{   899,   916, 3*16+ 2, 0x0 +NORM },	/* 912 */
{     0,   914, 2*16+ 2, 0x10+NORM },	/* 913 */
{     0,   915, 4*16+ 3, 0x0 +NORM },	/* 914 */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 915 */
{   913,   917, 3*16+ 5, 0x0 +NORM },	/* 916 White gets another move */
{     0,   918, 4*16+ 5, 0x10+NORM },	/* 917 */
{     0,   919, 2*16+ 3, 0x0 +NORM },	/* 918 */
{     0,   920, 2*16+ 4, 0x10+NORM },	/* 919 */
{     0,   921, 3*16+ 4, 0x0 +NORM },	/* 920 */
{     0,   922, 4*16+ 3, 0x10+NORM },	/* 921 */
{     0,   923, 2*16+ 5, 0x0 +NORM },	/* 922 */
{     0,   924, 2*16+ 2, 0x10+NORM },	/* 923 */
{     0,   925, 1*16+ 4, 0x0 +NORM },	/* 924 */
{     0,   926, 5*16+ 2, 0x10+NORM },	/* 925 */
{     0,   927, 2*16+ 1, 0x0 +NORM },	/* 926 */
{     0,     0, 5*16+ 3, 0x10+NORM },	/* 927 */
{   909,   929, 3*16+ 2, 0x0 +NORM },	/* 928 let white get another move */
{     0,   930, 2*16+ 2, 0x10+NORM },	/* 929 */
{     0,   931, 2*16+ 4, 0x0 +NORM },	/* 930 */
{     0,   932, 2*16+ 3, 0x10+NORM },	/* 931 */
{     0,   933, 3*16+ 5, 0x0 +NORM },	/* 932 */
{     0,   934, 4*16+ 5, 0x10+NORM },	/* 933 */
{     0,     0, 2*16+ 6, 0x0 +NORM },	/* 934 */
{    79,   936, 8*16+ 3, 0x10+NORM },	/* 935 pincer variation */
{     0,   937, 3*16+ 3, 0x0 +NORM },	/* 936 */
{     0,   938, 5*16+ 3, 0x10+NORM },	/* 937 */
{     0,   939, 5*16+ 2, 0x0 +NORM },	/* 938 */
{     0,   940, 6*16+ 4, 0x10+NORM },	/* 939 */
{     0,   941, 6*16+ 2, 0x0 +NORM },	/* 940 */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 941 */
{     0,   949, 7*16+ 7, 0x0 +NORM },	/* 942 finish standard joseki section 22 4-4, kakari, kosumi */
{   792,   944,11*16+ 3, 0x10+IGNR },	/* 943 if have long extension */
{     0,   945, 6*16+ 3, 0x0 +NORM },	/* 944 */
{     0,   946, 5*16+ 3, 0x10+NORM },	/* 945 */
{     0,   947, 6*16+ 4, 0x0 +NORM },	/* 946 */
{     0,   948, 4*16+ 6, 0x10+NORM },	/* 947 */
{     0,   963, 9*16+ 3, 0x0 +NORM },	/* 948 */
{     0,   991, 3*16+ 3, 0x0 +NORM },	/* 949 invade at 3-3 after joseki */
{     0,   951, 5*16+ 2, 0x10+NORM },	/* 950 */
{     0,   952, 3*16+ 5, 0x0 +NORM },	/* 951 */
{     0,   964, 4*16+ 5, 0x10+NORM },	/* 952 */
{     0,   954, 3*16+ 6, 0x0 +NORM },	/* 953 */
{     0,   955, 3*16+ 4, 0x10+NORM },	/* 954 */
{     0,   956, 2*16+ 4, 0x0 +NORM },	/* 955 */
{     0,   957, 3*16+ 7, 0x10+NORM },	/* 956 */
{     0,   958, 2*16+ 7, 0x0 +NORM },	/* 957 */
{     0,   959, 2*16+ 3, 0x10+NORM },	/* 958 */
{     0,   960, 2*16+ 5, 0x0 +NORM },	/* 959 */
{     0,   961, 3*16+ 2, 0x10+NORM },	/* 960 */
{     0,   962, 3*16+ 8, 0x0 +NORM },	/* 961 */
{     0,     0, 4*16+ 7, 0x10+NORM },	/* 962 */
{     0,   991, 3*16+ 3, 0x0 +NORM },	/* 963 alternate version, same invasion */
{   953,   965, 2*16+ 6, 0x0 +NORM },	/* 964 another way to live */
{     0,   966, 3*16+ 6, 0x10+NORM },	/* 965 */
{     0,   967, 2*16+ 5, 0x0 +NORM },	/* 966 */
{     0,   968, 2*16+ 7, 0x10+NORM },	/* 967 */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 968 */
{   950,   976, 3*16+ 4, 0x10+NORM },	/* 969 sacrifice a stone and keep corner */
{     0,   971, 5*16+ 2, 0x0 +NORM },	/* 970 */
{     0,   972, 4*16+ 2, 0x10+NORM },	/* 971 */
{     0,   973, 4*16+ 3, 0x0 +NORM },	/* 972 */
{     0,   974, 3*16+ 2, 0x10+NORM },	/* 973 */
{     0,   975, 5*16+ 4, 0x0 +NORM },	/* 974 */
{     0,     0, 3*16+ 2, 0x10+NORM },	/* 975 */
{   970,   977, 4*16+ 3, 0x10+NORM },	/* 976 another variation */
{     0,   978, 4*16+ 2, 0x0 +NORM },	/* 977 */
{     0,     0, 3*16+ 2, 0x10+NORM },	/* 978 */
{   969,   980, 4*16+ 2, 0x10+NORM },	/* 979 another way to handle 3-3 invasion*/
{     0,   981, 3*16+ 2, 0x0 +NORM },	/* 980 */
{     0,   982, 6*16+ 2, 0x10+NORM },	/* 981 */
{     0,     0, 2*16+ 5, 0x0 +NORM },	/* 982 */
{   979,   984, 6*16+ 2, 0x10+NORM },	/* 983 and another*/
{     0,   985, 7*16+ 2, 0x0 +NORM },	/* 984 */
{     0,   986, 4*16+ 2, 0x10+NORM },	/* 985 */
{     0,   987, 2*16+ 5, 0x0 +NORM },	/* 986 */
{     0,   988, 3*16+ 5, 0x10+NORM },	/* 987 */
{     0,   989, 2*16+ 6, 0x0 +NORM },	/* 988 */
{     0,   990, 2*16+ 4, 0x10+NORM },	/* 989 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 990 */
{   983,   992, 4*16+ 3, 0x10+NORM },	/* 991 last one*/
{     0,   993, 2*16+ 4, 0x0 +NORM },	/* 992 */
{     0,   994, 3*16+ 4, 0x10+NORM },	/* 993 */
{     0,   995, 2*16+ 2, 0x0 +NORM },	/* 994 */
{     0,   996, 3*16+ 2, 0x10+NORM },	/* 995 */
{     0,   997, 2*16+ 3, 0x0 +NORM },	/* 996 */
{     0,   998, 2*16+ 5, 0x10+NORM },	/* 997 */
{     0,   999, 4*16+ 2, 0x0 +NORM },	/* 998 */
{     0,  1000, 5*16+ 2, 0x10+NORM },	/* 999 */
{     0,     0, 3*16+ 1, 0x0 +NORM },	/* 1000 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 1001 */
{   894,  1159, 3*16+ 7, 0x10+NORM },	/* 1002 large knight's extension */
{    28,  1004, 9*16+ 4, 0x0 +NORM },	/* 1003 short extension to 4 line. section 23, white approaches, then just extend*/
{     0,  1005, 5*16+ 3, 0x10+NORM },	/* 1004 */
{     0,  1010, 6*16+ 4, 0x0 +NORM },	/* 1005 */
{  1003,  1230,10*16+ 4, 0x0 +NORM },	/* 1006 long 4 line extension */
{  1006,  1008, 9*16+ 3, 0x0 +BAD  },	/* 1007 short 3 line extension */
{     0,  1009, 5*16+ 3, 0x10+NORM },	/* 1008 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 1009 */
{     0,     0,11*16+ 3, 0x10+FOLL },	/* 1010 can attack extension to 4 line */
{  1007,  1015,10*16+ 3, 0x0 +NORM },	/* 1011 long 3 line extension */
{     0,  1013, 5*16+ 3, 0x10+BAD  },	/* 1012 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 1013 */
{  1012,     0, 4*16+ 2, 0x10+NORM },	/* 1014 take corner */
{  1014,     0, 8*16+ 3, 0x10+FOLL },	/* 1015 invasion later */
{  1011,  1017, 3*16+ 3, 0x0 +NORM },	/* 1016 section 24 4-4, kakari, extend, 3-3 invasion */
{     0,  1018, 4*16+ 3, 0x10+NORM },	/* 1017 */
{     0,  1051, 4*16+ 2, 0x0 +NORM },	/* 1018 */
{     0,  1042, 5*16+ 2, 0x10+NORM },	/* 1019 */
{     0,  1021, 3*16+ 2, 0x0 +NORM },	/* 1020 */
{     0,  1022, 5*16+ 3, 0x10+NORM },	/* 1021 */
{     0,  1025, 2*16+ 5, 0x0 +NORM },	/* 1022 */
{     0,  1024, 6*16+ 4, 0x10+NORM },	/* 1023 */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 1024 */
{  1023,  1026, 2*16+ 6, 0x10+COMP },	/* 1025 seal in corner variation */
{     0,  1027, 3*16+ 5, 0x0 +NORM },	/* 1026 */
{     0,  1028, 3*16+ 6, 0x10+NORM },	/* 1027 */
{     0,  1029, 4*16+ 5, 0x0 +NORM },	/* 1028 */
{     0,  1030, 5*16+ 5, 0x10+NORM },	/* 1029 */
{     0,  1031, 5*16+ 6, 0x0 +NORM },	/* 1030 */
{     0,  1032, 3*16+ 4, 0x10+NORM },	/* 1031 */
{     0,  1033, 2*16+ 4, 0x0 +NORM },	/* 1032 */
{     0,  1034, 6*16+ 5, 0x10+NORM },	/* 1033 */
{     0,  1039, 1*16+ 5, 0x0 +NORM },	/* 1034 */
{     0,  1036, 5*16+ 7, 0x10+COMP },	/* 1035 */
{     0,     0, 6*16+ 6, 0x0 +COMP },	/* 1036 */
{  1035,  1038, 1*16+ 6, 0x10+NORM },	/* 1037 */
{     0,     0, 1*16+ 3, 0x0 +NORM },	/* 1038 */
{  1037,  1040, 1*16+ 3, 0x10+NORM },	/* 1039 */
{     0,  1041, 2*16+ 3, 0x0 +NORM },	/* 1040 */
{     0,     0, 1*16+ 2, 0x10+NORM },	/* 1041 */
{  1020,  1043, 2*16+ 2, 0x0 +NORM },	/* 1042 */
{     0,  1044, 4*16+ 1, 0x10+NORM },	/* 1043 */
{     0,  1045, 3*16+ 2, 0x0 +NORM },	/* 1044 */
{     0,  1046, 5*16+ 3, 0x10+NORM },	/* 1045 */
{     0,  1048, 2*16+ 6, 0x0 +NORM },	/* 1046 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1047 */
{  1047,  1049, 2*16+ 7, 0x10+NORM },	/* 1048 */
{     0,  1050, 2*16+ 5, 0x0 +NORM },	/* 1049 */
{     0,     0, 3*16+ 7, 0x10+NORM },	/* 1050 */
{  1019,  1052, 3*16+ 4, 0x10+NORM },	/* 1051 variation to hang on to edge and allwo conn */
{     0,  1053, 2*16+ 2, 0x0 +NORM },	/* 1052 */
{     0,  1054, 2*16+ 3, 0x10+NORM },	/* 1053 */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 1054 */
{   895,  1056, 3*16+ 3, 0x0 +NORM },	/* 1055 3-3 inv after sm knight's*/
{     0,  1057, 4*16+ 3, 0x10+NORM },	/* 1056 */
{     0,  1058, 4*16+ 2, 0x0 +NORM },	/* 1057 */
{     0,  1059, 5*16+ 2, 0x10+NORM },	/* 1058 */
{     0,  1060, 3*16+ 2, 0x0 +NORM },	/* 1059 */
{     0,  1061, 5*16+ 3, 0x10+NORM },	/* 1060 */
{     0,  1062, 2*16+ 5, 0x0 +NORM },	/* 1061 */
{     0,  1063, 2*16+ 6, 0x10+NORM },	/* 1062 */
{     0,  1064, 3*16+ 5, 0x0 +NORM },	/* 1063 */
{     0,  1065, 4*16+ 5, 0x10+NORM },	/* 1064 */
{     0,  1066, 3*16+ 4, 0x0 +NORM },	/* 1065 */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 1066 */
{   943,  1068, 3*16+10, 0x0 +IGNR },	/* 1067 stone in middle of side */
{     0,  1069, 6*16+ 3, 0x0 +NORM },	/* 1068 */
{     0,  1070, 3*16+ 6, 0x10+NORM },	/* 1069 defend with low move */
{     0,  1071, 3*16+ 3, 0x0 +NORM },	/* 1070 3-3 invasion */
{     0,  1072, 4*16+ 3, 0x10+NORM },	/* 1071 */
{     0,  1073, 4*16+ 2, 0x0 +NORM },	/* 1072 */
{     0,  1074, 5*16+ 2, 0x10+NORM },	/* 1073 */
{     0,  1075, 3*16+ 2, 0x0 +NORM },	/* 1074 */
{     0,  1076, 5*16+ 3, 0x10+NORM },	/* 1075 */
{     0,  1081, 2*16+ 5, 0x0 +NORM },	/* 1076 */
{     0,  1078, 2*16+ 6, 0x10+BAD  },	/* 1077 */
{     0,  1079, 3*16+ 5, 0x0 +NORM },	/* 1078 */
{     0,  1080, 4*16+ 5, 0x10+NORM },	/* 1079 */
{     0,     0, 4*16+ 6, 0x0 +COMP },	/* 1080 */
{  1077,  1082, 3*16+ 5, 0x10+NORM },	/* 1081 right move */
{     0,  1083, 2*16+ 6, 0x0 +NORM },	/* 1082 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1083 */
{     0,  1085, 3*16+ 3, 0x0 +NORM },	/* 1084 3-3 invasion after large knights */
{     0,  1086, 4*16+ 3, 0x10+NORM },	/* 1085 */
{     0,  1087, 3*16+ 4, 0x0 +NORM },	/* 1086 */
{     0,  1088, 4*16+ 5, 0x10+NORM },	/* 1087 */
{     0,  1089, 4*16+ 2, 0x0 +NORM },	/* 1088 */
{     0,  1090, 5*16+ 2, 0x10+NORM },	/* 1089 */
{     0,  1091, 3*16+ 2, 0x0 +NORM },	/* 1090 */
{     0,  1092, 5*16+ 3, 0x10+NORM },	/* 1091 */
{     0,  1093, 3*16+ 5, 0x0 +NORM },	/* 1092 */
{     0,  1094, 3*16+ 6, 0x10+NORM },	/* 1093 */
{     0,  1095, 6*16+ 5, 0x0 +NORM },	/* 1094 */
{     0,  1096, 6*16+ 6, 0x10+NORM },	/* 1095 */
{     0,  1097, 7*16+ 5, 0x0 +NORM },	/* 1096 */
{     0,     0, 6*16+ 7, 0x10+NORM },	/* 1097 */
{     0,     0, 4*16+ 2, 0x0 +NORM },	/* 1098 */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 1099 */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 1100 */
{  1016,  1102, 4*16+ 3, 0x0 +COMP },	/* 1101 section 25 4-4 kakari, extend, tsuke */
{     0,  1115, 3*16+ 3, 0x10+URGT },	/* 1102 */
{     0,  1104, 5*16+ 4, 0x0 +COMP },	/* 1103 */
{     0,  1106, 4*16+ 2, 0x10+NORM },	/* 1104 */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 1105 */
{  1105,  1107, 4*16+ 5, 0x0 +COMP },	/* 1106 another variation */
{     0,  1108, 5*16+ 3, 0x10+NORM },	/* 1107 */
{     0,  1109, 3*16+ 4, 0x0 +NORM },	/* 1108 */
{     0,  1110, 4*16+ 3, 0x10+NORM },	/* 1109 */
{     0,  1111, 5*16+ 5, 0x0 +NORM },	/* 1110 */
{     0,  1112, 3*16+ 5, 0x10+NORM },	/* 1111 */
{     0,  1113, 3*16+ 6, 0x0 +NORM },	/* 1112 */
{     0,  1114, 2*16+ 4, 0x10+NORM },	/* 1113 */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 1114 */
{  1103,  1129, 3*16+ 2, 0x0 +NORM },	/* 1115 other major variation */
{     0,  1117, 3*16+ 4, 0x10+BAD  },	/* 1116 */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 1117 */
{  1116,  1119, 2*16+ 2, 0x10+NORM },	/* 1118 */
{     0,  1120, 5*16+ 2, 0x0 +NORM },	/* 1119 */
{     0,  1121, 3*16+ 1, 0x10+NORM },	/* 1120 */
{     0,  1125, 5*16+ 4, 0x0 +NORM },	/* 1121 */
{     0,  1123, 4*16+ 5, 0x10+NORM },	/* 1122 */
{     0,  1124, 4*16+ 2, 0x0 +NORM },	/* 1123 */
{     0,     0, 2*16+ 4, 0x10+NORM },	/* 1124 */
{  1122,  1126, 4*16+ 2, 0x10+NORM },	/* 1125 another variation*/
{     0,  1127, 4*16+ 5, 0x0 +NORM },	/* 1126 */
{     0,  1128, 3*16+ 4, 0x10+NORM },	/* 1127 */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 1128 */
{  1118,  1130, 5*16+ 3, 0x10+COMP },	/* 1129 aggressive move */
{     0,  1131, 4*16+ 2, 0x0 +NORM },	/* 1130 */
{     0,  1132, 5*16+ 2, 0x10+NORM },	/* 1131 */
{     0,  1133, 5*16+ 4, 0x0 +NORM },	/* 1132 */
{     0,  1134, 6*16+ 2, 0x10+NORM },	/* 1133 */
{     0,  1135, 6*16+ 4, 0x0 +NORM },	/* 1134 */
{     0,  1136, 7*16+ 3, 0x10+NORM },	/* 1135 */
{     0,  1137, 7*16+ 4, 0x0 +NORM },	/* 1136 */
{     0,  1141, 8*16+ 3, 0x10+NORM },	/* 1137 */
{     0,  1139, 3*16+ 4, 0x0 +NORM },	/* 1138 */
{     0,  1140, 4*16+ 5, 0x10+NORM },	/* 1139 */
{     0,     0, 2*16+ 4, 0x0 +NORM },	/* 1140 */
{  1138,  1142, 4*16+ 5, 0x0 +NORM },	/* 1141 */
{     0,  1143, 3*16+ 4, 0x10+NORM },	/* 1142 */
{     0,  1144, 3*16+ 5, 0x0 +NORM },	/* 1143 */
{     0,  1145, 2*16+ 2, 0x10+NORM },	/* 1144 */
{     0,  1146, 5*16+ 6, 0x0 +NORM },	/* 1145 */
{     0,  1147, 2*16+ 3, 0x10+FOLL },	/* 1146 */
{     0,  1148, 2*16+ 4, 0x0 +NORM },	/* 1147 */
{     0,     0, 1*16+ 2, 0x10+NORM },	/* 1148 */
{    13,  1150, 4*16+ 6, 0x10+NORM },	/* 1149 extend after 1 pt high approach */
{     0,  1151, 4*16+ 3, 0x0 +NORM },	/* 1150 tsuke*/
{     0,  1154, 3*16+ 3, 0x10+NORM },	/* 1151 */
{     0,  1153, 5*16+ 3, 0x0 +NORM },	/* 1152 */
{     0,     0, 3*16+ 2, 0x10+NORM },	/* 1153 */
{  1152,  1155, 3*16+ 2, 0x0 +NORM },	/* 1154 variation*/
{     0,  1156, 5*16+ 3, 0x10+NORM },	/* 1155 */
{     0,  1157, 4*16+ 2, 0x0 +NORM },	/* 1156 */
{     0,  1158, 5*16+ 4, 0x10+NORM },	/* 1157 */
{     0,     0, 3*16+ 4, 0x0 +NORM },	/* 1158 */
{  1084,  1160, 4*16+ 3, 0x0 +NORM },	/* 1159 tsuke after large knights extension */
{     0,  1161, 3*16+ 3, 0x10+NORM },	/* 1160 */
{     0,  1162, 3*16+ 4, 0x0 +NORM },	/* 1161 */
{     0,  1163, 4*16+ 5, 0x10+NORM },	/* 1162 */
{     0,  1164, 3*16+ 2, 0x0 +NORM },	/* 1163 */
{     0,  1165, 2*16+ 3, 0x10+NORM },	/* 1164 */
{     0,  1166, 5*16+ 2, 0x0 +NORM },	/* 1165 */
{     0,  1167, 2*16+ 2, 0x10+NORM },	/* 1166 */
{     0,  1168, 2*16+ 1, 0x0 +NORM },	/* 1167 */
{     0,     0, 3*16+ 5, 0x10+NORM },	/* 1168 */
{   103,  1175, 7*16+ 5, 0x0 +NORM },	/* 1169 first part already done elsewhere.  section 26 4-4, attach and block jose*/
{     0,  1171, 7*16+ 3, 0x10+NORM },	/* 1170 */
{     0,  1172, 8*16+ 3, 0x0 +NORM },	/* 1171 */
{     0,  1173, 6*16+ 2, 0x10+NORM },	/* 1172 */
{     0,  1174, 7*16+ 2, 0x0 +NORM },	/* 1173 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1174 */
{  1170,  1176, 4*16+ 6, 0x10+NORM },	/* 1175 can extend immediately */
{     0,  1177, 7*16+ 2, 0x10+FOLL },	/* 1176 */
{     0,     0, 6*16+ 2, 0x0 +FOLL },	/* 1177 */
{  1169,  1180, 7*16+ 3, 0x0 +NORM },	/* 1178 connect on bottom*/
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1179 */
{  1179,     0, 7*16+ 5, 0x10+COMP },	/* 1180 */
{   104,  1182, 7*16+ 5, 0x10+NORM },	/* 1181 cut after descent */
{     0,  1195, 5*16+ 2, 0x0 +NORM },	/* 1182 */
{     0,  1184, 4*16+ 2, 0x10+NORM },	/* 1183 */
{     0,  1185, 8*16+ 4, 0x0 +NORM },	/* 1184 */
{     0,  1186, 6*16+ 6, 0x10+NORM },	/* 1185 */
{     0,  1187, 8*16+ 5, 0x0 +NORM },	/* 1186 */
{     0,  1188, 7*16+ 6, 0x10+NORM },	/* 1187 */
{     0,  1193, 3*16+ 3, 0x0 +NORM },	/* 1188 */
{     0,  1190, 3*16+ 4, 0x10+NORM },	/* 1189 */
{     0,  1191, 3*16+ 2, 0x0 +NORM },	/* 1190 */
{     0,  1192, 4*16+ 3, 0x10+NORM },	/* 1191 */
{     0,     0, 4*16+ 1, 0x0 +NORM },	/* 1192 */
{  1189,  1194, 4*16+ 3, 0x10+BAD  },	/* 1193 */
{     0,     0, 3*16+ 4, 0x0 +URGT },	/* 1194 */
{  1183,  1196, 8*16+ 4, 0x10+COMP },	/* 1195 another variation after cut*/
{     0,  1197, 7*16+ 3, 0x0 +NORM },	/* 1196 */
{     0,  1198, 3*16+ 3, 0x10+NORM },	/* 1197 */
{     0,  1199, 6*16+ 6, 0x0 +NORM },	/* 1198 */
{     0,  1200, 8*16+ 5, 0x10+NORM },	/* 1199 */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 1200 */
{    94,  1219, 5*16+ 4, 0x0 +BAD  },	/* 1201 wedge after attatch */
{     0,  1203, 5*16+ 5, 0x10+NORM },	/* 1202 */
{     0,  1204, 5*16+ 3, 0x0 +NORM },	/* 1203 */
{     0,  1205, 4*16+ 5, 0x10+NORM },	/* 1204 */
{     0,  1206, 7*16+ 4, 0x0 +NORM },	/* 1205 */
{     0,  1207, 6*16+ 5, 0x10+NORM },	/* 1206 */
{     0,  1208, 4*16+ 3, 0x0 +NORM },	/* 1207 */
{     0,  1209, 3*16+ 3, 0x10+NORM },	/* 1208 */
{     0,  1210, 3*16+ 2, 0x0 +NORM },	/* 1209 */
{     0,  1211, 2*16+ 2, 0x10+NORM },	/* 1210 */
{     0,  1212, 2*16+ 3, 0x0 +NORM },	/* 1211 */
{     0,  1213, 3*16+ 4, 0x10+NORM },	/* 1212 */
{     0,  1214, 2*16+ 1, 0x0 +NORM },	/* 1213 */
{     0,  1215, 7*16+ 3, 0x10+NORM },	/* 1214 */
{     0,  1216, 8*16+ 3, 0x0 +NORM },	/* 1215 */
{     0,  1217, 7*16+ 2, 0x10+NORM },	/* 1216 */
{     0,  1218, 1*16+ 2, 0x0 +NORM },	/* 1217 */
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 1218 */
{  1202,  1220, 5*16+ 3, 0x10+NORM },	/* 1219 ladder variation */
{     0,  1221, 5*16+ 5, 0x0 +NORM },	/* 1220 */
{     0,  1223, 4*16+ 3, 0x10+NORM },	/* 1221 */
{     0,     0, 7*16+ 4, 0x0 +NORM },	/* 1222 */
{  1222,  1224, 7*16+ 3, 0x0 +BAD  },	/* 1223 when ladder doesn't work */
{     0,  1225, 4*16+ 6, 0x10+NORM },	/* 1224 */
{     0,  1226, 5*16+ 6, 0x0 +NORM },	/* 1225 */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 1226 */
{  1067,  1228,10*16+ 4, 0x0 +IGNR },	/* 1227 when white has a stone down the edge */
{     0,  1232, 6*16+ 3, 0x0 +NORM },	/* 1228 */
{     0,  1230, 4*16+ 6, 0x10+NORM },	/* 1229 */
{     0,  1231, 4*16+ 2, 0x0 +NORM },	/* 1230 */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 1231 */
{  1229,  1250, 6*16+ 4, 0x10+NORM },	/* 1232 approach, attach*/
{     0,  1246, 5*16+ 4, 0x0 +COMP },	/* 1233 wedge */
{     0,  1235, 5*16+ 5, 0x10+NORM },	/* 1234 */
{     0,  1236, 5*16+ 3, 0x0 +NORM },	/* 1235 */
{     0,  1237, 4*16+ 5, 0x10+NORM },	/* 1236 */
{     0,  1238, 7*16+ 4, 0x0 +NORM },	/* 1237 */
{     0,  1239, 6*16+ 5, 0x10+NORM },	/* 1238 */
{     0,  1240, 4*16+ 3, 0x0 +NORM },	/* 1239 */
{     0,  1241, 3*16+ 3, 0x10+NORM },	/* 1240 */
{     0,  1242, 3*16+ 2, 0x0 +NORM },	/* 1241 */
{     0,  1243, 2*16+ 2, 0x10+NORM },	/* 1242 */
{     0,  1244, 2*16+ 3, 0x0 +NORM },	/* 1243 */
{     0,  1245, 3*16+ 4, 0x10+NORM },	/* 1244 */
{     0,     0, 2*16+ 1, 0x0 +NORM },	/* 1245 */
{  1234,  1247, 5*16+ 3, 0x10+NORM },	/* 1246 wedge, ladder */
{     0,  1248, 5*16+ 5, 0x0 +NORM },	/* 1247 */
{     0,  1249, 4*16+ 3, 0x10+NORM },	/* 1248 */
{     0,     0, 7*16+ 4, 0x0 +NORM },	/* 1249 */
{  1233,  1251, 7*16+ 4, 0x0 +NORM },	/* 1250 hane after attatch*/
{     0,  1252, 5*16+ 3, 0x10+NORM },	/* 1251 */
{     0,  1253, 6*16+ 5, 0x0 +NORM },	/* 1252 */
{     0,  1254, 5*16+ 4, 0x10+NORM },	/* 1253 */
{     0,  1255, 7*16+ 3, 0x0 +NORM },	/* 1254 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1255 */
{     0,     0, 3*16+ 6, 0x0 +NORM },	/* 1256 */
{     0,  1283, 3*16+ 8, 0x10+NORM },	/* 1267 */
{     0,     0, 3*16+ 3, 0x0 +NORM },	/* 1258 */
{     0,     0, 4*16+ 3, 0x10+NORM },	/* 1259 */
{     0,  1281, 3*16+ 4, 0x0 +NORM },	/* 1260 */
{     0,     0, 4*16+ 5, 0x10+NORM },	/* 1261 */
{    73,  1263, 4*16+ 6, 0x0 +BAD  },	/* 1262 incorrect move */
{     0,  1264, 3*16+ 5, 0x10+NORM },	/* 1263 */
{     0,  1265, 2*16+ 5, 0x0 +NORM },	/* 1264 */
{     0,  1266, 2*16+ 4, 0x10+NORM },	/* 1265 */
{     0,  1267, 2*16+ 3, 0x0 +NORM },	/* 1266 */
{     0,  1268, 2*16+ 6, 0x10+NORM },	/* 1267 */
{     0,  1270, 1*16+ 4, 0x0 +NORM },	/* 1268 */
{     0,     0, 2*16+ 7, 0x10+NORM },	/* 1269 */
{  1269,  1271, 5*16+ 6, 0x10+NORM },	/* 1270 */
{     0,  1272, 2*16+ 7, 0x0 +NORM },	/* 1271 */
{     0,  1273, 4*16+ 7, 0x10+NORM },	/* 1272 */
{     0,  1274, 1*16+ 6, 0x0 +NORM },	/* 1273 */
{     0,  1275, 3*16+ 7, 0x10+NORM },	/* 1274 */
{     0,  1276, 2*16+ 6, 0x0 +NORM },	/* 1275 */
{     0,     0, 6*16+ 7, 0x10+NORM },	/* 1276 */
{    74,  1278, 3*16+ 2, 0x0 +NORM },	/* 1277 correct move */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1278 */
{     0,     0, 2*16+ 6, 0x0 +NORM },	/* 1279 */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 1280 */
{     0,  1282, 2*16+ 6, 0x0 +NORM },	/* 1281 other correct move */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 1282 */
{     0,  1284, 5*16+ 6, 0x0 +COMP },	/* 1283 one point jump after pincer */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1284 */
{    69,  1286, 4*16+ 9, 0x10+NORM },	/* 1285 2 pt high pincer */
{     0,  1287, 5*16+ 6, 0x0 +NORM },	/* 1286 */
{     0,  1288, 6*16+ 4, 0x10+NORM },	/* 1287 */
{     0,  1289, 2*16+ 4, 0x0 +NORM },	/* 1288 */
{     0,  1290, 3*16+ 3, 0x10+NORM },	/* 1289 */
{     0,     0, 2*16+ 9, 0x0 +NORM },	/* 1290 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1291 */
{     0,     0, 2*16+ 4, 0x0 +NORM },	/* 1292 */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 1293 */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 1294 */
{  1002,  1296,10*16+ 3, 0x10+NORM },	/* 1295 long low pincer */
{     0,  1297, 3*16+ 3, 0x0 +NORM },	/* 1296 */
{     0,  1298, 4*16+ 3, 0x10+NORM },	/* 1297 */
{     0,  1300, 3*16+ 4, 0x0 +NORM },	/* 1298 */
{     0,  1308, 4*16+ 5, 0x10+NORM },	/* 1299 */
{  1299,  1301, 3*16+ 5, 0x10+BAD  },	/* 1300 (bad variation cant hane)*/
{     0,  1302, 4*16+ 2, 0x0 +NORM },	/* 1301 */
{     0,  1303, 5*16+ 2, 0x10+NORM },	/* 1302 */
{     0,  1304, 3*16+ 2, 0x0 +NORM },	/* 1303 */
{     0,  1305, 5*16+ 3, 0x10+NORM },	/* 1304 */
{     0,  1306, 6*16+ 2, 0x0 +NORM },	/* 1305 */
{     0,  1307, 5*16+ 1, 0x10+NORM },	/* 1306 */
{     0,     0, 4*16+ 5, 0x0 +NORM },	/* 1307 */
{     0,  1309, 4*16+ 2, 0x0 +NORM },	/* 1308 correct move*/
{     0,  1310, 5*16+ 2, 0x10+NORM },	/* 1309 */
{     0,  1311, 3*16+ 2, 0x0 +NORM },	/* 1310 */
{     0,  1320, 5*16+ 3, 0x10+NORM },	/* 1311 */
{     0,  1314, 3*16+ 6, 0x0 +NORM },	/* 1312 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1313 */
{  1313,  1315, 6*16+ 5, 0x0 +COMP },	/* 1314 */
{     0,  1316, 4*16+ 6, 0x10+NORM },	/* 1315 */
{     0,  1317, 3*16+ 7, 0x0 +NORM },	/* 1316 */
{     0,  1318, 4*16+ 7, 0x10+NORM },	/* 1317 */
{     0,  1319, 4*16+ 8, 0x0 +NORM },	/* 1318 */
{     0,     0, 6*16+ 7, 0x10+NORM },	/* 1319 */
{  1312,  1321, 5*16+ 7, 0x10+NORM },	/* 1320 another ending */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 1321 */
{  1295,  1399, 3*16+ 6, 0x0 +NORM },	/* 1322 double low kakari.  section 28 double kakari against the 4-4 point */
{  1322,  1324,10*16+ 4, 0x10+NORM },	/* 1323 after long pincer */
{     0,  1399, 3*16+ 6, 0x0 +NORM },	/* 1324 */
{     0,  1326, 4*16+ 6, 0x10+NORM },	/* 1325 attach*/
{     0,  1386, 4*16+ 7, 0x0 +NORM },	/* 1326 */
{     0,  1373, 5*16+ 6, 0x10+NORM },	/* 1327 and extend*/
{     0,  1342, 3*16+ 3, 0x0 +NORM },	/* 1328 */
{     0,  1330, 4*16+ 3, 0x10+NORM },	/* 1329 */
{     0,  1331, 3*16+ 4, 0x0 +NORM },	/* 1330 */
{     0,  1332, 3*16+ 5, 0x10+NORM },	/* 1331 */
{     0,  1333, 2*16+ 5, 0x0 +NORM },	/* 1332 */
{     0,  1334, 4*16+ 5, 0x10+NORM },	/* 1333 */
{     0,  1335, 3*16+ 7, 0x0 +NORM },	/* 1334 */
{     0,  1336, 2*16+ 6, 0x10+NORM },	/* 1335 */
{     0,  1337, 2*16+ 4, 0x0 +NORM },	/* 1336 */
{     0,  1339, 5*16+ 7, 0x10+NORM },	/* 1337 */
{     0,     0, 4*16+ 9, 0x0 +NORM },	/* 1338 */
{  1338,  1340, 4*16+ 8, 0x10+NORM },	/* 1339 */
{     0,  1341, 2*16+ 8, 0x0 +NORM },	/* 1340 */
{     0,     0, 3*16+10, 0x10+NORM },	/* 1341 */
{  1329,  1345, 3*16+ 5, 0x10+NORM },	/* 1342 then block*/
{     0,  1344, 4*16+ 3, 0x0 +NORM },	/* 1343 */
{     0,     0, 3*16+ 7, 0x10+NORM },	/* 1344 */
{  1343,  1346, 3*16+ 7, 0x0 +COMP },	/* 1345 variation*/
{     0,     0, 4*16+ 3, 0x10+NORM },	/* 1346 */
{  1328,  1348, 3*16+ 4, 0x0 +NORM },	/* 1347 extend then tsuke */
{     0,  1349, 3*16+ 5, 0x10+NORM },	/* 1348 */
{     0,  1350, 2*16+ 5, 0x0 +NORM },	/* 1349 */
{     0,  1351, 4*16+ 5, 0x10+NORM },	/* 1350 */
{     0,  1352, 2*16+ 7, 0x0 +NORM },	/* 1351 */
{     0,  1353, 3*16+ 3, 0x10+NORM },	/* 1352 */
{     0,  1354, 2*16+ 4, 0x0 +NORM },	/* 1353 */
{     0,  1355, 5*16+ 7, 0x10+NORM },	/* 1354 */
{     0,  1356, 4*16+ 8, 0x0 +NORM },	/* 1355 */
{     0,     0, 4*16+ 3, 0x0 +FOLL },	/* 1356 */
{  1347,  1358, 5*16+ 7, 0x0 +NORM },	/* 1357 extend then push*/
{     0,  1365, 6*16+ 7, 0x10+NORM },	/* 1358 */
{     0,  1360, 6*16+ 6, 0x0 +NORM },	/* 1359 */
{     0,  1361, 3*16+ 5, 0x10+NORM },	/* 1360 */
{     0,  1362, 3*16+ 7, 0x0 +NORM },	/* 1361 */
{     0,  1363, 4*16+ 3, 0x10+NORM },	/* 1362 */
{     0,  1364, 7*16+ 6, 0x0 +NORM },	/* 1363 */
{     0,     0, 6*16+ 8, 0x10+NORM },	/* 1364 */
{  1359,  1366, 3*16+ 3, 0x0 +NORM },	/* 1365 takes corner*/
{     0,  1367, 3*16+ 5, 0x10+NORM },	/* 1366 */
{     0,  1368, 3*16+ 7, 0x0 +NORM },	/* 1367 */
{     0,  1369, 4*16+ 3, 0x10+NORM },	/* 1368 */
{     0,  1370, 4*16+ 2, 0x0 +NORM },	/* 1369 */
{     0,  1371, 2*16+ 6, 0x10+NORM },	/* 1370 */
{     0,  1372, 2*16+ 2, 0x0 +NORM },	/* 1371 */
{     0,     0, 2*16+ 7, 0x10+NORM },	/* 1372 */
{  1357,  1374, 3*16+ 5, 0x0 +NORM },	/* 1373 extend then push into corner*/
{     0,  1375, 3*16+ 4, 0x10+NORM },	/* 1374 */
{     0,  1376, 7*16+ 4, 0x0 +NORM },	/* 1375 */
{     0,  1377, 5*16+ 3, 0x10+NORM },	/* 1376 */
{     0,  1378, 5*16+ 2, 0x0 +NORM },	/* 1377 */
{     0,  1379, 6*16+ 4, 0x10+NORM },	/* 1378 */
{     0,  1380, 6*16+ 2, 0x0 +NORM },	/* 1379 */
{     0,  1381, 4*16+ 2, 0x10+NORM },	/* 1380 */
{     0,  1382, 7*16+ 6, 0x0 +NORM },	/* 1381 */
{     0,  1383, 6*16+ 7, 0x10+NORM },	/* 1382 */
{     0,  1385, 3*16+ 9, 0x0 +NORM },	/* 1383 */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 1384 */
{  1384,     0, 3*16+ 2, 0x0 +NORM },	/* 1385 */
{  1327,  1387, 3*16+ 5, 0x10+NORM },	/* 1386 attach then block*/
{     0,  1388, 5*16+ 6, 0x0 +NORM },	/* 1387 */
{     0,  1389, 4*16+ 5, 0x10+NORM },	/* 1388 */
{     0,  1390, 3*16+ 7, 0x0 +NORM },	/* 1389 */
{     0,  1391, 6*16+ 4, 0x10+NORM },	/* 1390 */
{     0,  1392, 7*16+ 4, 0x0 +NORM },	/* 1391 */
{     0,  1393, 6*16+ 5, 0x10+NORM },	/* 1392 */
{     0,  1394, 5*16+ 2, 0x0 +NORM },	/* 1393 */
{     0,  1395, 4*16+ 2, 0x10+NORM },	/* 1394 */
{     0,  1396, 4*16+ 3, 0x0 +NORM },	/* 1395 */
{     0,  1397, 3*16+ 3, 0x10+NORM },	/* 1396 */
{     0,  1398, 5*16+ 3, 0x0 +NORM },	/* 1397 */
{     0,     0, 3*16+ 2, 0x10+FOLL },	/* 1398 */
{  1325,  1400, 5*16+ 5, 0x10+NORM },	/* 1399 diagonal play in center */
{     0,  1401, 3*16+ 3, 0x0 +NORM },	/* 1400 */
{     0,  1402, 4*16+ 3, 0x10+NORM },	/* 1401 */
{     0,  1403, 3*16+ 4, 0x0 +NORM },	/* 1402 */
{     0,  1404, 5*16+ 7, 0x10+NORM },	/* 1403 */
{     0,     0, 4*16+ 8, 0x0 +NORM },	/* 1404 */
{     0,    46, 5*16+ 3, 0x10+NORM },	/* 1405 diagonal attach from back when*/
{  1405,  1408, 3*16+ 6, 0x40+NORM },	/* 1406 */
{     0,  1326, 4*16+ 6, 0x10+NORM },	/* 1407 */
{  1407,  1409, 3*16+ 3, 0x10+NORM },	/* 1408 drop back to 3-3 point */
{     0,  1410, 5*16+ 5, 0x0 +NORM },	/* 1409 */
{     0,  1411, 5*16+ 4, 0x10+NORM },	/* 1410 */
{     0,  1412, 6*16+ 4, 0x0 +NORM },	/* 1411 */
{     0,  1413, 6*16+ 5, 0x10+NORM },	/* 1412 */
{     0,  1414, 5*16+ 6, 0x0 +NORM },	/* 1413 */
{     0,     0, 7*16+ 5, 0x10+NORM },	/* 1414 */
{    63,  1416, 4*16+ 6, 0x0 +NORM },	/* 1415 double kakari with extension already.  section 29, other 4-4 double kakar*/
{     0,  1428, 5*16+ 6, 0x10+NORM },	/* 1416 */
{     0,  1418, 5*16+ 7, 0x0 +NORM },	/* 1417 */
{     0,  1421, 5*16+ 5, 0x10+NORM },	/* 1418 */
{     0,  1420, 4*16+ 8, 0x0 +NORM },	/* 1419 */
{     0,     0, 4*16+ 3, 0x10+NORM },	/* 1420 */
{  1419,  1422, 3*16+ 3, 0x0 +NORM },	/* 1421 variation */
{     0,  1423, 4*16+ 3, 0x10+NORM },	/* 1422 */
{     0,  1424, 3*16+ 4, 0x0 +NORM },	/* 1423 */
{     0,  1425, 3*16+ 2, 0x10+NORM },	/* 1424 */
{     0,  1426, 4*16+ 5, 0x0 +NORM },	/* 1425 */
{     0,  1427, 6*16+ 4, 0x10+NORM },	/* 1426 */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 1427 */
{  1417,  1436, 3*16+ 3, 0x0 +NORM },	/* 1428 another variation*/
{     0,  1430, 4*16+ 3, 0x10+NORM },	/* 1429 */
{     0,  1431, 3*16+ 4, 0x0 +NORM },	/* 1430 */
{     0,  1432, 4*16+ 5, 0x10+NORM },	/* 1431 */
{     0,  1433, 3*16+ 5, 0x0 +NORM },	/* 1432 */
{     0,  1434, 4*16+ 7, 0x10+NORM },	/* 1433 */
{     0,  1435, 3*16+ 6, 0x0 +NORM },	/* 1434 */
{     0,     0, 6*16+ 5, 0x10+NORM },	/* 1435 */
{  1429,  1437, 4*16+ 7, 0x10+NORM },	/* 1436 another variation */
{     0,  1438, 4*16+ 3, 0x0 +NORM },	/* 1437 */
{     0,     0, 3*16+ 6, 0x10+NORM },	/* 1438 */
{  1323,  1455, 4*16+ 6, 0x0 +NORM },	/* 1439 double approach, no side stones */
{     0,  1441, 6*16+ 4, 0x10+NORM },	/* 1440 */
{     0,  1442, 7*16+ 4, 0x0 +NORM },	/* 1441 */
{     0,  1443, 6*16+ 5, 0x10+NORM },	/* 1442 */
{     0,  1444, 5*16+ 3, 0x0 +NORM },	/* 1443 */
{     0,  1445, 4*16+ 5, 0x10+NORM },	/* 1444 */
{     0,  1446, 3*16+ 2, 0x0 +NORM },	/* 1445 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1446 */
{  1285,  1448, 6*16+ 4, 0x0 +NORM },	/* 1447 double approach side stone other side */
{     0,  1449, 4*16+ 6, 0x10+NORM },	/* 1448 */
{     0,  1450, 4*16+ 7, 0x0 +NORM },	/* 1449 */
{     0,  1451, 5*16+ 7, 0x10+NORM },	/* 1450 */
{     0,  1452, 3*16+ 5, 0x0 +NORM },	/* 1451 */
{     0,  1453, 5*16+ 4, 0x10+NORM },	/* 1452 */
{     0,  1454, 3*16+ 2, 0x0 +NORM },	/* 1453 */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 1454 */
{  1440,  1464, 5*16+ 3, 0x10+COMP },	/* 1455 diagonal tsuke */
{     0,  1457, 6*16+ 4, 0x0 +NORM },	/* 1456 */
{     0,  1458, 5*16+ 5, 0x10+NORM },	/* 1457 */
{     0,  1459, 5*16+ 2, 0x0 +NORM },	/* 1458 */
{     0,  1460, 4*16+ 2, 0x10+NORM },	/* 1459 */
{     0,  1461, 5*16+ 4, 0x0 +NORM },	/* 1460 */
{     0,  1462, 4*16+ 5, 0x10+NORM },	/* 1461 */
{     0,  1463, 4*16+ 3, 0x0 +NORM },	/* 1462 */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 1463 */
{  1456,  1465, 5*16+ 5, 0x0 +NORM },	/* 1464 */
{     0,     0, 6*16+ 2, 0x10+NORM },	/* 1465 */
{  1149,  1474, 4*16+ 6, 0x40+NORM },	/* 1466 */
{     0,  1468, 5*16+ 5, 0x10+NORM },	/* 1467 */
{     0,  1469, 3*16+ 3, 0x0 +NORM },	/* 1468 */
{     0,  1470, 3*16+ 4, 0x10+NORM },	/* 1469 */
{     0,  1471, 4*16+ 3, 0x0 +NORM },	/* 1470 */
{     0,  1472, 5*16+ 6, 0x10+NORM },	/* 1471 */
{     0,  1473, 5*16+ 4, 0x0 +NORM },	/* 1472 */
{     0,     0, 4*16+ 7, 0x10+NORM },	/* 1473 */
{  1467,  1475, 3*16+ 4, 0x10+NORM },	/* 1474 alternative*/
{     0,  1476, 6*16+ 6, 0x0 +NORM },	/* 1475 */
{     0,     0, 2*16+ 6, 0x10+NORM },	/* 1476 */
{     0,     0, 4*16+ 3, 0x10+SHIM },	/* 1477 make a shimari.  chapter 6 3-5 point joseki */
{  1477,  2220, 4*16+ 3, 0x20+NORM },	/* 1478 */
{  1478,  1488, 3*16+ 3, 0x0 +NORM },	/* 1479 kakari ar 3-3 point */
{    36,  1484, 3*16+10, 0x0 +IGNR },	/* 1480 just make a long extension */
{  1480,  1484, 3*16+ 9, 0x0 +IGNR },	/* 1481 */
{  1481,  1484, 3*16+11, 0x0 +NORM },	/* 1482 */
{     0,     0, 5*16+ 4, 0x10+NORM },	/* 1483 */
{  1483,  1485, 4*16+ 7, 0x10+COMP },	/* 1484 */
{     0,  1486, 5*16+ 5, 0x0 +NORM },	/* 1485 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1486 */
{  1479,     0, 5*16+ 4, 0x10+SHIM },	/* 1487 */
{     0,  1489, 5*16+ 4, 0x10+NORM },	/* 1488 section 30 3-3 invasion after 3-5 */
{     0,  1507, 5*16+ 3, 0x0 +NORM },	/* 1489 */
{     0,  1491, 6*16+ 4, 0x10+NORM },	/* 1490 */
{     0,  1492, 6*16+ 3, 0x0 +NORM },	/* 1491 */
{     0,  1504, 7*16+ 4, 0x10+NORM },	/* 1492 */
{     0,  1496, 2*16+ 5, 0x0 +NORM },	/* 1493 */
{     0,  1495, 2*16+ 6, 0x10+BAD  },	/* 1494 */
{     0,     0, 3*16+ 6, 0x0 +NORM },	/* 1495 */
{  1494,  1497, 3*16+ 6, 0x10+NORM },	/* 1496 */
{     0,  1498, 2*16+ 6, 0x0 +NORM },	/* 1497 */
{     0,  1499, 3*16+ 7, 0x10+NORM },	/* 1498 */
{     0,  1500, 4*16+ 4, 0x0 +NORM },	/* 1499 */
{     0,  1501, 4*16+ 5, 0x10+NORM },	/* 1500 */
{     0,     0, 3*16+ 4, 0x0 +NORM },	/* 1501 */
{  1493,     0, 7*16+ 3, 0x10+NORM },	/* 1502 if black tenukis */
{  1502,     0, 4*16+ 3, 0x10+COMP },	/* 1503 another*/
{  1503,  1505, 2*16+ 3, 0x10+NORM },	/* 1504 */
{     0,  1506, 2*16+ 2, 0x0 +NORM },	/* 1505 */
{     0,     0, 2*16+ 4, 0x10+NORM },	/* 1506 */
{  1490,  1521, 6*16+ 3, 0x10+NORM },	/* 1507 hane after attach*/
{     0,  1509, 6*16+ 4, 0x0 +COMP },	/* 1508 depends on ladder */
{     0,  1510, 4*16+ 3, 0x10+NORM },	/* 1509 */
{     0,  1514, 5*16+ 2, 0x0 +NORM },	/* 1510 */
{     0,  1512, 5*16+ 5, 0x10+NORM },	/* 1511 */
{     0,  1513, 4*16+ 2, 0x0 +NORM },	/* 1512 */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 1513 */
{  1511,  1515, 4*16+ 4, 0x10+NORM },	/* 1514 when ladder doesn't work*/
{     0,  1516, 4*16+ 2, 0x0 +NORM },	/* 1515 */
{     0,  1517, 7*16+ 3, 0x10+NORM },	/* 1516 */
{     0,  1518, 3*16+ 4, 0x0 +NORM },	/* 1517 */
{     0,  1519, 4*16+ 5, 0x10+NORM },	/* 1518 */
{     0,  1520, 6*16+ 5, 0x0 +NORM },	/* 1519 */
{     0,     0, 9*16+ 4, 0x10+NORM },	/* 1520 */
{  1508,  1522, 4*16+ 4, 0x0 +NORM },	/* 1521 variation for black if ladder bad*/
{     0,  1523, 4*16+ 5, 0x10+NORM },	/* 1522 */
{     0,  1524, 6*16+ 2, 0x0 +NORM },	/* 1523 */
{     0,  1525, 6*16+ 4, 0x10+NORM },	/* 1524 */
{     0,     0, 7*16+ 2, 0x0 +NORM },	/* 1525 */
{     0,     0, 5*16+ 4, 0x0 +NORM },	/* 1526 */
{     0,  1572, 5*16+ 3, 0x10+NORM },	/* 1527 */
{     0,  1529, 6*16+ 4, 0x0 +NORM },	/* 1528 */
{     0,  1563, 7*16+ 3, 0x10+NORM },	/* 1529 */
{     0,  1531, 7*16+ 4, 0x0 +NORM },	/* 1530 */
{     0,  1532, 8*16+ 3, 0x10+NORM },	/* 1531 */
{     0,  1533, 8*16+ 4, 0x0 +NORM },	/* 1532 */
{     0,  1534, 9*16+ 4, 0x10+NORM },	/* 1533 */
{     0,  1535, 9*16+ 5, 0x0 +NORM },	/* 1534 */
{     0,  1536,10*16+ 5, 0x10+NORM },	/* 1535 */
{     0,  1537, 6*16+ 3, 0x0 +NORM },	/* 1536 */
{     0,  1538, 6*16+ 2, 0x10+NORM },	/* 1537 */
{     0,  1539, 5*16+ 2, 0x0 +NORM },	/* 1538 */
{     0,  1540, 7*16+ 2, 0x10+NORM },	/* 1539 */
{     0,  1541, 3*16+ 3, 0x0 +NORM },	/* 1540 */
{     0,  1542, 9*16+ 6, 0x10+NORM },	/* 1541 */
{     0,  1543, 8*16+ 5, 0x0 +NORM },	/* 1542 */
{     0,  1544, 4*16+ 2, 0x10+NORM },	/* 1543 */
{     0,  1545,10*16+ 6, 0x0 +NORM },	/* 1544 */
{     0,     0,11*16+ 5, 0x10+NORM },	/* 1545 */
{  1530,  1547, 6*16+ 3, 0x0 +NORM },	/* 1546 another variation */
{     0,  1548, 6*16+ 2, 0x10+NORM },	/* 1547 */
{     0,  1549, 7*16+ 2, 0x0 +NORM },	/* 1548 */
{     0,  1550, 8*16+ 2, 0x10+NORM },	/* 1549 */
{     0,  1551, 7*16+ 4, 0x0 +NORM },	/* 1550 */
{     0,  1552, 8*16+ 3, 0x10+NORM },	/* 1551 */
{     0,  1553, 5*16+ 2, 0x0 +NORM },	/* 1552 */
{     0,  1554, 7*16+ 1, 0x10+NORM },	/* 1553 */
{     0,  1555, 3*16+ 3, 0x0 +NORM },	/* 1554 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 1555 */
{  1546,  1557, 3*16+10, 0x0 +NORM },	/* 1556 just extend down side */
{     0,  1558, 7*16+ 4, 0x10+NORM },	/* 1557 */
{     0,  1559, 7*16+ 5, 0x0 +NORM },	/* 1558 */
{     0,  1560, 8*16+ 5, 0x10+NORM },	/* 1559 */
{     0,  1561, 7*16+ 6, 0x0 +NORM },	/* 1560 */
{     0,  1562, 8*16+ 6, 0x10+NORM },	/* 1561 */
{     0,     0, 7*16+ 7, 0x0 +NORM },	/* 1562 */
{  1556,  1568, 6*16+ 3, 0x10+BAD  },	/* 1563 push one more time*/
{     0,  1593, 7*16+ 4, 0x0 +URGT },	/* 1564 */
{     0,  1566, 3*16+10, 0x10+NORM },	/* 1565 */
{     0,  1567, 7*16+ 3, 0x0 +NORM },	/* 1566 */
{     0,     0, 2*16+ 4, 0x10+URGT },	/* 1567 */
{  1564,  1569, 3*16+10, 0x0 +BAD  },	/* 1568 bad extension now */
{     0,  1570, 7*16+ 4, 0x10+NORM },	/* 1569 */
{     0,  1571, 7*16+ 5, 0x0 +NORM },	/* 1570 */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 1571 */
{  1528,  1580, 7*16+ 4, 0x0 +NORM },	/* 1572 another main variation */
{     0,  1574, 6*16+ 4, 0x10+NORM },	/* 1573 */
{     0,  1575, 6*16+ 5, 0x0 +NORM },	/* 1574 */
{     0,  1576, 6*16+ 3, 0x10+NORM },	/* 1575 */
{     0,  1577, 7*16+ 5, 0x0 +NORM },	/* 1576 */
{     0,  1578, 3*16+ 4, 0x10+NORM },	/* 1577 */
{     0,  1579, 2*16+ 5, 0x0 +NORM },	/* 1578 */
{     0,     0, 8*16+ 3, 0x10+NORM },	/* 1579 */
{  1573,  1581, 4*16+ 4, 0x10+BAD  },	/* 1580 a bad response */
{     0,  1582, 4*16+ 5, 0x0 +NORM },	/* 1581 */
{     0,  1583, 6*16+ 4, 0x10+NORM },	/* 1582 */
{     0,  1584, 6*16+ 5, 0x0 +NORM },	/* 1583 */
{     0,  1585, 5*16+ 5, 0x10+NORM },	/* 1584 */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 1585 */
{  1482,  1612, 6*16+ 4, 0x0 +COMP },	/* 1586 section 32 Taisha */
{     0,     0, 4*16+ 5, 0x0 +NORM },	/* 1587 if tenuki*/
{  1587,  1589, 5*16+ 3, 0x10+NORM },	/* 1588 simple variation */
{     0,     0, 5*16+ 5, 0x0 +NORM },	/* 1589 */
{  1588,  1591, 6*16+ 3, 0x10+NORM },	/* 1590 1st tsuke*/
{     0,  1592, 5*16+ 4, 0x0 +NORM },	/* 1591 */
{     0,  1568, 5*16+ 3, 0x10+NORM },	/* 1592 */
{  1565,     0, 8*16+ 3, 0x10+NORM },	/* 1593 */
{  1590,  1603, 5*16+ 4, 0x10+NORM },	/* 1594 2nd tsuke*/
{     0,  1596, 5*16+ 5, 0x0 +NORM },	/* 1595 */
{     0,  1597, 6*16+ 3, 0x10+NORM },	/* 1596 */
{     0,  1598, 7*16+ 4, 0x0 +NORM },	/* 1597 */
{     0,  1599, 7*16+ 3, 0x10+NORM },	/* 1598 */
{     0,  1600, 8*16+ 4, 0x0 +NORM },	/* 1599 */
{     0,  1601, 8*16+ 3, 0x10+NORM },	/* 1600 */
{     0,  1602, 9*16+ 4, 0x0 +NORM },	/* 1601 */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 1602 */
{  1595,  1604, 4*16+ 5, 0x10+NORM },	/* 1603 another variation*/
{     0,  1605, 4*16+ 6, 0x0 +NORM },	/* 1604 */
{     0,  1606, 4*16+ 4, 0x10+NORM },	/* 1605 */
{     0,  1607, 6*16+ 5, 0x0 +NORM },	/* 1606 */
{     0,  1608, 3*16+ 6, 0x10+NORM },	/* 1607 */
{     0,  1609, 3*16+ 7, 0x0 +NORM },	/* 1608 */
{     0,  1610, 2*16+ 6, 0x10+NORM },	/* 1609 */
{     0,  1611, 4*16+ 7, 0x0 +NORM },	/* 1610 */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 1611 */
{  1594,  1613, 4*16+ 5, 0x10+COMP },	/* 1612 real taisha tsuke*/
{     0,  1614, 4*16+ 4, 0x0 +URGT },	/* 1613 */
{     0,  1615, 5*16+ 4, 0x10+URGT },	/* 1614 */
{     0,  1624, 3*16+ 4, 0x0 +NORM },	/* 1615 */
{     0,  1617, 5*16+ 5, 0x10+NORM },	/* 1616 connect outside (depends on ladder) */
{     0,  1618, 5*16+ 3, 0x0 +NORM },	/* 1617 */
{     0,  1619, 6*16+ 3, 0x10+NORM },	/* 1618 */
{     0,  1620, 5*16+ 2, 0x0 +NORM },	/* 1619 */
{     0,  1621, 7*16+ 4, 0x10+NORM },	/* 1620 */
{     0,  1622, 3*16+ 7, 0x0 +NORM },	/* 1621 */
{     0,  1623, 6*16+ 5, 0x10+NORM },	/* 1622 */
{     0,     0, 4*16+ 2, 0x0 +NORM },	/* 1623 */
{  1616,  1625, 5*16+ 3, 0x10+NORM },	/* 1624 connect on inside*/
{     0,  1636, 5*16+ 5, 0x0 +NORM },	/* 1625 */
{     0,  1627, 6*16+ 5, 0x10+NORM },	/* 1626 */
{     0,  1628, 4*16+ 6, 0x0 +NORM },	/* 1627 */
{     0,  1630, 5*16+ 6, 0x10+NORM },	/* 1628 */
{     0,     0, 6*16+ 6, 0x0 +COMP },	/* 1629 needs big ko threats*/
{  1629,  1631, 5*16+ 7, 0x0 +NORM },	/* 1630 */
{     0,  1632, 4*16+ 5, 0x10+NORM },	/* 1631 */
{     0,  1633, 3*16+ 3, 0x0 +NORM },	/* 1632 */
{     0,  1634, 5*16+ 5, 0x10+NORM },	/* 1633 */
{     0,  1635, 4*16+ 7, 0x0 +NORM },	/* 1634 */
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 1635 */
{  1626,  1637, 4*16+ 6, 0x10+NORM },	/* 1636 pull out of  atari*/
{     0,  1638, 3*16+ 3, 0x0 +URGT },	/* 1637 */
{     0,  1639, 6*16+ 3, 0x10+NORM },	/* 1638 */
{     0,  1640, 6*16+ 5, 0x0 +NORM },	/* 1639 */
{     0,  1641, 8*16+ 3, 0x10+NORM },	/* 1640 */
{     0,  1642, 2*16+ 7, 0x0 +NORM },	/* 1641 */
{     0,  1643, 4*16+ 8, 0x10+NORM },	/* 1642 */
{     0,  1644, 8*16+ 5, 0x0 +NORM },	/* 1643 */
{     0,  1645,10*16+ 4, 0x10+NORM },	/* 1644 */
{     0,  1646, 7*16+ 8, 0x0 +NORM },	/* 1645 */
{     0,     0, 4*16+10, 0x10+NORM },	/* 1646 */
{  1586,  1648, 6*16+ 3, 0x0 +NORM },	/* 1647 section 33 one point pincer */
{     0,  1649, 5*16+ 4, 0x10+NORM },	/* 1648 */
{     0,  1650, 6*16+ 4, 0x0 +NORM },	/* 1649 */
{     0,  1679, 5*16+ 6, 0x10+NORM },	/* 1650 */
{     0,  1666, 6*16+ 5, 0x0 +NORM },	/* 1651 */
{     0,  1653, 5*16+ 5, 0x10+NORM },	/* 1652 */
{     0,  1654, 3*16+ 8, 0x0 +NORM },	/* 1653 */
{     0,  1658, 2*16+ 5, 0x10+NORM },	/* 1654 */
{     0,  1656, 2*16+ 6, 0x0 +NORM },	/* 1655 */
{     0,  1657, 2*16+ 4, 0x10+NORM },	/* 1656 */
{     0,     0, 3*16+11, 0x0 +NORM },	/* 1657 */
{  1655,  1659, 2*16+ 4, 0x0 +NORM },	/* 1658 inside hane*/
{     0,  1660, 3*16+ 6, 0x10+NORM },	/* 1659 */
{     0,  1661, 2*16+ 6, 0x0 +NORM },	/* 1660 */
{     0,  1662, 2*16+ 7, 0x10+NORM },	/* 1661 */
{     0,  1663, 1*16+ 5, 0x0 +NORM },	/* 1662 */
{     0,  1664, 3*16+ 7, 0x10+NORM },	/* 1663 */
{     0,  1665, 2*16+ 2, 0x0 +NORM },	/* 1664 */
{     0,     0, 7*16+ 7, 0x10+FOLL },	/* 1665 */
{  1652,  1667, 3*16+ 6, 0x10+NORM },	/* 1666 indirect cutting point defence*/
{     0,  1668, 2*16+ 6, 0x0 +NORM },	/* 1667 */
{     0,  1669, 3*16+ 7, 0x10+NORM },	/* 1668 */
{     0,  1670, 3*16+ 4, 0x0 +NORM },	/* 1669 */
{     0,  1671, 5*16+ 5, 0x10+NORM },	/* 1670 */
{     0,  1672, 2*16+ 7, 0x0 +NORM },	/* 1671 */
{     0,  1673, 3*16+ 9, 0x10+NORM },	/* 1672 */
{     0,  1674, 3*16+ 8, 0x0 +NORM },	/* 1673 */
{     0,  1675, 4*16+ 8, 0x10+NORM },	/* 1674 */
{     0,  1676, 2*16+ 8, 0x0 +NORM },	/* 1675 */
{     0,  1677, 4*16+10, 0x10+NORM },	/* 1676 */
{     0,  1678, 3*16+ 3, 0x0 +NORM },	/* 1677 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 1678 */
{  1651,  1680, 4*16+ 7, 0x0 +NORM },	/* 1679 newfangled move*/
{     0,  1681, 3*16+ 4, 0x10+NORM },	/* 1680 */
{     0,  1685, 5*16+ 7, 0x0 +NORM },	/* 1681 */
{     0,  1683, 6*16+ 6, 0x10+BAD  },	/* 1682 */
{     0,  1684, 4*16+ 5, 0x0 +NORM },	/* 1683 */
{     0,     0, 5*16+ 5, 0x10+NORM },	/* 1684 */
{  1682,  1686, 4*16+ 6, 0x10+NORM },	/* 1685 */
{     0,  1687, 6*16+ 6, 0x0 +NORM },	/* 1686 */
{     0,  1688, 3*16+ 6, 0x10+NORM },	/* 1687 */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 1688 */
{  1647,  1702, 7*16+ 3, 0x0 +NORM },	/* 1689 2 pt lowp incer.  section 34 2 pt pincers */
{     0,  1691, 5*16+ 4, 0x10+NORM },	/* 1690 */
{     0,  1692,10*16+ 3, 0x0 +NORM },	/* 1691 */
{     0,  1698, 3*16+ 8, 0x10+NORM },	/* 1692 */
{     0,  1694, 2*16+ 3, 0x0 +NORM },	/* 1693 */
{     0,  1695, 4*16+ 6, 0x10+NORM },	/* 1694 */
{     0,  1696, 3*16+ 6, 0x0 +NORM },	/* 1695 */
{     0,  1697, 4*16+ 7, 0x10+NORM },	/* 1696 */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 1697 */
{  1693,  1699, 4*16+ 6, 0x0 +COMP },	/* 1698 aggressive response*/
{     0,  1700, 6*16+ 5, 0x10+NORM },	/* 1699 */
{     0,  1701, 5*16+ 7, 0x0 +NORM },	/* 1700 */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 1701 */
{  1690,  1703, 6*16+ 3, 0x10+NORM },	/* 1702 other answer to pincer*/
{     0,  1704, 6*16+ 4, 0x0 +NORM },	/* 1703 */
{     0,  1705, 5*16+ 4, 0x10+NORM },	/* 1704 */
{     0,  1706, 6*16+ 2, 0x0 +NORM },	/* 1705 */
{     0,  1707, 5*16+ 3, 0x10+NORM },	/* 1706 */
{     0,  1708, 7*16+ 4, 0x0 +NORM },	/* 1707 */
{     0,  1709, 4*16+ 6, 0x10+NORM },	/* 1708 */
{     0,  1710, 3*16+ 7, 0x0 +NORM },	/* 1709 */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 1710 */
{  1689,  1712, 7*16+ 4, 0x0 +NORM },	/* 1711 2 pt high pincer*/
{     0,  1713, 5*16+ 4, 0x10+NORM },	/* 1712 */
{     0,  1714, 4*16+ 7, 0x0 +NORM },	/* 1713 */
{     0,  1715, 3*16+ 4, 0x10+NORM },	/* 1714 */
{     0,  1716, 4*16+ 5, 0x0 +NORM },	/* 1715 */
{     0,  1717, 2*16+ 4, 0x10+NORM },	/* 1716 */
{     0,  1718, 3*16+11, 0x0 +NORM },	/* 1717 */
{     0,     0, 7*16+ 6, 0x0 +FOLL },	/* 1718 */
{  1487,  1720, 5*16+ 4, 0x0 +NORM },	/* 1719 section 35 high kakari to 3-5 */
{     0,  1721, 4*16+ 3, 0x10+NORM },	/* 1720 */
{     0,  1752, 5*16+ 3, 0x0 +NORM },	/* 1721 */
{     0,  1723, 4*16+ 2, 0x10+NORM },	/* 1722 */
{     0,  1733, 9*16+ 3, 0x0 +NORM },	/* 1723 */
{     0,  1726, 5*16+ 2, 0x0 +FOLL },	/* 1724 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 1725 */
{  1725,  1727, 3*16+ 4, 0x0 +NORM },	/* 1726 */
{     0,  1728, 4*16+ 4, 0x10+NORM },	/* 1727 */
{     0,  1729, 4*16+ 5, 0x0 +NORM },	/* 1728 */
{     0,  1730, 2*16+ 4, 0x10+NORM },	/* 1729 */
{     0,  1731, 3*16+ 6, 0x0 +NORM },	/* 1730 */
{     0,  1732, 2*16+ 5, 0x10+NORM },	/* 1731 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 1732 */
{  1724,  1734, 5*16+ 2, 0x10+NORM },	/* 1733 response to extension*/
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 1734 */
{  1719,  1738,10*16+ 4, 0x0 +NORM },	/* 1735 black gets move in on side from other corn*/
{  1735,  1738,11*16+ 4, 0x0 +NORM },	/* 1736 */
{     0,     0, 4*16+ 3, 0x10+SHIM },	/* 1737 make a shimari of course*/
{  1737,  1739, 5*16+ 4, 0x0 +NORM },	/* 1738 */
{     0,  1740, 4*16+ 3, 0x10+NORM },	/* 1739 */
{     0,  1741, 5*16+ 3, 0x0 +NORM },	/* 1740 */
{     0,  1747, 4*16+ 2, 0x10+NORM },	/* 1741 */
{     0,  1743, 5*16+ 6, 0x0 +NORM },	/* 1742 */
{     0,  1744, 4*16+ 5, 0x10+NORM },	/* 1743 */
{     0,  1745, 5*16+ 5, 0x0 +NORM },	/* 1744 */
{     0,  1746, 5*16+ 2, 0x10+NORM },	/* 1745 */
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 1746 */
{  1742,  1748, 5*16+ 7, 0x10+NORM },	/* 1747 longer extendsion in center*/
{     0,  1749, 5*16+ 2, 0x0 +NORM },	/* 1748 */
{     0,  1751, 7*16+ 3, 0x10+NORM },	/* 1749 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 1750 */
{  1750,     0, 3*16+ 8, 0x10+NORM },	/* 1751 */
{  1722,  1753, 4*16+ 4, 0x10+NORM },	/* 1752 different major response */
{     0,  1756, 5*16+ 5, 0x0 +NORM },	/* 1753 */
{     0,  1755, 4*16+ 6, 0x10+NORM },	/* 1754 */
{     0,     0,10*16+ 3, 0x0 +NORM },	/* 1755 */
{  1754,  1757, 9*16+ 4, 0x10+NORM },	/* 1756 */
{     0,  1758, 3*16+ 6, 0x0 +NORM },	/* 1757 */
{     0,  1759, 2*16+ 6, 0x10+NORM },	/* 1758 */
{     0,  1760, 4*16+ 5, 0x0 +NORM },	/* 1759 */
{     0,  1761, 2*16+ 4, 0x10+NORM },	/* 1760 */
{     0,  1762, 3*16+ 7, 0x0 +NORM },	/* 1761 */
{     0,  1763, 2*16+ 7, 0x10+NORM },	/* 1762 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 1763 */
{     0,     0, 4*16+ 3, 0x10+SHIM },	/* 1764 chapter 7 4-5 point joseki */
{  1764,     0, 3*16+ 3, 0x0 +NORM },	/* 1765 3-3 invasion*/
{     0,  1489, 5*16+ 3, 0x10+NORM },	/* 1766 */
{  1765,  1768, 3*16+10, 0x10+NORM },	/* 1767 if immediate extension*/
{     0,     0, 3*16+ 3, 0x0 +NORM },	/* 1768 */
{  1767,  1839, 4*16+ 3, 0x20+NORM },	/* 1769 */
{   607,  1802, 6*16+ 4, 0x0 +NORM },	/* 1770 knights move after kakari*/
{     0,  1779, 3*16+ 5, 0x10+NORM },	/* 1771 */
{     0,  1773, 3*16+ 6, 0x0 +NORM },	/* 1772 */
{     0,  1795, 3*16+ 4, 0x10+NORM },	/* 1773 */
{     0,  1777, 4*16+ 6, 0x0 +NORM },	/* 1774 */
{  1772,     0, 3*16+ 4, 0x0 +COMP },	/* 1775 leads to complicated variation*/
{     0,     0, 6*16+ 2, 0x10+NORM },	/* 1776 */
{  1776,  1778, 5*16+ 2, 0x0 +NORM },	/* 1777 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 1778 */
{  1775,  1780, 4*16+ 4, 0x0 +NORM },	/* 1779 another major variation*/
{     0,  1781, 3*16+ 4, 0x10+NORM },	/* 1780 */
{     0,  1782, 3*16+ 3, 0x0 +NORM },	/* 1781 */
{     0,  1783, 3*16+ 2, 0x10+NORM },	/* 1782 */
{     0,  1784, 2*16+ 3, 0x0 +NORM },	/* 1783 */
{     0,  1785, 2*16+ 2, 0x10+NORM },	/* 1784 */
{     0,  1786, 5*16+ 3, 0x0 +NORM },	/* 1785 */
{     0,  1787, 4*16+ 2, 0x10+NORM },	/* 1786 */
{     0,  1788, 3*16+ 6, 0x0 +NORM },	/* 1787 */
{     0,  1789, 2*16+ 4, 0x10+NORM },	/* 1788 */
{     0,  1790, 2*16+ 5, 0x0 +NORM },	/* 1789 */
{     0,  1791, 1*16+ 3, 0x10+NORM },	/* 1790 */
{     0,  1792, 4*16+ 7, 0x0 +NORM },	/* 1791 */
{     0,  1793, 5*16+ 6, 0x10+NORM },	/* 1792 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 1793 */
{  1774,     0, 4*16+ 7, 0x0 +NORM },	/* 1794 another major variation*/
{  1794,  1796, 2*16+ 6, 0x0 +NORM },	/* 1795 and another*/
{     0,  1797, 6*16+ 2, 0x10+NORM },	/* 1796 */
{     0,     0, 7*16+ 6, 0x0 +NORM },	/* 1797 */
{  1771,  1799, 5*16+ 2, 0x0 +NORM },	/* 1798 attack after tenuki*/
{     0,  1800, 2*16+ 5, 0x10+NORM },	/* 1799 */
{     0,  1801, 3*16+ 4, 0x0 +NORM },	/* 1800 */
{     0,     0, 3*16+ 8, 0x10+NORM },	/* 1801 */
{  1798,  1803, 3*16+ 3, 0x0 +NORM },	/* 1802 another attack*/
{     0,  1804, 5*16+ 2, 0x10+NORM },	/* 1803 */
{     0,  1805, 4*16+ 4, 0x0 +NORM },	/* 1804 */
{     0,     0, 7*16+ 3, 0x10+NORM },	/* 1805 */
{  1770,  1807, 5*16+ 3, 0x0 +NORM },	/* 1806 section 37 contact on outside */
{     0,  1808, 5*16+ 2, 0x10+NORM },	/* 1807 */
{     0,  1809, 5*16+ 4, 0x0 +NORM },	/* 1808 */
{     0,  1827, 6*16+ 3, 0x10+NORM },	/* 1809 */
{     0,  1811, 6*16+ 2, 0x0 +NORM },	/* 1810 */
{     0,  1823, 7*16+ 2, 0x10+NORM },	/* 1811 */
{     0,  1813, 4*16+ 2, 0x0 +NORM },	/* 1812 */
{     0,  1814, 6*16+ 1, 0x10+NORM },	/* 1813 */
{     0,  1821, 3*16+ 3, 0x0 +NORM },	/* 1814 */
{     0,  1819, 6*16+ 4, 0x10+NORM },	/* 1815 */
{     0,  1817, 6*16+ 5, 0x0 +NORM },	/* 1816 */
{     0,  1818, 7*16+ 5, 0x10+NORM },	/* 1817 */
{     0,  1820, 6*16+ 6, 0x0 +NORM },	/* 1818 */
{  1816,     0, 5*16+ 6, 0x10+NORM },	/* 1819 if no hane*/
{     0,     0, 3*16+ 2, 0x10+FOLL },	/* 1820 followup*/
{  1815,  1822, 7*16+ 4, 0x0 +NORM },	/* 1821 if doesn't push up*/
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 1822 */
{  1812,  1825, 6*16+ 4, 0x0 +NORM },	/* 1823 variation*/
{     0,     0, 7*16+ 3, 0x10+BAD  },	/* 1824 */
{  1824,  1826, 6*16+ 1, 0x10+NORM },	/* 1825 */
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 1826 */
{  1810,  1828, 4*16+ 2, 0x0 +NORM },	/* 1827 cut on inside*/
{     0,  1829, 3*16+ 2, 0x10+NORM },	/* 1828 */
{     0,  1830, 6*16+ 2, 0x0 +NORM },	/* 1829 */
{     0,  1831, 4*16+ 1, 0x10+NORM },	/* 1830 */
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 1831 */
{  1806,  1833, 4*16+ 8, 0x10+NORM },	/* 1832 2 pt high pincer (why is it here!!!???) */
{     0,  1834, 5*16+ 3, 0x0 +NORM },	/* 1833 */
{     0,  1835, 5*16+ 2, 0x10+NORM },	/* 1834 */
{     0,  1836, 5*16+ 4, 0x0 +NORM },	/* 1835 */
{     0,  1837, 6*16+ 2, 0x10+NORM },	/* 1836 */
{     0,  1838, 3*16+ 3, 0x0 +NORM },	/* 1837 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 1838 */
{  1832,  1879, 3*16+ 3, 0x0 +NORM },	/* 1839 section 38 inside tsuke */
{     0,  1841, 3*16+ 2, 0x10+NORM },	/* 1840 */
{     0,  1842, 3*16+ 4, 0x0 +NORM },	/* 1841 */
{     0,  1843, 5*16+ 2, 0x10+NORM },	/* 1842 */
{     0,  1867, 3*16+ 9, 0x0 +NORM },	/* 1843 */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 1844 */
{  1844,     0, 7*16+ 4, 0x10+NORM },	/* 1845 */
{  1845,  1847, 5*16+ 5, 0x10+NORM },	/* 1846 */
{     0,  1848, 5*16+ 6, 0x0 +NORM },	/* 1847 */
{     0,  1849, 6*16+ 5, 0x10+NORM },	/* 1848 */
{     0,  1850, 6*16+ 3, 0x0 +NORM },	/* 1849 */
{     0,  1851, 4*16+ 4, 0x10+NORM },	/* 1850 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 1851 */
{  1846,  1855, 7*16+ 4, 0x0 +NORM },	/* 1852 after tenuki*/
{     0,  1854, 6*16+ 3, 0x10+NORM },	/* 1853 */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 1854 */
{  1853,  1856, 5*16+ 5, 0x10+COMP },	/* 1855 fight back*/
{     0,  1857, 5*16+ 4, 0x0 +NORM },	/* 1856 */
{     0,  1858, 4*16+ 4, 0x10+NORM },	/* 1857 */
{     0,  1859, 3*16+ 5, 0x0 +NORM },	/* 1858 */
{     0,  1860, 5*16+ 6, 0x10+NORM },	/* 1859 */
{     0,  1861, 5*16+ 3, 0x0 +NORM },	/* 1860 */
{     0,  1862, 4*16+ 2, 0x10+NORM },	/* 1861 */
{     0,  1863, 7*16+ 3, 0x0 +NORM },	/* 1862 */
{     0,  1864, 2*16+ 3, 0x10+NORM },	/* 1863 */
{     0,  1865, 2*16+ 4, 0x0 +NORM },	/* 1864 */
{     0,  1866, 1*16+ 3, 0x10+NORM },	/* 1865 */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 1866 */
{  1852,  1868, 6*16+ 3, 0x0 +NORM },	/* 1867 variation after tenuki*/
{     0,  1869, 5*16+ 4, 0x10+NORM },	/* 1868 */
{     0,  1870, 7*16+ 4, 0x0 +NORM },	/* 1869 */
{     0,  1871, 5*16+ 5, 0x10+NORM },	/* 1870 */
{     0,     0, 7*16+ 6, 0x0 +NORM },	/* 1871 */
{  1840,  1873, 3*16+10, 0x10+NORM },	/* 1872 pincer after tsuke*/
{     0,  1874, 4*16+ 4, 0x0 +NORM },	/* 1873 */
{     0,  1875, 5*16+ 3, 0x10+NORM },	/* 1874 */
{     0,  1876, 3*16+ 2, 0x0 +NORM },	/* 1875 */
{     0,  1877, 8*16+ 3, 0x10+NORM },	/* 1876 */
{     0,  1878, 3*16+ 8, 0x0 +NORM },	/* 1877 */
{     0,     0, 5*16+10, 0x10+NORM },	/* 1878 */
{  1872,  1880, 3*16+ 7, 0x10+NORM },	/* 1879 closer pincer after tsuke*/
{     0,  1881, 4*16+ 4, 0x0 +NORM },	/* 1880 */
{     0,  1884, 5*16+ 3, 0x10+NORM },	/* 1881 */
{     0,  1883, 2*16+ 4, 0x0 +NORM },	/* 1882 */
{     0,     0, 8*16+ 3, 0x10+NORM },	/* 1883 */
{  1882,  1885, 3*16+ 2, 0x0 +BAD  },	/* 1884 mistake*/
{     0,  1886, 2*16+ 5, 0x10+NORM },	/* 1885 */
{     0,  1887, 2*16+ 4, 0x0 +NORM },	/* 1886 */
{     0,  1888, 3*16+ 5, 0x10+NORM },	/* 1887 */
{     0,     0, 3*16+ 4, 0x0 +NORM },	/* 1888 this concludes all of the joseki variations from 38 basic joseki! */
{  1406,     0, 4*16+ 3, 0x10+BAD  },	/* 1889 don't close corner in 8 or 9 handicaps*/
{     0,  1891, 4*16+10, 0x10+IGNR },	/* 1890 */
{     0,  1439, 6*16+ 3, 0x0 +NORM },	/* 1891 */
{  1227,  1893, 3*16+10, 0x10+IGNR },	/* 1892 knight approach after extension */
{     0,  1439, 6*16+ 3, 0x0 +NORM },	/* 1893 */
{   767,     0, 3*16+ 4, 0x0 +NORM },	/* 1894 3-3 invasion with tenuki */
{     0,     0, 6*16+ 3, 0x10+NORM },	/* 1895 joseki library bug fix.  block in 3-3 invasion final connection */
{   487,  2547, 5*16+ 4, 0x0 +NORM },	/* 1896 Knigh pressing move from dictionary of basic joseki: 3-4, 1 pt low pincer*/
{     0,  1898, 4*16+ 4, 0x10+NORM },	/* 1897 */
{     0,  1899, 4*16+ 5, 0x0 +NORM },	/* 1898 */
{     0,  1900, 5*16+ 5, 0x10+NORM },	/* 1899 */
{     0,  1901, 5*16+ 6, 0x0 +NORM },	/* 1900 */
{     0,  1902, 6*16+ 5, 0x10+NORM },	/* 1901 */
{     0,  2461, 4*16+ 7, 0x0 +NORM },	/* 1902 */
{     0,  1904, 6*16+ 4, 0x10+NORM },	/* 1903 */
{     0,  2423, 3*16+ 8, 0x0 +NORM },	/* 1904 */
{     0,  1912, 6*16+ 6, 0x10+FOLL },	/* 1905 */
{     0,  1907, 6*16+ 7, 0x0 +URGT },	/* 1906 */
{     0,  1908, 7*16+ 7, 0x10+NORM },	/* 1907 */
{     0,  1909, 6*16+ 8, 0x0 +NORM },	/* 1908 */
{     0,  1910, 7*16+ 8, 0x10+NORM },	/* 1909 */
{     0,  1911, 6*16+ 9, 0x0 +NORM },	/* 1910 */
{     0,     0,11*16+ 3, 0x10+NORM },	/* 1911 */
{  1906,  2419, 5*16+ 7, 0x10+NORM },	/* 1912 */
{     0,  1916,10*16+ 3, 0x10+NORM },	/* 1913 Ishida 1-1 */
{     0,  1915, 3*16+11, 0x0 +NORM },	/* 1914  */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 1915  */
{  1914,  1917, 3*16+ 4, 0x10+NORM },	/* 1916 More Ishida 1-1 */
{     0,  1918, 4*16+ 5, 0x0 +NORM },	/* 1917  */
{     0,     0, 2*16+ 4, 0x10+NORM },	/* 1918  */
{  1913,  1923, 3*16+ 9, 0x10+NORM },	/* 1919 More 1-1 */
{     0,  1921, 4*16+12, 0x0 +NORM },	/* 1920  */
{     0,  1922, 5*16+ 9, 0x10+NORM },	/* 1921  */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 1922  */
{  1920,  1924, 2*16+ 3, 0x0 +NORM },	/* 1923 More 1-1 */
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 1924  */
{   278,  1931, 3*16+ 8, 0x0 +NORM },	/* 1925 I 1-2 2 space extension */
{     0,  1927, 3*16+ 4, 0x10+NORM },	/* 1926  */
{     0,  1928, 4*16+ 5, 0x0 +NORM },	/* 1927  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 1928  */
{  1926,  1930, 2*16+ 3, 0x0 +NORM },	/* 1929 dia 7 */
{     0,     0, 9*16+ 3, 0x10+NORM },	/* 1930  */
{  1929,  1936,10*16+ 3, 0x10+NORM },	/* 1931 dia 8 */
{     0,  1933, 5*16+ 6, 0x0 +NORM },	/* 1932  */
{     0,  1934, 3*16+ 4, 0x10+NORM },	/* 1933  */
{     0,  1935, 2*16+ 5, 0x0 +NORM },	/* 1934  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 1935  */
{  1932,  1937, 2*16+ 3, 0x0 +BAD  },	/* 1936 dia 9 */
{     0,  1938, 4*16+ 6, 0x10+NORM },	/* 1937  */
{     0,  1939, 3*16+ 6, 0x0 +NORM },	/* 1938  */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 1939  */
{     0,  1948, 3*16+ 7, 0x10+NORM },	/* 1940 I 1-3 */
{     0,  1942, 5*16+ 5, 0x0 +NORM },	/* 1941  */
{     0,  1943, 6*16+ 5, 0x10+NORM },	/* 1942  */
{     0,  1944, 5*16+ 6, 0x0 +NORM },	/* 1943  */
{     0,  1945, 7*16+ 4, 0x10+NORM },	/* 1944  */
{     0,  1946, 4*16+ 8, 0x0 +NORM },	/* 1945  */
{     0,  1947, 3*16+ 4, 0x10+NORM },	/* 1946  */
{     0,     0, 2*16+ 5, 0x0 +NORM },	/* 1947  */
{  1941,  1970, 2*16+ 7, 0x0 +NORM },	/* 1948 dia 13 */
{     0,  1981,10*16+ 3, 0x10+NORM },	/* 1949  */
{  1949,  1951, 2*16+ 8, 0x10+NORM },	/* 1950 dia 14 */
{     0,  1952, 3*16+ 8, 0x0 +NORM },	/* 1951  */
{     0,  1953, 2*16+ 6, 0x10+NORM },	/* 1952  */
{     0,  1954, 4*16+ 7, 0x0 +NORM },	/* 1953  */
{     0,  1955, 3*16+ 6, 0x10+NORM },	/* 1954  */
{     0,  1966, 4*16+ 6, 0x0 +NORM },	/* 1955  */
{     0,  1957, 4*16+ 5, 0x10+NORM },	/* 1956  */
{     0,  1958, 2*16+ 5, 0x0 +NORM },	/* 1957  */
{     0,  1959, 1*16+ 7, 0x10+NORM },	/* 1958  */
{     0,  1960, 1*16+ 5, 0x0 +NORM },	/* 1959  */
{     0,  1961, 1*16+ 6, 0x10+NORM },	/* 1960  */
{     0,  1962, 2*16+ 9, 0x0 +NORM },	/* 1961  */
{     0,  1963, 3*16+ 4, 0x10+NORM },	/* 1962  */
{     0,  1964, 5*16+ 5, 0x0 +NORM },	/* 1963  */
{     0,  1965, 4*16+ 4, 0x10+NORM },	/* 1964  */
{     0,     0, 6*16+ 5, 0x0 +NORM },	/* 1965  */
{  1956,  1967, 2*16+ 9, 0x10+BAD  },	/* 1966 dia 16 */
{     0,  1968, 2*16+ 5, 0x0 +NORM },	/* 1967  */
{     0,  1969, 1*16+ 7, 0x10+NORM },	/* 1968  */
{     0,     0, 5*16+ 5, 0x0 +NORM },	/* 1969  */
{  1950,  1971, 2*16+ 6, 0x10+BAD  },	/* 1970 dia 17 */
{     0,  1972, 3*16+ 6, 0x0 +NORM },	/* 1971  */
{     0,  1973, 2*16+ 8, 0x10+NORM },	/* 1972  */
{     0,  1974, 4*16+ 7, 0x0 +NORM },	/* 1973  */
{     0,  1975, 3*16+ 8, 0x10+NORM },	/* 1974  */
{     0,  1976, 4*16+ 8, 0x0 +NORM },	/* 1975  */
{     0,  1977, 2*16+ 5, 0x10+NORM },	/* 1976  */
{     0,  1978, 2*16+ 9, 0x0 +NORM },	/* 1977  */
{     0,  1979, 1*16+ 7, 0x10+NORM },	/* 1978  */
{     0,  1980, 5*16+ 9, 0x0 +NORM },	/* 1979  */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 1980  */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 1981 ref 1 */
{  1925,  2003, 3*16+10, 0x0 +NORM },	/* 1982 I 1-4 */
{     0,  1984, 4*16+ 6, 0x10+NORM },	/* 1983  */
{     0,  1999, 3*16+ 6, 0x0 +NORM },	/* 1984  */
{     0,  1990, 4*16+ 8, 0x10+NORM },	/* 1985  */
{     0,  1987, 3*16+ 7, 0x0 +NORM },	/* 1986  */
{     0,  1988, 5*16+ 6, 0x10+NORM },	/* 1987  */
{     0,  1989, 3*16+ 8, 0x0 +NORM },	/* 1988  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 1989  */
{  1986,  1991, 4*16+ 5, 0x0 +BAD  },	/* 1990 dia 20 */
{     0,  1992, 5*16+ 5, 0x10+NORM },	/* 1991  */
{     0,  1993, 4*16+ 7, 0x0 +NORM },	/* 1992  */
{     0,  1994, 5*16+ 7, 0x10+NORM },	/* 1993  */
{     0,  1995, 5*16+ 6, 0x0 +NORM },	/* 1994  */
{     0,  1996, 6*16+ 6, 0x10+NORM },	/* 1995  */
{     0,  1997, 3*16+ 8, 0x0 +NORM },	/* 1996  */
{     0,  1998, 4*16+ 6, 0x10+NORM },	/* 1997  */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 1998  */
{  1985,  2000, 4*16+ 7, 0x10+BAD  },	/* 1999 dia 21 */
{     0,  2001, 3*16+ 7, 0x0 +NORM },	/* 2000  */
{     0,  2002, 4*16+ 8, 0x10+NORM },	/* 2001  */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 2002  */
{  1983,  2004, 3*16+ 8, 0x10+NORM },	/* 2003 dia 22 */
{     0,  2005, 5*16+ 5, 0x0 +NORM },	/* 2004  */
{     0,  2006, 6*16+ 5, 0x10+NORM },	/* 2005  */
{     0,  2012, 5*16+ 6, 0x0 +NORM },	/* 2006  */
{     0,  2018, 3*16+ 4, 0x10+NORM },	/* 2007  */
{     0,  2009, 2*16+ 4, 0x0 +NORM },	/* 2008  */
{     0,  2010, 2*16+ 3, 0x10+NORM },	/* 2009  */
{     0,  2011, 5*16+ 8, 0x0 +NORM },	/* 2010  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 2011  */
{  2007,  2013, 5*16+ 8, 0x10+BAD  },	/* 2012 dia 23 */
{     0,  2014, 6*16+ 4, 0x0 +NORM },	/* 2013  */
{     0,  2015, 7*16+ 4, 0x10+NORM },	/* 2014  */
{     0,  2016, 6*16+ 3, 0x0 +NORM },	/* 2015  */
{     0,  2017, 7*16+ 3, 0x10+NORM },	/* 2016  */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 2017  */
{  2008,  2019, 6*16+ 4, 0x0 +BAD  },	/* 2018 dia 23 */
{     0,  2020, 7*16+ 4, 0x10+NORM },	/* 2019  */
{     0,  2021, 6*16+ 3, 0x0 +NORM },	/* 2020  */
{     0,  2022, 7*16+ 3, 0x10+NORM },	/* 2021  */
{     0,  2023, 5*16+ 2, 0x0 +NORM },	/* 2022  */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 2023  */
{     0,  2102, 4*16+ 8, 0x10+NORM },	/* 2024 I 1-5 */
{     0,  2026, 2*16+ 8, 0x0 +NORM },	/* 2025  */
{     0,  2027, 4*16+ 7, 0x10+NORM },	/* 2026  */
{     0,     0, 3*16+10, 0x0 +NORM },	/* 2027  */
{  2025,  2029, 4*16+ 7, 0x0 +BAD  },	/* 2028 dia 27 */
{     0,  2030, 5*16+ 7, 0x10+NORM },	/* 2029  */
{     0,  2031, 5*16+ 8, 0x0 +NORM },	/* 2030  */
{     0,  2095, 3*16+ 8, 0x10+NORM },	/* 2031  */
{     0,  2033, 5*16+ 6, 0x0 +BAD  },	/* 2032  */
{     0,  2034, 6*16+ 7, 0x10+NORM },	/* 2033  */
{     0,  2035, 4*16+ 5, 0x0 +NORM },	/* 2034  */
{     0,  2036, 6*16+ 6, 0x10+NORM },	/* 2035  */
{     0,  2037, 5*16+ 5, 0x0 +NORM },	/* 2036  */
{     0,  2040, 2*16+ 7, 0x10+NORM },	/* 2037  */
{     0,  2039, 3*16+ 6, 0x0 +NORM },	/* 2038  */
{     0,     0, 6*16+ 5, 0x10+NORM },	/* 2039  */
{  2038,  2041, 6*16+ 5, 0x0 +BAD  },	/* 2040 dia 28 */
{     0,  2042, 3*16+ 6, 0x10+NORM },	/* 2041  */
{     0,  2043, 6*16+ 4, 0x0 +NORM },	/* 2042  */
{     0,  2044, 6*16+ 3, 0x10+NORM },	/* 2043  */
{     0,  2045, 7*16+ 3, 0x0 +NORM },	/* 2044  */
{     0,  2051, 7*16+ 2, 0x10+NORM },	/* 2045  */
{     0,  2047, 8*16+ 3, 0x0 +NORM },	/* 2046  */
{     0,  2048, 8*16+ 2, 0x10+NORM },	/* 2047  */
{     0,  2049, 9*16+ 3, 0x0 +NORM },	/* 2048  */
{     0,  2050, 9*16+ 2, 0x10+NORM },	/* 2049  */
{     0,     0,10*16+ 3, 0x0 +NORM },	/* 2050  */
{  2046,  2052, 6*16+ 2, 0x0 +BAD  },	/* 2051 dia 28 */
{     0,  2053, 5*16+ 3, 0x10+NORM },	/* 2052  */
{     0,  2054, 8*16+ 2, 0x0 +NORM },	/* 2053  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 2054  */
{  2032,  2056, 2*16+ 7, 0x0 +BAD  },	/* 2055 dia 30 */
{     0,  2057, 5*16+ 9, 0x10+NORM },	/* 2056  */
{     0,  2058, 6*16+ 8, 0x0 +NORM },	/* 2057  */
{     0,  2083, 2*16+ 8, 0x10+NORM },	/* 2058  */
{     0,  2060, 5*16+ 6, 0x0 +NORM },	/* 2059  */
{     0,  2061, 6*16+ 7, 0x10+NORM },	/* 2060  */
{     0,  2062, 6*16+ 6, 0x0 +NORM },	/* 2061  */
{     0,  2063, 7*16+ 7, 0x10+NORM },	/* 2062  */
{     0,  2064, 4*16+ 5, 0x0 +NORM },	/* 2063  */
{     0,  2067, 7*16+ 6, 0x10+NORM },	/* 2064  */
{     0,  2066, 6*16+ 5, 0x0 +NORM },	/* 2065  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 2066  */
{  2065,  2068, 6*16+ 4, 0x0 +NORM },	/* 2067 dia 31 */
{     0,  2069, 6*16+ 3, 0x10+NORM },	/* 2068  */
{     0,  2070, 7*16+ 4, 0x0 +NORM },	/* 2069  */
{     0,  2071, 7*16+ 3, 0x10+NORM },	/* 2070  */
{     0,  2072, 8*16+ 4, 0x0 +NORM },	/* 2071  */
{     0,  2073, 6*16+ 5, 0x10+NORM },	/* 2072  */
{     0,  2074, 5*16+ 5, 0x0 +NORM },	/* 2073  */
{     0,  2081, 7*16+ 5, 0x10+NORM },	/* 2074  */
{     0,  2076, 2*16+ 3, 0x0 +NORM },	/* 2075  */
{     0,  2077, 9*16+ 4, 0x10+NORM },	/* 2076  */
{     0,  2078, 8*16+ 3, 0x0 +NORM },	/* 2077  */
{     0,  2079, 9*16+ 3, 0x10+NORM },	/* 2078  */
{     0,  2080, 8*16+ 5, 0x0 +NORM },	/* 2079  */
{     0,     0, 8*16+ 2, 0x10+NORM },	/* 2080  */
{  2075,  2082, 8*16+ 3, 0x0 +NORM },	/* 2081 dia 31 */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 2082  */
{  2059,  2084, 3*16+ 3, 0x0 +NORM },	/* 2083 dia 33 */
{     0,  2085, 3*16+ 2, 0x10+NORM },	/* 2084  */
{     0,  2086, 3*16+ 4, 0x0 +NORM },	/* 2085  */
{     0,  2087, 6*16+ 6, 0x10+NORM },	/* 2086  */
{     0,  2088, 2*16+ 2, 0x0 +NORM },	/* 2087  */
{     0,  2089, 4*16+ 2, 0x10+NORM },	/* 2088  */
{     0,  2090, 2*16+ 1, 0x0 +NORM },	/* 2089  */
{     0,  2091, 6*16+ 9, 0x10+NORM },	/* 2090  */
{     0,  2092, 7*16+ 8, 0x0 +NORM },	/* 2091  */
{     0,  2093, 7*16+ 9, 0x10+NORM },	/* 2092  */
{     0,  2094, 8*16+ 8, 0x0 +NORM },	/* 2093  */
{     0,     0, 8*16+ 6, 0x10+NORM },	/* 2094  */
{  2055,  2096, 3*16+ 6, 0x0 +NORM },	/* 2095 dia 34 */
{     0,  2097, 5*16+ 6, 0x10+NORM },	/* 2096  */
{     0,  2098, 2*16+ 8, 0x0 +NORM },	/* 2097  */
{     0,  2099, 2*16+ 9, 0x10+NORM },	/* 2098  */
{     0,  2100, 1*16+ 7, 0x0 +NORM },	/* 2099  */
{     0,  2101, 4*16+10, 0x10+NORM },	/* 2100  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2101  */
{  2028,  2103, 3*16+ 3, 0x0 +NORM },	/* 2102 dia 35 */
{     0,  2104, 3*16+ 2, 0x10+NORM },	/* 2103  */
{     0,  2105, 2*16+ 4, 0x0 +NORM },	/* 2104  */
{     0,  2106, 2*16+ 3, 0x10+NORM },	/* 2105  */
{     0,  2111, 3*16+ 4, 0x0 +NORM },	/* 2106  */
{     0,  2108, 4*16+ 7, 0x10+NORM },	/* 2107  */
{     0,  2109, 2*16+ 2, 0x0 +NORM },	/* 2108  */
{     0,  2110, 2*16+ 1, 0x10+NORM },	/* 2109  */
{     0,     0, 1*16+ 2, 0x0 +NORM },	/* 2110  */
{  2107,  2112, 2*16+ 2, 0x10+BAD  },	/* 2111 dia 35 */
{     0,     0, 5*16+ 5, 0x0 +NORM },	/* 2112  */
{  2024,  2116, 3*16+ 9, 0x10+NORM },	/* 2113 dia 36 */
{     0,  2115, 4*16+ 7, 0x0 +NORM },	/* 2114  */
{     0,     0, 3*16+12, 0x10+NORM },	/* 2115  */
{  2114,  2117, 3*16+11, 0x0 +NORM },	/* 2116 dia 37 */
{     0,  2118, 4*16+ 8, 0x10+NORM },	/* 2117  */
{     0,  2119, 3*16+ 3, 0x0 +NORM },	/* 2118  */
{     0,  2120, 3*16+ 2, 0x10+NORM },	/* 2119  */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 2120  */
{   279,  2122, 3*16+ 6, 0x0 +BAD  },	/* 2121 dia 38 */
{     0,  2126, 4*16+ 7, 0x10+NORM },	/* 2122  */
{     0,  2124, 3*16+ 7, 0x0 +NORM },	/* 2123  */
{     0,  2125, 4*16+ 8, 0x10+NORM },	/* 2124  */
{     0,     0, 3*16+ 9, 0x0 +NORM },	/* 2125  */
{  2123,  2158, 3*16+ 8, 0x0 +BAD  },	/* 2126 dia 39 */
{     0,  2128, 3*16+ 7, 0x10+NORM },	/* 2127  */
{     0,  2129, 2*16+ 7, 0x0 +NORM },	/* 2128  */
{     0,  2136, 2*16+ 6, 0x10+NORM },	/* 2129  */
{     0,  2131, 2*16+ 5, 0x0 +NORM },	/* 2130  */
{     0,  2132, 2*16+ 8, 0x10+NORM },	/* 2131  */
{     0,  2133, 1*16+ 6, 0x0 +NORM },	/* 2132  */
{     0,  2134, 3*16+ 9, 0x10+NORM },	/* 2133  */
{     0,  2135, 2*16+ 3, 0x0 +NORM },	/* 2134  */
{     0,     0, 4*16+ 8, 0x10+NORM },	/* 2135  */
{  2130,  2137, 2*16+ 8, 0x0 +BAD  },	/* 2136 dia 40 */
{     0,  2138, 2*16+ 5, 0x10+NORM },	/* 2137  */
{     0,  2139, 3*16+ 4, 0x0 +NORM },	/* 2138  */
{     0,  2140, 2*16+ 4, 0x10+NORM },	/* 2139  */
{     0,  2141, 3*16+ 3, 0x0 +NORM },	/* 2140  */
{     0,  2142, 2*16+ 3, 0x10+NORM },	/* 2141  */
{     0,  2143, 2*16+ 2, 0x0 +NORM },	/* 2142  */
{     0,  2144, 3*16+ 2, 0x10+NORM },	/* 2143  */
{     0,  2145, 4*16+ 2, 0x0 +NORM },	/* 2144  */
{     0,  2146, 3*16+ 1, 0x10+NORM },	/* 2145  */
{     0,  2147, 4*16+ 1, 0x0 +NORM },	/* 2146  */
{     0,  2148, 5*16+ 2, 0x10+NORM },	/* 2147  */
{     0,  2149, 2*16+ 1, 0x0 +NORM },	/* 2148  */
{     0,  2150, 3*16+ 2, 0x10+NORM },	/* 2149  */
{     0,  2151, 1*16+ 2, 0x0 +NORM },	/* 2150  */
{     0,  2152, 4*16+ 4, 0x10+NORM },	/* 2151  */
{     0,  2153, 3*16+ 1, 0x0 +NORM },	/* 2152  */
{     0,  2154, 4*16+ 5, 0x10+NORM },	/* 2153  */
{     0,  2155, 3*16+ 2, 0x0 +NORM },	/* 2154  */
{     0,  2156, 5*16+ 1, 0x10+NORM },	/* 2155  */
{     0,  2157, 1*16+ 3, 0x0 +NORM },	/* 2156  */
{     0,     0, 1*16+ 4, 0x10+NORM },	/* 2157  */
{  2127,  2159, 4*16+ 8, 0x10+NORM },	/* 2158 dia 41 */
{     0,  2160, 3*16+ 9, 0x0 +NORM },	/* 2159  */
{     0,  2161, 3*16+ 7, 0x10+NORM },	/* 2160  */
{     0,  2162, 2*16+ 7, 0x0 +NORM },	/* 2161  */
{     0,  2163, 2*16+ 8, 0x10+NORM },	/* 2162  */
{     0,  2164, 2*16+ 6, 0x0 +NORM },	/* 2163  */
{     0,  2165, 3*16+10, 0x10+NORM },	/* 2164  */
{     0,  2166, 2*16+ 9, 0x0 +NORM },	/* 2165  */
{     0,     0, 4*16+ 9, 0x10+NORM },	/* 2166  */
{  1982,  2185, 3*16+ 8, 0x10+NORM },	/* 2167 dia 42 */
{     0,  2175, 4*16+ 6, 0x0 +NORM },	/* 2168  */
{     0,  2170, 3*16+ 4, 0x10+NORM },	/* 2169  */
{     0,  2171, 4*16+11, 0x0 +NORM },	/* 2170  */
{     0,  2172, 5*16+ 8, 0x10+NORM },	/* 2171  */
{     0,  2173, 6*16+ 6, 0x0 +NORM },	/* 2172  */
{     0,  2174, 7*16+ 8, 0x10+NORM },	/* 2173  */
{     0,     0, 8*16+ 6, 0x0 +NORM },	/* 2174  */
{  2169,  2176, 3*16+11, 0x10+NORM },	/* 2175 dia 43 */
{     0,  2177, 2*16+ 3, 0x0 +NORM },	/* 2176  */
{     0,  2178, 9*16+ 3, 0x10+NORM },	/* 2177  */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 2178  */
{  2168,  2180, 5*16+ 5, 0x0 +BAD  },	/* 2179 dia 44 */
{     0,  2181, 6*16+ 5, 0x10+NORM },	/* 2180  */
{     0,  2182, 5*16+ 6, 0x0 +NORM },	/* 2181  */
{     0,  2183, 5*16+ 8, 0x10+NORM },	/* 2182  */
{     0,  2184, 6*16+ 6, 0x0 +NORM },	/* 2183  */
{     0,     0, 7*16+ 5, 0x10+NORM },	/* 2184  */
{  2179,  2186, 2*16+ 3, 0x0 +NORM },	/* 2185 dia 45 */
{     0,  2187, 4*16+ 6, 0x10+NORM },	/* 2186  */
{     0,  2188, 3*16+ 6, 0x0 +NORM },	/* 2187  */
{     0,  2189, 4*16+ 7, 0x10+NORM },	/* 2188  */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 2189  */
{  2167,  2207, 3*16+ 9, 0x10+NORM },	/* 2190 dia 46 */
{     0,  2192, 3*16+ 7, 0x0 +NORM },	/* 2191  */
{     0,  2193, 3*16+ 4, 0x10+NORM },	/* 2192  */
{     0,  2194, 5*16+ 7, 0x0 +NORM },	/* 2193  */
{     0,  2195, 5*16+ 9, 0x10+NORM },	/* 2194  */
{     0,     0, 7*16+ 7, 0x0 +NORM },	/* 2195  */
{  2191,     0, 4*16+ 8, 0x0 +NORM },	/* 2196 dia 46 */
{  2196,  2198, 2*16+ 3, 0x0 +NORM },	/* 2197 dia 47 */
{     0,  2199, 4*16+ 6, 0x10+NORM },	/* 2198  */
{     0,  2200, 3*16+ 6, 0x0 +NORM },	/* 2199  */
{     0,  2201, 4*16+ 7, 0x10+NORM },	/* 2200  */
{     0,  2202, 2*16+ 8, 0x0 +NORM },	/* 2201  */
{     0,  2203, 2*16+ 9, 0x10+NORM },	/* 2202  */
{     0,  2204, 3*16+ 8, 0x0 +NORM },	/* 2203  */
{     0,  2205, 4*16+ 8, 0x10+NORM },	/* 2204  */
{     0,  2206, 3*16+ 2, 0x0 +NORM },	/* 2205  */
{     0,     0, 5*16+10, 0x10+NORM },	/* 2206  */
{  2197,  2208, 3*16+11, 0x0 +NORM },	/* 2207 dia 48 */
{     0,  2209, 4*16+ 6, 0x10+NORM },	/* 2208  */
{     0,  2210, 3*16+ 7, 0x0 +NORM },	/* 2209  */
{     0,  2211, 4*16+ 8, 0x10+NORM },	/* 2210  */
{     0,  2212, 3*16+ 3, 0x0 +NORM },	/* 2211  */
{     0,  2213, 3*16+ 2, 0x10+NORM },	/* 2212  */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 2213  */
{  2190,  2215, 4*16+ 6, 0x0 +BAD  },	/* 2214 dia 49 */
{     0,  2216, 5*16+ 6, 0x10+NORM },	/* 2215  */
{     0,  2217, 5*16+ 7, 0x0 +NORM },	/* 2216  */
{     0,  2218, 6*16+ 6, 0x10+NORM },	/* 2217  */
{     0,  2219, 3*16+ 9, 0x0 +NORM },	/* 2218  */
{     0,     0, 6*16+ 7, 0x10+NORM },	/* 2219  */
{  1711,  2310, 6*16+ 4, 0x10+NORM },	/* 2220 Ishida vol 1 chapt II sec 1 */
{     0,  2222, 3*16+ 3, 0x0 +NORM },	/* 2221  */
{     0,  2223, 3*16+ 2, 0x10+NORM },	/* 2222  */
{     0,  2227, 4*16+ 4, 0x0 +NORM },	/* 2223  */
{     0,  2225, 5*16+ 3, 0x10+NORM },	/* 2224  */
{     0,  2226, 2*16+ 3, 0x0 +NORM },	/* 2225  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 2226  */
{  2224,  2231, 2*16+ 3, 0x10+NORM },	/* 2227 dia 3 */
{     0,  2229, 3*16+ 4, 0x0 +BAD  },	/* 2228  */
{     0,  2230, 5*16+ 3, 0x10+NORM },	/* 2229  */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 2230  */
{  2228,  2232, 5*16+ 3, 0x0 +NORM },	/* 2231 dia 4 */
{     0,  2233, 3*16+ 4, 0x10+NORM },	/* 2232  */
{     0,  2235, 4*16+ 2, 0x0 +NORM },	/* 2233  */
{     0,     0, 4*16+ 5, 0x10+BAD  },	/* 2234  */
{  2234,  2236, 3*16+ 3, 0x10+NORM },	/* 2235 dia 5 */
{     0,  2237, 5*16+ 4, 0x0 +NORM },	/* 2236  */
{     0,  2238, 4*16+ 5, 0x10+NORM },	/* 2237  */
{     0,  2239, 5*16+ 5, 0x0 +NORM },	/* 2238  */
{     0,  2240, 4*16+ 6, 0x10+NORM },	/* 2239  */
{     0,  2241, 3*16+ 8, 0x0 +NORM },	/* 2240  */
{     0,  2242, 5*16+ 6, 0x10+NORM },	/* 2241  */
{     0,  2243, 6*16+ 5, 0x0 +NORM },	/* 2242  */
{     0,     0, 3*16+ 6, 0x10+NORM },	/* 2243  */
{  2221,  2249, 3*16+ 8, 0x0 +NORM },	/* 2244 dia 6 */
{     0,  2246, 3*16+ 4, 0x10+NORM },	/* 2245  */
{     0,  2248, 4*16+ 5, 0x0 +NORM },	/* 2246  */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 2247  */
{  2247,     0, 9*16+ 3, 0x10+NORM },	/* 2248 dia 6 */
{  2245,  2250,10*16+ 3, 0x10+BAD  },	/* 2249 dia 7 */
{     0,  2251, 3*16+ 3, 0x0 +NORM },	/* 2250  */
{     0,  2252, 3*16+ 2, 0x10+NORM },	/* 2251  */
{     0,  2253, 4*16+ 4, 0x0 +NORM },	/* 2252  */
{     0,  2254, 5*16+ 3, 0x10+NORM },	/* 2253  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2254  */
{  2244,  2268, 3*16+ 9, 0x0 +NORM },	/* 2255 dia 8 */
{     0,     0, 3*16+ 4, 0x10+BAD  },	/* 2256  */
{  2256,  2258, 3*16+11, 0x10+NORM },	/* 2257 dia 8 */
{     0,  2259, 3*16+ 3, 0x0 +NORM },	/* 2258  */
{     0,  2260, 3*16+ 2, 0x10+NORM },	/* 2259  */
{     0,  2261, 4*16+ 4, 0x0 +NORM },	/* 2260  */
{     0,  2262, 5*16+ 3, 0x10+NORM },	/* 2261  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2262  */
{  2257,  2264,10*16+ 3, 0x10+NORM },	/* 2263 dia 8 */
{     0,  2265, 3*16+ 3, 0x0 +NORM },	/* 2264  */
{     0,  2266, 3*16+ 2, 0x10+NORM },	/* 2265  */
{     0,  2267, 4*16+ 4, 0x0 +NORM },	/* 2266  */
{     0,     0, 5*16+ 3, 0x10+NORM },	/* 2267  */
{  2263,  2285, 3*16+ 7, 0x10+NORM },	/* 2268 dia 9 */
{     0,  2270, 2*16+ 7, 0x0 +NORM },	/* 2269  */
{     0,  2271, 2*16+ 8, 0x10+NORM },	/* 2270  */
{     0,  2272, 3*16+ 8, 0x0 +NORM },	/* 2271  */
{     0,  2273, 2*16+ 6, 0x10+NORM },	/* 2272  */
{     0,  2274, 4*16+ 7, 0x0 +NORM },	/* 2273  */
{     0,  2275, 3*16+ 6, 0x10+NORM },	/* 2274  */
{     0,  2276, 4*16+ 6, 0x0 +NORM },	/* 2275  */
{     0,  2277, 4*16+ 5, 0x10+NORM },	/* 2276  */
{     0,  2278, 2*16+ 5, 0x0 +NORM },	/* 2277  */
{     0,  2279, 1*16+ 7, 0x10+NORM },	/* 2278  */
{     0,  2280, 1*16+ 5, 0x0 +NORM },	/* 2279  */
{     0,  2281, 1*16+ 6, 0x10+NORM },	/* 2280  */
{     0,  2282, 2*16+ 9, 0x0 +NORM },	/* 2281  */
{     0,  2283, 3*16+ 4, 0x10+NORM },	/* 2282  */
{     0,  2284, 5*16+ 5, 0x0 +NORM },	/* 2283  */
{     0,     0, 4*16+ 4, 0x10+NORM },	/* 2284  */
{  2269,  2286, 3*16+ 3, 0x0 +BAD  },	/* 2285 dia 11 */
{     0,  2287, 3*16+ 2, 0x10+NORM },	/* 2286  */
{     0,  2290, 4*16+ 4, 0x0 +NORM },	/* 2287  */
{     0,  2289, 5*16+ 3, 0x10+BAD  },	/* 2288  */
{     0,     0, 2*16+ 7, 0x0 +NORM },	/* 2289  */
{  2288,  2291, 2*16+ 3, 0x10+NORM },	/* 2290 dia 12 */
{     0,  2292, 5*16+ 3, 0x0 +NORM },	/* 2291  */
{     0,  2293, 3*16+ 4, 0x10+NORM },	/* 2292  */
{     0,  2294, 4*16+ 2, 0x0 +NORM },	/* 2293  */
{     0,  2295, 3*16+ 3, 0x10+NORM },	/* 2294  */
{     0,  2296, 5*16+ 4, 0x0 +NORM },	/* 2295  */
{     0,  2297, 4*16+ 5, 0x10+NORM },	/* 2296  */
{     0,  2298, 5*16+ 5, 0x0 +NORM },	/* 2297  */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 2298  */
{  2255,  2301, 8*16+ 3, 0x0 +NORM },	/* 2299 Sec 4 */
{     0,     0, 3*16+ 9, 0x10+NORM },	/* 2300  */
{  2300,     0,11*16+ 4, 0x10+NORM },	/* 2301 dia 13 */
{  2299,  2309, 3*16+ 9, 0x10+NORM },	/* 2302 sec 5 */
{     0,  2304, 3*16+ 3, 0x0 +NORM },	/* 2303  */
{     0,  2305, 3*16+ 2, 0x10+NORM },	/* 2304  */
{     0,  2306, 4*16+ 4, 0x0 +NORM },	/* 2305  */
{     0,  2307, 5*16+ 3, 0x10+NORM },	/* 2306  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2307  */
{  2303,     0, 3*16+ 7, 0x0 +NORM },	/* 2308 dia 14 */
{  2308,     0, 4*16+ 8, 0x0 +NORM },	/* 2309 dia 14 */
{  2302,  2311, 3*16+ 4, 0x10+NORM },	/* 2310 dia 15 */
{     0,  2312, 4*16+ 5, 0x0 +NORM },	/* 2311  */
{     0,     0, 4*16+ 8, 0x10+NORM },	/* 2312  */
{     0,  2314, 6*16+ 4, 0x0 +NORM },	/* 2313 Ishida vol 1 chapter III sec 1 */
{     0,  2315, 6*16+ 3, 0x10+NORM },	/* 2314  */
{     0,  2316, 3*16+ 3, 0x0 +NORM },	/* 2315  */
{     0,  2317, 3*16+ 2, 0x10+NORM },	/* 2316  */
{     0,  2320, 4*16+ 4, 0x0 +NORM },	/* 2317  */
{     0,  2319, 5*16+ 3, 0x10+NORM },	/* 2318  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2319  */
{  2318,  2321, 2*16+ 3, 0x10+BAD  },	/* 2320 dia 3 */
{     0,  2322, 5*16+ 3, 0x0 +NORM },	/* 2321  */
{     0,  2323, 3*16+ 4, 0x10+NORM },	/* 2322  */
{     0,  2324, 4*16+ 2, 0x0 +NORM },	/* 2323  */
{     0,  2325, 3*16+ 3, 0x10+NORM },	/* 2324  */
{     0,  2326, 6*16+ 2, 0x0 +NORM },	/* 2325  */
{     0,  2327, 4*16+ 5, 0x10+NORM },	/* 2326  */
{     0,  2328, 5*16+ 4, 0x0 +NORM },	/* 2327  */
{     0,     0, 3*16+ 6, 0x10+NORM },	/* 2328  */
{  2313,  2347, 3*16+ 9, 0x0 +NORM },	/* 2329 sec 2 */
{     0,  2348, 3*16+ 7, 0x10+NORM },	/* 2330  */
{     0,  2332, 2*16+ 7, 0x0 +NORM },	/* 2331  */
{     0,  2333, 2*16+ 8, 0x10+NORM },	/* 2332  */
{     0,  2334, 3*16+ 8, 0x0 +NORM },	/* 2333  */
{     0,  2335, 2*16+ 6, 0x10+NORM },	/* 2334  */
{     0,  2336, 4*16+ 7, 0x0 +NORM },	/* 2335  */
{     0,  2337, 3*16+ 6, 0x10+NORM },	/* 2336  */
{     0,  2338, 4*16+ 6, 0x0 +NORM },	/* 2337  */
{     0,  2339, 4*16+ 5, 0x10+NORM },	/* 2338  */
{     0,  2340, 2*16+ 5, 0x0 +NORM },	/* 2339  */
{     0,  2341, 1*16+ 7, 0x10+NORM },	/* 2340  */
{     0,  2342, 1*16+ 5, 0x0 +NORM },	/* 2341  */
{     0,  2343, 1*16+ 6, 0x10+NORM },	/* 2342  */
{     0,  2344, 2*16+ 9, 0x0 +NORM },	/* 2343  */
{     0,  2345, 3*16+ 4, 0x10+NORM },	/* 2344  */
{     0,  2346, 5*16+ 5, 0x0 +NORM },	/* 2345  */
{     0,     0, 4*16+ 4, 0x10+NORM },	/* 2346  */
{  2330,     0, 3*16+ 4, 0x10+BAD  },	/* 2347 bad one */
{  2331,  2363, 6*16+ 4, 0x0 +NORM },	/* 2348 dia 6 */
{     0,  2350, 6*16+ 3, 0x10+NORM },	/* 2349  */
{     0,  2353, 2*16+ 7, 0x0 +NORM },	/* 2350  */
{     0,  2352, 5*16+ 4, 0x10+NORM },	/* 2351  */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 2352  */
{  2351,  2354, 2*16+ 8, 0x10+BAD  },	/* 2353 dia 7 */
{     0,  2355, 3*16+ 8, 0x0 +NORM },	/* 2354  */
{     0,  2356, 2*16+ 6, 0x10+NORM },	/* 2355  */
{     0,  2357, 4*16+ 7, 0x0 +NORM },	/* 2356  */
{     0,  2358, 3*16+ 6, 0x10+NORM },	/* 2357  */
{     0,  2359, 4*16+ 6, 0x0 +NORM },	/* 2358  */
{     0,  2360, 4*16+ 5, 0x10+NORM },	/* 2359  */
{     0,  2361, 4*16+ 4, 0x0 +NORM },	/* 2360  */
{     0,  2362, 5*16+ 5, 0x10+NORM },	/* 2361  */
{     0,     0, 5*16+ 4, 0x0 +NORM },	/* 2362  */
{  2349,  2366, 4*16+ 5, 0x10+NORM },	/* 2363 dia 8 */
{     0,  2365, 6*16+ 3, 0x0 +NORM },	/* 2364  */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 2365  */
{  2364,  2367, 4*16+ 6, 0x0 +BAD  },	/* 2366 dia 9 */
{     0,  2368, 5*16+ 5, 0x10+NORM },	/* 2367  */
{     0,  2369, 3*16+ 6, 0x0 +NORM },	/* 2368  */
{     0,  2372, 3*16+ 4, 0x10+NORM },	/* 2369  */
{     0,  2371, 4*16+ 7, 0x0 +NORM },	/* 2370  */
{     0,     0, 6*16+ 3, 0x10+NORM },	/* 2371  */
{  2370,  2373, 6*16+ 3, 0x0 +BAD  },	/* 2372 dia 10 */
{     0,  2374, 5*16+ 6, 0x10+NORM },	/* 2373  */
{     0,  2375, 4*16+ 7, 0x0 +NORM },	/* 2374  */
{     0,  2376, 6*16+ 2, 0x10+NORM },	/* 2375  */
{     0,  2377, 5*16+ 2, 0x0 +NORM },	/* 2376  */
{     0,  2378, 8*16+ 2, 0x10+NORM },	/* 2377  */
{     0,  2379, 5*16+ 3, 0x0 +NORM },	/* 2378  */
{     0,     0, 7*16+ 5, 0x10+NORM },	/* 2379  */
{  2329,  2381, 3*16+10, 0x0 +NORM },	/* 2380 dia 11 */
{     0,  2411, 3*16+ 8, 0x10+NORM },	/* 2381  */
{     0,  2400, 6*16+ 4, 0x0 +NORM },	/* 2382  */
{     0,  2384, 6*16+ 3, 0x10+NORM },	/* 2383  */
{     0,  2385, 3*16+ 3, 0x0 +NORM },	/* 2384  */
{     0,  2386, 3*16+ 2, 0x10+NORM },	/* 2385  */
{     0,  2390, 4*16+ 4, 0x0 +NORM },	/* 2386  */
{     0,  2388, 5*16+ 3, 0x10+NORM },	/* 2387  */
{     0,  2389, 2*16+ 3, 0x0 +NORM },	/* 2388  */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 2389  */
{  2387,  2391, 2*16+ 3, 0x10+NORM },	/* 2390 dia 12 */
{     0,  2392, 5*16+ 3, 0x0 +NORM },	/* 2391  */
{     0,  2393, 3*16+ 4, 0x10+NORM },	/* 2392  */
{     0,  2394, 4*16+ 2, 0x0 +NORM },	/* 2393  */
{     0,  2395, 3*16+ 3, 0x10+NORM },	/* 2394  */
{     0,  2396, 6*16+ 2, 0x0 +NORM },	/* 2395  */
{     0,  2397, 4*16+ 5, 0x10+NORM },	/* 2396  */
{     0,  2398, 5*16+ 4, 0x0 +NORM },	/* 2397  */
{     0,  2399, 4*16+ 6, 0x10+NORM },	/* 2398  */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 2399  */
{  2383,  2401, 4*16+ 5, 0x10+BAD  },	/* 2400 dia 13 */
{     0,  2402, 4*16+ 6, 0x0 +NORM },	/* 2401  */
{     0,  2403, 5*16+ 5, 0x10+NORM },	/* 2402  */
{     0,  2404, 3*16+ 4, 0x0 +NORM },	/* 2403  */
{     0,  2405, 3*16+ 6, 0x10+NORM },	/* 2404  */
{     0,  2406, 5*16+ 6, 0x0 +NORM },	/* 2405  */
{     0,  2407, 5*16+ 3, 0x10+NORM },	/* 2406  */
{     0,  2408, 3*16+ 7, 0x0 +NORM },	/* 2407  */
{     0,  2409, 2*16+ 6, 0x10+NORM },	/* 2408  */
{     0,     0, 2*16+ 7, 0x0 +NORM },	/* 2409  */
{  2382,     0, 6*16+ 5, 0x0 +NORM },	/* 2410 dia 14 */
{  2410,  2412, 3*16+ 4, 0x10+NORM },	/* 2411 dia 15 */
{     0,  2413, 4*16+ 5, 0x0 +NORM },	/* 2412  */
{     0,  2414, 6*16+ 4, 0x10+NORM },	/* 2413  */
{     0,  2415, 5*16+ 7, 0x0 +NORM },	/* 2414  */
{     0,  2416, 5*16+ 8, 0x10+NORM },	/* 2415  */
{     0,  2417, 6*16+ 8, 0x0 +NORM },	/* 2416  */
{     0,  2418, 5*16+ 9, 0x10+NORM },	/* 2417  */
{     0,     0, 7*16+ 7, 0x0 +NORM },	/* 2418  */
{     0,  2420, 4*16+ 6, 0x0 +NORM },	/* 2419 Ishida C IV sec 1 dia 4 */
{     0,  2421, 5*16+ 8, 0x10+NORM },	/* 2420  */
{     0,  2422, 4*16+ 8, 0x0 +NORM },	/* 2421  */
{     0,     0, 5*16+ 9, 0x10+NORM },	/* 2422  */
{  1905,  2445, 6*16+ 6, 0x0 +NORM },	/* 2423 dia 5 */
{     0,  2425, 8*16+ 5, 0x10+NORM },	/* 2424  */
{     0,  2426, 7*16+ 6, 0x0 +NORM },	/* 2425  */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 2426  */
{  2424,  2428, 7*16+ 6, 0x10+BAD  },	/* 2427 dia 6 */
{     0,  2429, 7*16+ 5, 0x0 +NORM },	/* 2428  */
{     0,  2430, 8*16+ 5, 0x10+NORM },	/* 2429  */
{     0,  2431, 7*16+ 4, 0x0 +NORM },	/* 2430  */
{     0,  2432, 6*16+ 3, 0x10+NORM },	/* 2431  */
{     0,  2435, 7*16+ 3, 0x0 +NORM },	/* 2432  */
{     0,  2434, 6*16+ 2, 0x10+NORM },	/* 2433  */
{     0,     0, 9*16+ 4, 0x0 +NORM },	/* 2434  */
{  2433,  2436, 9*16+ 4, 0x10+BAD  },	/* 2435 dia 7 */
{     0,  2437, 6*16+ 2, 0x0 +NORM },	/* 2436  */
{     0,  2438, 5*16+ 3, 0x10+NORM },	/* 2437  */
{     0,  2439, 8*16+ 2, 0x0 +NORM },	/* 2438  */
{     0,  2440, 9*16+ 3, 0x10+NORM },	/* 2439  */
{     0,  2443, 9*16+ 2, 0x0 +NORM },	/* 2440  */
{     0,  2442,11*16+ 3, 0x10+BAD  },	/* 2441  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 2442  */
{  2441,  2444, 2*16+ 3, 0x10+NORM },	/* 2443 dia 7 */
{     0,     0,11*16+ 3, 0x0 +NORM },	/* 2444  */
{  2427,  2452, 8*16+ 4, 0x0 +NORM },	/* 2445 dia 8 */
{     0,  2447, 6*16+ 3, 0x10+NORM },	/* 2446  */
{     0,     0, 8*16+ 6, 0x0 +NORM },	/* 2447  */
{  2446,  2449, 7*16+ 5, 0x10+BAD  },	/* 2448 dia 8 */
{     0,     0, 8*16+ 6, 0x0 +NORM },	/* 2449  */
{  2448,  2451, 8*16+ 5, 0x10+NORM },	/* 2450 dia 8 */
{     0,     0, 7*16+ 5, 0x0 +NORM },	/* 2451  */
{  2450,  2453, 6*16+ 3, 0x0 +NORM },	/* 2452 dia 9 */
{     0,  2454, 7*16+ 3, 0x10+NORM },	/* 2453  */
{     0,  2455, 7*16+ 4, 0x0 +NORM },	/* 2454  */
{     0,  2456, 5*16+ 3, 0x10+NORM },	/* 2455  */
{     0,  2457, 6*16+ 2, 0x0 +NORM },	/* 2456  */
{     0,  2458, 7*16+ 2, 0x10+NORM },	/* 2457  */
{     0,  2459, 7*16+ 5, 0x0 +NORM },	/* 2458  */
{     0,  2460, 5*16+ 4, 0x10+NORM },	/* 2459  */
{     0,     0, 8*16+ 2, 0x0 +NORM },	/* 2460  */
{  1903,  2462, 4*16+ 8, 0x10+NORM },	/* 2461 dia 10 */
{     0,  2463, 5*16+ 8, 0x0 +NORM },	/* 2462  */
{     0,  2464, 4*16+ 9, 0x10+NORM },	/* 2463  */
{     0,  2465, 5*16+ 9, 0x0 +NORM },	/* 2464  */
{     0,  2466, 4*16+10, 0x10+NORM },	/* 2465  */
{     0,  2473, 5*16+ 2, 0x0 +NORM },	/* 2466  */
{     0,  2468, 5*16+ 3, 0x10+BAD  },	/* 2467  */
{     0,  2469, 6*16+ 4, 0x0 +NORM },	/* 2468  */
{     0,  2470, 6*16+ 3, 0x10+NORM },	/* 2469  */
{     0,  2471, 7*16+ 4, 0x0 +NORM },	/* 2470  */
{     0,  2472, 7*16+ 3, 0x10+NORM },	/* 2471  */
{     0,     0, 8*16+ 4, 0x0 +NORM },	/* 2472  */
{  2467,  2510, 2*16+ 3, 0x10+NORM },	/* 2473 dia 12 */
{     0,  2475, 5*16+ 3, 0x0 +NORM },	/* 2474  */
{     0,  2476, 2*16+ 5, 0x10+NORM },	/* 2475  */
{     0,  2477, 3*16+ 4, 0x0 +NORM },	/* 2476  */
{     0,  2478, 3*16+ 3, 0x10+NORM },	/* 2477  */
{     0,  2479, 2*16+ 4, 0x0 +NORM },	/* 2478  */
{     0,  2480, 3*16+ 6, 0x10+NORM },	/* 2479  */
{     0,  2481, 4*16+ 6, 0x0 +NORM },	/* 2480  */
{     0,  2482, 1*16+ 4, 0x10+NORM },	/* 2481  */
{     0,  2483, 2*16+ 6, 0x0 +NORM },	/* 2482  */
{     0,  2484, 1*16+ 5, 0x10+NORM },	/* 2483  */
{     0,  2485, 2*16+ 7, 0x0 +NORM },	/* 2484  */
{     0,  2486, 3*16+ 8, 0x10+NORM },	/* 2485  */
{     0,  2487, 6*16+ 6, 0x0 +NORM },	/* 2486  */
{     0,  2488, 7*16+ 5, 0x10+NORM },	/* 2487  */
{     0,  2499, 3*16+ 2, 0x0 +NORM },	/* 2488  */
{     0,  2490, 1*16+ 2, 0x10+NORM },	/* 2489  */
{     0,  2491, 7*16+ 6, 0x0 +NORM },	/* 2490  */
{     0,  2492, 8*16+ 5, 0x10+NORM },	/* 2491  */
{     0,  2506, 8*16+ 3, 0x0 +NORM },	/* 2492  */
{  2489,  2494, 4*16+ 2, 0x10+BAD  },	/* 2493 dia 13 */
{     0,  2495, 2*16+ 2, 0x0 +NORM },	/* 2494  */
{     0,  2496, 4*16+ 1, 0x10+NORM },	/* 2495  */
{     0,  2497, 1*16+ 2, 0x0 +NORM },	/* 2496  */
{     0,  2498, 1*16+ 3, 0x10+NORM },	/* 2497  */
{     0,     0, 5*16+ 1, 0x0 +NORM },	/* 2498  */
{  2493,  2500, 2*16+ 8, 0x10+BAD  },	/* 2499 dia 14 */
{     0,  2501, 1*16+ 2, 0x0 +NORM },	/* 2500  */
{     0,  2502, 1*16+ 3, 0x10+NORM },	/* 2501  */
{     0,  2503, 2*16+ 2, 0x0 +NORM },	/* 2502  */
{     0,  2504, 1*16+ 7, 0x10+NORM },	/* 2503  */
{     0,  2505, 4*16+ 2, 0x0 +NORM },	/* 2504  */
{     0,     0, 1*16+ 6, 0x10+NORM },	/* 2505  */
{     0,  2507, 4*16+ 2, 0x0 +NORM },	/* 2506 dia 15 */
{     0,  2508, 2*16+ 8, 0x10+NORM },	/* 2507  */
{     0,  2509, 2*16+ 2, 0x0 +NORM },	/* 2508  */
{     0,     0, 1*16+ 3, 0x10+NORM },	/* 2509  */
{  2474,  2511, 6*16+ 4, 0x0 +NORM },	/* 2510 dia 16 */
{     0,  2512, 7*16+ 5, 0x10+NORM },	/* 2511  */
{     0,  2513, 3*16+ 4, 0x0 +NORM },	/* 2512  */
{     0,  2519, 3*16+ 3, 0x10+NORM },	/* 2513  */
{     0,  2515, 8*16+ 3, 0x0 +NORM },	/* 2514  */
{     0,  2516, 3*16+ 6, 0x10+NORM },	/* 2515  */
{     0,  2517, 4*16+ 6, 0x0 +NORM },	/* 2516  */
{     0,  2518, 2*16+ 5, 0x10+NORM },	/* 2517  */
{     0,     0, 9*16+ 4, 0x0 +NORM },	/* 2518  */
{  2514,  2520, 2*16+ 4, 0x0 +NORM },	/* 2519 dia 17 */
{     0,  2521, 5*16+ 3, 0x10+NORM },	/* 2520  */
{     0,  2522, 6*16+ 3, 0x0 +NORM },	/* 2521  */
{     0,  2523, 6*16+ 2, 0x10+NORM },	/* 2522  */
{     0,  2524, 7*16+ 2, 0x0 +NORM },	/* 2523  */
{     0,  2525, 6*16+ 1, 0x10+NORM },	/* 2524  */
{     0,  2526, 8*16+ 4, 0x0 +NORM },	/* 2525  */
{     0,  2527, 4*16+ 2, 0x10+NORM },	/* 2526  */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 2527  */
{  1897,  2529, 5*16+ 3, 0x10+NORM },	/* 2528 dia 18 crawl */
{     0,  2530, 6*16+ 4, 0x0 +NORM },	/* 2529  */
{     0,  2531, 7*16+ 3, 0x10+NORM },	/* 2530  */
{     0,  2532, 3*16+10, 0x0 +NORM },	/* 2531  */
{     0,  2533, 5*16+ 7, 0x10+NORM },	/* 2532  */
{     0,  2534, 7*16+ 4, 0x0 +NORM },	/* 2533  */
{     0,  2535, 8*16+ 3, 0x10+NORM },	/* 2534  */
{     0,  2536, 8*16+ 4, 0x0 +NORM },	/* 2535  */
{     0,  2537, 9*16+ 4, 0x10+NORM },	/* 2536  */
{     0,  2538, 6*16+ 3, 0x0 +NORM },	/* 2537  */
{     0,  2539, 6*16+ 2, 0x10+NORM },	/* 2538  */
{     0,  2540, 7*16+ 2, 0x0 +NORM },	/* 2539  */
{     0,  2541, 8*16+ 2, 0x10+NORM },	/* 2540  */
{     0,  2542, 5*16+ 2, 0x0 +NORM },	/* 2541  */
{     0,  2543, 7*16+ 1, 0x10+NORM },	/* 2542  */
{     0,  2544, 3*16+ 3, 0x0 +NORM },	/* 2543  */
{     0,  2545, 4*16+ 2, 0x10+NORM },	/* 2544  */
{     0,  2546, 9*16+ 5, 0x0 +NORM },	/* 2545  */
{     0,     0, 5*16+ 9, 0x10+NORM },	/* 2546  */
{  2528,  2548, 7*16+ 3, 0x10+NORM },	/* 2547 dia 20 19 pt trick move */
{     0,  2549, 5*16+ 3, 0x0 +NORM },	/* 2548  */
{     0,  2577, 5*16+ 2, 0x10+NORM },	/* 2549  */
{     0,  2551, 6*16+ 2, 0x0 +BAD  },	/* 2550  */
{     0,  2552, 4*16+ 2, 0x10+NORM },	/* 2551  */
{     0,  2553, 6*16+ 3, 0x0 +NORM },	/* 2552  */
{     0,  2554, 4*16+ 4, 0x10+NORM },	/* 2553  */
{     0,  2555, 4*16+ 5, 0x0 +NORM },	/* 2554  */
{     0,  2556, 5*16+ 5, 0x10+NORM },	/* 2555  */
{     0,  2557, 6*16+ 5, 0x0 +NORM },	/* 2556  */
{     0,  2558, 5*16+ 6, 0x10+NORM },	/* 2557  */
{     0,  2559, 2*16+ 3, 0x0 +NORM },	/* 2558  */
{     0,  2560, 3*16+ 4, 0x10+NORM },	/* 2559  */
{     0,  2561, 2*16+ 4, 0x0 +NORM },	/* 2560  */
{     0,  2562, 2*16+ 5, 0x10+NORM },	/* 2561  */
{     0,  2563, 2*16+ 6, 0x0 +NORM },	/* 2562  */
{     0,  2564, 3*16+ 6, 0x10+NORM },	/* 2563  */
{     0,  2565, 1*16+ 5, 0x0 +NORM },	/* 2564  */
{     0,  2566, 4*16+ 6, 0x10+NORM },	/* 2565  */
{     0,  2567, 2*16+ 5, 0x0 +NORM },	/* 2566  */
{     0,  2568, 7*16+ 2, 0x10+NORM },	/* 2567  */
{     0,  2569, 6*16+ 1, 0x0 +NORM },	/* 2568  */
{     0,  2570, 7*16+ 4, 0x10+NORM },	/* 2569  */
{     0,  2571, 6*16+ 4, 0x0 +NORM },	/* 2570  */
{     0,  2572, 7*16+ 6, 0x10+NORM },	/* 2571  */
{     0,  2573, 3*16+ 2, 0x0 +NORM },	/* 2572  */
{     0,  2574, 7*16+ 5, 0x10+NORM },	/* 2573  */
{     0,  2575, 3*16+ 3, 0x0 +NORM },	/* 2574  */
{     0,  2576, 6*16+ 6, 0x10+NORM },	/* 2575  */
{     0,     0, 4*16+ 1, 0x0 +NORM },	/* 2576  */
{  2550,  2578, 5*16+ 6, 0x0 +NORM },	/* 2577 dia 23 */
{     0,  2579, 3*16+ 2, 0x10+NORM },	/* 2578  */
{     0,     0, 4*16+ 8, 0x0 +NORM },	/* 2579  */
{   464,  2581, 2*16+ 4, 0x0 +NORM },	/* 2580 Ishida V1 CIV sec 2 dia 25 */
{     0,  2582, 2*16+ 3, 0x10+NORM },	/* 2581  */
{     0,  2583, 4*16+ 4, 0x0 +NORM },	/* 2582  */
{     0,  2584, 3*16+ 3, 0x10+NORM },	/* 2583  */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 2584  */
{   465,  2586, 4*16+ 7, 0x10+BAD  },	/* 2585 38 basic jos pg 96 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 2586  */
{     0,  2588, 3*16+11, 0x10+NORM },	/* 2587 Ishida V1 pg 28 dia 26 */
{     0,  2589, 4*16+ 6, 0x0 +NORM },	/* 2588  */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 2589  */
{  2587,  2591, 8*16+ 4, 0x0 +NORM },	/* 2590 dia 26 */
{     0,  2592, 6*16+ 5, 0x10+NORM },	/* 2591  */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 2592  */
{  2590,  2594, 5*16+12, 0x0 +NORM },	/* 2593 dia 27 */
{     0,     0, 3*16+12, 0x10+NORM },	/* 2594  */
{  2593,  2618, 3*16+10, 0x10+NORM },	/* 2595 dia 28 */
{     0,  2600, 6*16+ 5, 0x0 +NORM },	/* 2596  */
{     0,  2598, 7*16+ 4, 0x10+NORM },	/* 2597  */
{     0,  2599, 4*16+11, 0x0 +NORM },	/* 2598  */
{     0,     0, 3*16+12, 0x10+NORM },	/* 2599  */
{  2597,  2601, 4*16+11, 0x10+NORM },	/* 2600 dia 29 */
{     0,  2602, 7*16+ 4, 0x0 +NORM },	/* 2601  */
{     0,  2603, 7*16+ 3, 0x10+NORM },	/* 2602  */
{     0,  2604, 5*16+ 4, 0x0 +NORM },	/* 2603  */
{     0,  2605, 6*16+ 3, 0x10+NORM },	/* 2604  */
{     0,  2606, 5*16+ 3, 0x0 +NORM },	/* 2605  */
{     0,  2607, 5*16+ 2, 0x10+NORM },	/* 2606  */
{     0,     0, 8*16+ 3, 0x0 +NORM },	/* 2607  */
{  2596,  2609, 4*16+11, 0x0 +BAD  },	/* 2608 ref fig 2 */
{     0,  2610, 5*16+ 6, 0x10+NORM },	/* 2609  */
{     0,  2611, 5*16+ 4, 0x0 +NORM },	/* 2610  */
{     0,  2612, 5*16+ 3, 0x10+NORM },	/* 2611  */
{     0,  2613, 6*16+ 5, 0x0 +NORM },	/* 2612  */
{     0,  2614, 6*16+ 7, 0x10+NORM },	/* 2613  */
{     0,  2615, 7*16+ 4, 0x0 +NORM },	/* 2614  */
{     0,  2616, 6*16+ 3, 0x10+NORM },	/* 2615  */
{     0,  2617, 7*16+ 5, 0x0 +NORM },	/* 2616  */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 2617  */
{  2608,  2633, 5*16+ 6, 0x10+NORM },	/* 2618 dia 30 */
{     0,  2620, 6*16+ 6, 0x0 +NORM },	/* 2619  */
{     0,  2621, 4*16+ 6, 0x10+NORM },	/* 2620  */
{     0,  2622, 4*16+ 5, 0x0 +NORM },	/* 2621  */
{     0,     0, 6*16+ 5, 0x10+NORM },	/* 2622  */
{  2619,  2624, 5*16+ 4, 0x0 +NORM },	/* 2623 ref 2 */
{     0,  2625, 5*16+ 3, 0x10+NORM },	/* 2624  */
{     0,  2626, 6*16+ 5, 0x0 +NORM },	/* 2625  */
{     0,  2653, 6*16+ 7, 0x10+NORM },	/* 2626  */
{     0,  2628, 7*16+ 4, 0x0 +NORM },	/* 2627  */
{     0,  2629, 6*16+ 3, 0x10+NORM },	/* 2628  */
{     0,  2630, 7*16+ 5, 0x0 +NORM },	/* 2629  */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 2630  */
{  2623,  2632, 6*16+ 5, 0x0 +NORM },	/* 2631 dia 30 */
{     0,     0, 4*16+ 5, 0x10+NORM },	/* 2632  */
{  2631,  2634, 4*16+ 6, 0x0 +NORM },	/* 2633 dia 31 */
{     0,  2635, 6*16+ 6, 0x10+NORM },	/* 2634  */
{     0,  2636, 6*16+ 5, 0x0 +NORM },	/* 2635  */
{     0,  2637, 7*16+ 5, 0x10+NORM },	/* 2636  */
{     0,  2638, 7*16+ 6, 0x0 +NORM },	/* 2637  */
{     0,  2639, 6*16+ 7, 0x10+NORM },	/* 2638  */
{     0,  2640, 7*16+ 4, 0x0 +NORM },	/* 2639  */
{     0,  2641, 8*16+ 5, 0x10+NORM },	/* 2640  */
{     0,  2642, 5*16+ 4, 0x0 +NORM },	/* 2641  */
{     0,  2643, 6*16+ 3, 0x10+NORM },	/* 2642  */
{     0,  2644, 5*16+ 3, 0x0 +NORM },	/* 2643  */
{     0,  2645, 5*16+ 2, 0x10+NORM },	/* 2644  */
{     0,  2646, 6*16+ 2, 0x0 +NORM },	/* 2645  */
{     0,  2647, 7*16+ 3, 0x10+NORM },	/* 2646  */
{     0,  2648, 4*16+ 2, 0x0 +NORM },	/* 2647  */
{     0,  2649, 4*16+ 5, 0x10+NORM },	/* 2648  */
{     0,  2650, 5*16+ 1, 0x0 +NORM },	/* 2649  */
{     0,  2651, 4*16+ 4, 0x10+NORM },	/* 2650  */
{     0,  2652, 5*16+ 2, 0x0 +NORM },	/* 2651  */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 2652  */
{  2627,  2654, 5*16+ 8, 0x0 +NORM },	/* 2653  */
{     0,  2655, 4*16+ 5, 0x10+NORM },	/* 2654  */
{     0,  2656, 4*16+ 4, 0x0 +NORM },	/* 2655  */
{     0,  2657, 4*16+ 6, 0x10+NORM },	/* 2656  */
{     0,  2658, 3*16+ 3, 0x0 +NORM },	/* 2657  */
{     0,  2659, 7*16+ 5, 0x10+NORM },	/* 2658  */
{     0,  2660, 2*16+ 4, 0x0 +NORM },	/* 2659  */
{     0,  2661, 6*16+ 6, 0x10+NORM },	/* 2660  */
{     0,  2662, 3*16+ 4, 0x0 +NORM },	/* 2661  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 2662  */
{     0,  2664, 5*16+ 6, 0x10+NORM },	/* 2663 dia 35 */
{     0,  2665, 6*16+ 7, 0x0 +NORM },	/* 2664  */
{     0,  2666, 4*16+ 6, 0x10+NORM },	/* 2665  */
{     0,  2667, 6*16+ 6, 0x0 +NORM },	/* 2666  */
{     0,  2668, 4*16+ 5, 0x10+NORM },	/* 2667  */
{     0,  2669, 2*16+ 7, 0x0 +NORM },	/* 2668  */
{     0,  2670, 3*16+ 4, 0x10+NORM },	/* 2669  */
{     0,     0, 5*16+ 9, 0x0 +NORM },	/* 2670  */
{   474,  2672, 6*16+ 5, 0x0 +NORM },	/* 2671 dia 35 */
{     0,  2673, 7*16+ 4, 0x10+NORM },	/* 2672  */
{     0,  2674, 3*16+ 8, 0x0 +NORM },	/* 2673  */
{     0,  2675, 5*16+ 6, 0x10+NORM },	/* 2674  */
{     0,  2676, 6*16+ 7, 0x0 +NORM },	/* 2675  */
{     0,  2677, 4*16+ 6, 0x10+NORM },	/* 2676  */
{     0,  2678, 6*16+ 6, 0x0 +NORM },	/* 2677  */
{     0,  2679, 4*16+ 5, 0x10+NORM },	/* 2678  */
{     0,  2680, 2*16+ 7, 0x0 +NORM },	/* 2679  */
{     0,  2681, 3*16+ 4, 0x10+NORM },	/* 2680  */
{     0,     0, 5*16+ 9, 0x0 +NORM },	/* 2681  */
{  2663,  2683, 2*16+ 8, 0x10+BAD  },	/* 2682 dia 36 */
{     0,  2684, 2*16+ 7, 0x0 +NORM },	/* 2683  */
{     0,  2685, 2*16+ 6, 0x10+NORM },	/* 2684  */
{     0,  2686, 3*16+ 6, 0x0 +NORM },	/* 2685  */
{     0,  2687, 1*16+ 7, 0x10+NORM },	/* 2686  */
{     0,  2688, 4*16+ 6, 0x0 +NORM },	/* 2687  */
{     0,  2689, 2*16+ 7, 0x10+NORM },	/* 2688  */
{     0,     0, 3*16+10, 0x0 +NORM },	/* 2689  */
{  2585,  2691, 6*16+ 5, 0x10+NORM },	/* 2690 dia 37 */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 2691  */
{   483,     0, 3*16+11, 0x0 +NORM },	/* 2692 dia 39 */
{   481,  2694, 7*16+ 5, 0x0 +NORM },	/* 2693 dia 40 */
{     0,  2695, 7*16+ 4, 0x10+NORM },	/* 2694  */
{     0,  2696, 8*16+ 5, 0x0 +NORM },	/* 2695  */
{     0,  2697, 5*16+ 6, 0x10+NORM },	/* 2696  */
{     0,  2698, 6*16+ 7, 0x0 +NORM },	/* 2697  */
{     0,  2699, 4*16+ 6, 0x10+NORM },	/* 2698  */
{     0,  2700, 3*16+ 3, 0x0 +NORM },	/* 2699  */
{     0,  2701, 4*16+ 5, 0x10+NORM },	/* 2700  */
{     0,  2702, 4*16+ 2, 0x0 +NORM },	/* 2701  */
{     0,  2703, 4*16+ 4, 0x10+NORM },	/* 2702  */
{     0,  2706, 8*16+ 7, 0x0 +NORM },	/* 2703  */
{     0,  2705, 5*16+ 2, 0x10+NORM },	/* 2704  */
{     0,     0, 3*16+ 4, 0x0 +NORM },	/* 2705  */
{  2704,  2707, 3*16+ 4, 0x10+NORM },	/* 2706 dia 41 */
{     0,  2708, 2*16+ 2, 0x0 +NORM },	/* 2707  */
{     0,  2709, 2*16+ 3, 0x10+NORM },	/* 2708  */
{     0,  2710, 3*16+ 2, 0x0 +NORM },	/* 2709  */
{     0,  2711, 2*16+ 4, 0x10+NORM },	/* 2710  */
{     0,  2712, 6*16+ 2, 0x0 +NORM },	/* 2711  */
{     0,  2713, 7*16+ 2, 0x10+NORM },	/* 2712  */
{     0,  2714, 8*16+ 3, 0x0 +NORM },	/* 2713  */
{     0,  2715, 7*16+ 3, 0x10+NORM },	/* 2714  */
{     0,  2716, 5*16+ 3, 0x0 +NORM },	/* 2715  */
{     0,  2717, 5*16+ 4, 0x10+NORM },	/* 2716  */
{     0,  2718, 1*16+ 2, 0x0 +NORM },	/* 2717  */
{     0,     0, 6*16+ 8, 0x10+NORM },	/* 2718  */
{   480,  2733, 8*16+ 5, 0x10+NORM },	/* 2719 dia 42 */
{     0,  2721, 5*16+ 4, 0x0 +NORM },	/* 2720  */
{     0,  2722, 5*16+ 3, 0x10+NORM },	/* 2721  */
{     0,  2723, 7*16+ 5, 0x0 +NORM },	/* 2722  */
{     0,  2724, 7*16+ 6, 0x10+NORM },	/* 2723  */
{     0,  2725, 7*16+ 4, 0x0 +NORM },	/* 2724  */
{     0,  2726, 6*16+ 3, 0x10+NORM },	/* 2725  */
{     0,  2727, 6*16+ 7, 0x0 +NORM },	/* 2726  */
{     0,  2728, 9*16+ 4, 0x10+NORM },	/* 2727  */
{     0,  2729, 8*16+ 6, 0x0 +NORM },	/* 2728  */
{     0,  2730, 9*16+ 5, 0x10+NORM },	/* 2729  */
{     0,  2731, 7*16+ 7, 0x0 +NORM },	/* 2730  */
{     0,  2732, 8*16+ 3, 0x10+NORM },	/* 2731  */
{     0,     0, 3*16+11, 0x0 +NORM },	/* 2732  */
{  2720,  2734,10*16+ 4, 0x0 +NORM },	/* 2733 dia 42 */
{     0,  2735, 7*16+ 6, 0x10+NORM },	/* 2734  */
{     0,  2736, 6*16+ 7, 0x0 +NORM },	/* 2735  */
{     0,  2737, 3*16+ 4, 0x10+NORM },	/* 2736  */
{     0,  2738, 3*16+ 6, 0x0 +NORM },	/* 2737  */
{     0,     0, 3*16+11, 0x10+NORM },	/* 2738  */
{   495,  2740, 2*16+ 9, 0x0 +BAD  },	/* 2739 sec 3 dia 46 */
{     0,  2741, 2*16+10, 0x10+NORM },	/* 2740  */
{     0,  2745, 3*16+10, 0x0 +NORM },	/* 2741  */
{     0,  2743, 2*16+11, 0x10+NORM },	/* 2742  */
{     0,  2744, 2*16+ 7, 0x0 +NORM },	/* 2743  */
{     0,     0, 4*16+11, 0x10+NORM },	/* 2744  */
{  2742,  2746, 3*16+11, 0x10+NORM },	/* 2745 dia 46 */
{     0,  2747, 2*16+ 7, 0x0 +NORM },	/* 2746  */
{     0,     0, 4*16+10, 0x10+NORM },	/* 2747  */
{   496,  2813, 3*16+ 9, 0x10+NORM },	/* 2748 dia 48 */
{     0,  2750, 6*16+ 4, 0x0 +NORM },	/* 2749  */
{     0,  2751, 6*16+ 3, 0x10+NORM },	/* 2750  */
{     0,  2752, 7*16+ 3, 0x0 +NORM },	/* 2751  */
{     0,  2753, 7*16+ 4, 0x10+NORM },	/* 2752  */
{     0,  2754, 5*16+ 3, 0x0 +NORM },	/* 2753  */
{     0,  2755, 6*16+ 2, 0x10+NORM },	/* 2754  */
{     0,  2761, 6*16+ 5, 0x0 +NORM },	/* 2755  */
{     0,  2757, 5*16+ 2, 0x10+NORM },	/* 2756  */
{     0,  2758, 8*16+ 4, 0x0 +NORM },	/* 2757  */
{     0,  2759, 2*16+ 3, 0x10+NORM },	/* 2758  */
{     0,  2760, 7*16+ 5, 0x0 +NORM },	/* 2759  */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 2760  */
{  2756,  2762, 8*16+ 3, 0x10+NORM },	/* 2761 dia 49 */
{     0,  2763, 5*16+ 2, 0x0 +NORM },	/* 2762  */
{     0,  2764, 7*16+ 2, 0x10+NORM },	/* 2763  */
{     0,     0, 4*16+ 4, 0x0 +NORM },	/* 2764  */
{  2749,  2775, 7*16+ 4, 0x0 +NORM },	/* 2765 dia 50 */
{     0,  2767, 3*16+ 4, 0x10+NORM },	/* 2766  */
{     0,  2768, 2*16+ 4, 0x0 +NORM },	/* 2767  */
{     0,  2769, 2*16+ 3, 0x10+NORM },	/* 2768  */
{     0,  2770, 4*16+ 4, 0x0 +NORM },	/* 2769  */
{     0,  2771, 3*16+ 3, 0x10+NORM },	/* 2770  */
{     0,  2772, 5*16+ 4, 0x0 +NORM },	/* 2771  */
{     0,  2773, 6*16+ 2, 0x10+NORM },	/* 2772  */
{     0,  2774, 8*16+ 3, 0x0 +NORM },	/* 2773  */
{     0,     0, 6*16+ 8, 0x10+NORM },	/* 2774  */
{  2766,  2776, 5*16+ 4, 0x10+NORM },	/* 2775 dia 51 */
{     0,  2777, 6*16+ 6, 0x0 +NORM },	/* 2776  */
{     0,  2778, 3*16+ 4, 0x10+NORM },	/* 2777  */
{     0,  2779, 2*16+ 5, 0x0 +NORM },	/* 2778  */
{     0,  2780, 2*16+ 4, 0x10+NORM },	/* 2779  */
{     0,  2781, 6*16+ 8, 0x0 +NORM },	/* 2780  */
{     0,     0, 7*16+ 2, 0x10+NORM },	/* 2781  */
{  2765,  2783, 6*16+ 3, 0x0 +NORM },	/* 2782 dia 53 */
{     0,  2802, 6*16+ 4, 0x10+NORM },	/* 2783  */
{     0,  2785, 7*16+ 4, 0x0 +NORM },	/* 2784  */
{     0,  2793, 5*16+ 4, 0x10+NORM },	/* 2785  */
{     0,  2787, 7*16+ 5, 0x0 +NORM },	/* 2786  */
{     0,  2788, 7*16+ 3, 0x10+NORM },	/* 2787  */
{     0,  2789, 8*16+ 3, 0x0 +NORM },	/* 2788  */
{     0,  2790, 7*16+ 2, 0x10+NORM },	/* 2789  */
{     0,  2791, 8*16+ 2, 0x0 +NORM },	/* 2790  */
{     0,  2792, 6*16+ 2, 0x10+NORM },	/* 2791  */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 2792  */
{  2786,  2794, 6*16+ 7, 0x0 +NORM },	/* 2793 dia 52 */
{     0,  2795, 7*16+ 3, 0x10+NORM },	/* 2794  */
{     0,  2796, 8*16+ 3, 0x0 +NORM },	/* 2795  */
{     0,  2800, 7*16+ 2, 0x10+NORM },	/* 2796  */
{     0,  2798, 8*16+ 2, 0x0 +NORM },	/* 2797  */
{     0,  2799, 6*16+ 2, 0x10+NORM },	/* 2798  */
{     0,     0, 7*16+ 5, 0x0 +NORM },	/* 2799  */
{  2797,  2801, 8*16+ 4, 0x0 +NORM },	/* 2800 dia 52 */
{     0,     0, 6*16+ 2, 0x10+NORM },	/* 2801  */
{  2784,  2803, 5*16+ 3, 0x0 +BAD  },	/* 2802 dia 53 */
{     0,  2804, 5*16+ 4, 0x10+NORM },	/* 2803  */
{     0,  2805, 4*16+ 4, 0x0 +NORM },	/* 2804  */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 2805  */
{  2782,  2807, 7*16+ 3, 0x0 +BAD  },	/* 2806 dia 54 */
{     0,  2808, 6*16+ 4, 0x10+NORM },	/* 2807  */
{     0,  2809, 7*16+ 4, 0x0 +NORM },	/* 2808  */
{     0,  2810, 6*16+ 5, 0x10+NORM },	/* 2809  */
{     0,  2811, 6*16+ 3, 0x0 +NORM },	/* 2810  */
{     0,  2812, 5*16+ 3, 0x10+NORM },	/* 2811  */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 2812  */
{  2806,  2831, 5*16+ 3, 0x0 +NORM },	/* 2813 dia 55 */
{     0,  2815, 5*16+ 2, 0x10+NORM },	/* 2814  */
{     0,  2816, 4*16+ 4, 0x0 +NORM },	/* 2815  */
{     0,  2817, 6*16+ 3, 0x10+NORM },	/* 2816  */
{     0,  2818, 5*16+ 4, 0x0 +NORM },	/* 2817  */
{     0,  2824, 3*16+ 3, 0x10+NORM },	/* 2818  */
{     0,  2820, 4*16+ 2, 0x0 +NORM },	/* 2819  */
{     0,  2821, 3*16+ 2, 0x10+NORM },	/* 2820  */
{     0,  2822, 6*16+ 2, 0x0 +NORM },	/* 2821  */
{     0,  2823, 4*16+ 1, 0x10+NORM },	/* 2822  */
{     0,     0, 7*16+ 3, 0x0 +NORM },	/* 2823  */
{  2819,  2825, 6*16+ 2, 0x0 +NORM },	/* 2824 dia 55 */
{     0,  2826, 7*16+ 2, 0x10+NORM },	/* 2825  */
{     0,  2827, 4*16+ 2, 0x0 +NORM },	/* 2826  */
{     0,  2828, 6*16+ 1, 0x10+NORM },	/* 2827  */
{     0,  2829, 3*16+ 2, 0x0 +NORM },	/* 2828  */
{     0,  2830, 2*16+ 3, 0x10+NORM },	/* 2829  */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 2830  */
{  2814,  2832, 4*16+ 4, 0x10+BAD  },	/* 2831 dia 56 */
{     0,  2843, 5*16+ 4, 0x0 +NORM },	/* 2832  */
{     0,  2834, 4*16+ 5, 0x10+NORM },	/* 2833  */
{     0,  2835, 5*16+ 5, 0x0 +NORM },	/* 2834  */
{     0,  2836, 3*16+ 6, 0x10+NORM },	/* 2835  */
{     0,  2837, 4*16+ 2, 0x0 +NORM },	/* 2836  */
{     0,  2838, 2*16+ 4, 0x10+NORM },	/* 2837  */
{     0,  2839, 2*16+ 3, 0x0 +NORM },	/* 2838  */
{     0,  2840, 3*16+ 4, 0x10+NORM },	/* 2839  */
{     0,  2841, 3*16+ 3, 0x0 +NORM },	/* 2840  */
{     0,  2842, 2*16+ 5, 0x10+NORM },	/* 2841  */
{     0,     0,10*16+ 3, 0x0 +NORM },	/* 2842  */
{  2833,  2844, 5*16+ 2, 0x10+NORM },	/* 2843 dia 57 */
{     0,  2845, 6*16+ 2, 0x0 +NORM },	/* 2844  */
{     0,  2846, 5*16+ 5, 0x10+NORM },	/* 2845  */
{     0,  2847, 4*16+ 5, 0x0 +NORM },	/* 2846  */
{     0,  2848, 4*16+ 2, 0x10+NORM },	/* 2847  */
{     0,     0, 7*16+ 4, 0x0 +NORM },	/* 2848  */
{   489,  2850, 5*16+ 7, 0x0 +NORM },	/* 2849 sec 4 attach and extend */
{     0,  2961, 3*16+ 6, 0x10+NORM },	/* 2850  */
{     0,  2946, 4*16+ 5, 0x0 +NORM },	/* 2851  */
{     0,  2853, 6*16+ 4, 0x10+NORM },	/* 2852  */
{     0,  2854, 6*16+ 5, 0x0 +NORM },	/* 2853  */
{     0,  2855, 7*16+ 4, 0x10+NORM },	/* 2854  */
{     0,  2856, 4*16+10, 0x0 +NORM },	/* 2855  */
{     0,  2857, 5*16+ 8, 0x10+NORM },	/* 2856  */
{     0,  2858, 6*16+ 8, 0x0 +NORM },	/* 2857  */
{     0,  2859, 5*16+ 9, 0x10+NORM },	/* 2858  */
{     0,  2860, 3*16+ 3, 0x0 +NORM },	/* 2859  */
{     0,  2861, 3*16+ 2, 0x10+NORM },	/* 2860  */
{     0,  2887, 4*16+ 2, 0x0 +NORM },	/* 2861  */
{     0,  2881, 2*16+ 3, 0x10+NORM },	/* 2862  */
{     0,  2864, 2*16+ 5, 0x0 +NORM },	/* 2863  */
{     0,  2865, 5*16+ 3, 0x10+NORM },	/* 2864  */
{     0,  2866, 3*16+ 8, 0x0 +NORM },	/* 2865  */
{     0,  2867, 3*16+ 9, 0x10+NORM },	/* 2866  */
{     0,  2868, 2*16+ 8, 0x0 +NORM },	/* 2867  */
{     0,  2869, 3*16+10, 0x10+NORM },	/* 2868  */
{     0,  2870, 2*16+ 7, 0x0 +NORM },	/* 2869  */
{     0,     0, 4*16+11, 0x10+NORM },	/* 2870  */
{  2863,  2872, 3*16+ 4, 0x0 +BAD  },	/* 2871 dia 61 */
{     0,  2873, 5*16+ 2, 0x10+NORM },	/* 2872  */
{     0,  2874, 2*16+ 2, 0x0 +NORM },	/* 2873  */
{     0,  2875, 4*16+ 1, 0x10+NORM },	/* 2874  */
{     0,  2876, 2*16+ 1, 0x0 +NORM },	/* 2875  */
{     0,  2877, 1*16+ 2, 0x10+NORM },	/* 2876  */
{     0,  2878, 2*16+ 4, 0x0 +NORM },	/* 2877  */
{     0,  2879, 1*16+ 3, 0x10+NORM },	/* 2878  */
{     0,  2880, 1*16+ 4, 0x0 +NORM },	/* 2879  */
{     0,     0, 2*16+ 6, 0x10+NORM },	/* 2880  */
{  2871,  2882, 5*16+ 3, 0x0 +BAD  },	/* 2881 dia 62 */
{     0,  2883, 3*16+ 4, 0x10+NORM },	/* 2882  */
{     0,  2884, 4*16+ 4, 0x0 +NORM },	/* 2883  */
{     0,  2885, 3*16+ 3, 0x10+NORM },	/* 2884  */
{     0,  2886, 5*16+ 4, 0x0 +NORM },	/* 2885  */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 2886  */
{  2862,  2888, 5*16+ 2, 0x10+BAD  },	/* 2887 dia 63 */
{     0,  2889, 5*16+ 3, 0x0 +NORM },	/* 2888  */
{     0,  2890, 4*16+ 1, 0x10+NORM },	/* 2889  */
{     0,  2891, 4*16+ 4, 0x0 +NORM },	/* 2890  */
{     0,  2892, 4*16+ 2, 0x10+NORM },	/* 2891  */
{     0,  2893, 5*16+ 4, 0x0 +NORM },	/* 2892  */
{     0,  2894, 6*16+ 3, 0x10+NORM },	/* 2893  */
{     0,     0, 4*16+12, 0x0 +NORM },	/* 2894  */
{  2852,  2896, 6*16+ 3, 0x10+NORM },	/* 2895 dia 64 */
{     0,  2945, 5*16+ 4, 0x0 +NORM },	/* 2896  */
{     0,  2898, 3*16+11, 0x10+NORM },	/* 2897  */
{     0,  2899, 5*16+ 3, 0x0 +NORM },	/* 2898  */
{     0,  2900, 5*16+ 2, 0x10+NORM },	/* 2899  */
{     0,  2901, 4*16+ 2, 0x0 +NORM },	/* 2900  */
{     0,  2902, 3*16+ 2, 0x10+NORM },	/* 2901  */
{     0,  2903, 6*16+ 2, 0x0 +NORM },	/* 2902  */
{     0,  2904, 4*16+ 1, 0x10+NORM },	/* 2903  */
{     0,  2905, 7*16+ 3, 0x0 +NORM },	/* 2904  */
{     0,  2906, 2*16+ 3, 0x10+NORM },	/* 2905  */
{     0,     0, 6*16+ 4, 0x0 +NORM },	/* 2906  */
{  2897,  2908, 6*16+ 4, 0x0 +NORM },	/* 2907 dia 66 */
{     0,     0, 2*16+ 3, 0x10+NORM },	/* 2908  */
{  2907,  2910, 2*16+ 3, 0x0 +NORM },	/* 2909 dia 66 */
{     0,  2911, 2*16+ 4, 0x10+NORM },	/* 2910  */
{     0,  2912, 2*16+ 5, 0x0 +NORM },	/* 2911  */
{     0,  2913, 2*16+ 2, 0x10+NORM },	/* 2912  */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 2913  */
{  2909,  2915, 3*16+ 3, 0x0 +BAD  },	/* 2914 dia 67 */
{     0,  2916, 3*16+ 4, 0x10+NORM },	/* 2915  */
{     0,  2917, 4*16+ 4, 0x0 +NORM },	/* 2916  */
{     0,  2918, 2*16+ 5, 0x10+NORM },	/* 2917  */
{     0,     0, 4*16+ 2, 0x0 +NORM },	/* 2918  */
{  2914,  2920, 6*16+ 2, 0x10+NORM },	/* 2919 dia 68 */
{     0,  2921, 3*16+ 3, 0x0 +NORM },	/* 2920  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 2921  */
{  2919,  2923, 6*16+ 2, 0x0 +BAD  },	/* 2922 dia 69 */
{     0,  2924, 7*16+ 2, 0x10+NORM },	/* 2923  */
{     0,  2925, 4*16+ 2, 0x0 +NORM },	/* 2924  */
{     0,  2926, 6*16+ 1, 0x10+NORM },	/* 2925  */
{     0,     0, 3*16+ 3, 0x0 +NORM },	/* 2926  */
{  2922,  2934, 7*16+ 4, 0x0 +NORM },	/* 2927 dia 72 */
{     0,  2929, 7*16+ 3, 0x10+NORM },	/* 2928  */
{     0,  2930, 5*16+ 3, 0x0 +NORM },	/* 2929  */
{     0,  2931, 5*16+ 2, 0x10+NORM },	/* 2930  */
{     0,  2932, 8*16+ 3, 0x0 +NORM },	/* 2931  */
{     0,  2933, 8*16+ 2, 0x10+NORM },	/* 2932  */
{     0,     0, 8*16+ 4, 0x0 +NORM },	/* 2933  */
{  2928,  2935, 8*16+ 3, 0x10+NORM },	/* 2934 dia 73 */
{     0,  2936, 5*16+ 3, 0x0 +NORM },	/* 2935  */
{     0,  2937, 5*16+ 2, 0x10+NORM },	/* 2936  */
{     0,  2938, 4*16+ 2, 0x0 +NORM },	/* 2937  */
{     0,  2939, 6*16+ 2, 0x10+NORM },	/* 2938  */
{     0,  2940, 3*16+ 3, 0x0 +NORM },	/* 2939  */
{     0,     0,11*16+ 3, 0x10+NORM },	/* 2940  */
{  2927,  2942, 8*16+ 3, 0x0 +NORM },	/* 2941 dia 75 */
{     0,  2943, 5*16+ 3, 0x10+NORM },	/* 2942  */
{     0,  2944, 8*16+ 5, 0x0 +NORM },	/* 2943  */
{     0,     0, 2*16+ 3, 0x10+NORM },	/* 2944  */
{  2941,     0, 3*16+ 2, 0x0 +NORM },	/* 2945 dia 75 */
{  2895,  2947, 4*16+ 6, 0x10+NORM },	/* 2946 dia 76 */
{     0,  2948, 5*16+ 6, 0x0 +NORM },	/* 2947  */
{     0,  2949, 5*16+ 5, 0x10+NORM },	/* 2948  */
{     0,  2950, 5*16+ 4, 0x0 +NORM },	/* 2949  */
{     0,  2951, 6*16+ 5, 0x10+NORM },	/* 2950  */
{     0,  2952, 4*16+ 4, 0x0 +NORM },	/* 2951  */
{     0,  2953, 3*16+ 9, 0x10+NORM },	/* 2952  */
{     0,  2954, 7*16+ 7, 0x0 +NORM },	/* 2953  */
{     0,  2955, 8*16+ 5, 0x10+NORM },	/* 2954  */
{     0,  2956, 5*16+ 3, 0x0 +NORM },	/* 2955  */
{     0,  2957, 6*16+ 9, 0x10+NORM },	/* 2956  */
{     0,  2958, 7*16+ 6, 0x0 +NORM },	/* 2957  */
{     0,  2959, 7*16+ 5, 0x10+NORM },	/* 2958  */
{     0,  2960, 9*16+ 7, 0x0 +NORM },	/* 2959  */
{     0,     0,10*16+ 5, 0x10+NORM },	/* 2960  */
{  2851,  3005, 5*16+ 4, 0x0 +TRIK },	/* 2961 dia 78 */
{     0,  2963, 5*16+ 3, 0x10+NORM },	/* 2962  */
{     0,  2989, 6*16+ 4, 0x0 +NORM },	/* 2963  */
{     0,  2965, 7*16+ 3, 0x10+BAD  },	/* 2964  */
{     0,  2966, 2*16+ 5, 0x0 +NORM },	/* 2965  */
{     0,  2967, 3*16+ 9, 0x10+NORM },	/* 2966  */
{     0,  2968, 6*16+ 3, 0x0 +NORM },	/* 2967  */
{     0,  2969, 6*16+ 2, 0x10+NORM },	/* 2968  */
{     0,  2974, 5*16+ 2, 0x0 +NORM },	/* 2969  */
{     0,  2971, 4*16+ 2, 0x10+NORM },	/* 2970  */
{     0,  2972, 7*16+ 2, 0x0 +NORM },	/* 2971  */
{     0,  2973, 5*16+ 1, 0x10+NORM },	/* 2972  */
{     0,     0, 8*16+ 3, 0x0 +NORM },	/* 2973  */
{  2970,  2975, 7*16+ 2, 0x10+BAD  },	/* 2974 dia 81 */
{     0,  2976, 4*16+ 2, 0x0 +NORM },	/* 2975  */
{     0,  2977, 3*16+ 3, 0x10+BAD  },	/* 2976  */
{     0,  2978, 3*16+ 2, 0x0 +NORM },	/* 2977  */
{     0,  2979, 2*16+ 2, 0x10+BAD  },	/* 2978  */
{     0,  2980, 2*16+ 3, 0x0 +NORM },	/* 2979  */
{     0,  2981, 2*16+ 4, 0x10+BAD  },	/* 2980  */
{     0,  2982, 1*16+ 3, 0x0 +NORM },	/* 2981  */
{     0,  2983, 1*16+ 4, 0x10+BAD  },	/* 2982  */
{     0,  2984, 3*16+ 4, 0x0 +NORM },	/* 2983  */
{     0,  2985, 1*16+ 2, 0x10+BAD  },	/* 2984  */
{     0,  2986, 2*16+ 3, 0x0 +NORM },	/* 2985  */
{     0,  2987, 1*16+ 3, 0x10+BAD  },	/* 2986  */
{     0,  2988, 4*16+ 4, 0x0 +NORM },	/* 2987  */
{     0,     0, 2*16+ 3, 0x10+BAD  },	/* 2988  */
{  2964,  2990, 6*16+ 3, 0x10+NORM },	/* 2989 dia 82 */
{     0,  2997, 4*16+ 6, 0x0 +NORM },	/* 2990  */
{     0,  2992, 4*16+ 4, 0x10+NORM },	/* 2991  */
{     0,  2993, 4*16+ 5, 0x0 +NORM },	/* 2992  */
{     0,  2994, 7*16+ 4, 0x10+NORM },	/* 2993  */
{     0,  2995, 3*16+ 8, 0x0 +NORM },	/* 2994  */
{     0,  2996, 2*16+ 4, 0x10+NORM },	/* 2995  */
{     0,     0, 2*16+ 5, 0x0 +NORM },	/* 2996  */
{  2991,  2998, 3*16+ 9, 0x10+COMP },	/* 2997 dia 83 */
{     0,  2999, 8*16+ 3, 0x0 +NORM },	/* 2998  */
{     0,  3000, 7*16+ 3, 0x10+NORM },	/* 2999  */
{     0,  3001, 7*16+ 4, 0x0 +NORM },	/* 3000  */
{     0,  3002, 8*16+ 2, 0x10+NORM },	/* 3001  */
{     0,  3003, 9*16+ 3, 0x0 +NORM },	/* 3002  */
{     0,  3004, 9*16+ 2, 0x10+NORM },	/* 3003  */
{     0,     0,10*16+ 3, 0x0 +NORM },	/* 3004  */
{  2962,  3006, 2*16+ 5, 0x10+NORM },	/* 3005 dia 85 */
{     0,  3007, 3*16+ 4, 0x0 +NORM },	/* 3006  */
{     0,  3008, 2*16+ 4, 0x10+NORM },	/* 3007  */
{     0,  3009, 3*16+ 3, 0x0 +NORM },	/* 3008  */
{     0,     0, 5*16+ 8, 0x10+NORM },	/* 3009  */
{  1896,  3018, 3*16+ 3, 0x0 +NORM },	/* 3010 sec 5 3-3 contact play */
{     0,  3065, 3*16+ 2, 0x10+NORM },	/* 3011  */
{     0,  3029, 4*16+ 4, 0x0 +NORM },	/* 3012  */
{     0,  3014, 5*16+ 3, 0x10+NORM },	/* 3013  */
{     0,  3015, 2*16+ 3, 0x0 +NORM },	/* 3014  */
{     0,  3021, 7*16+ 4, 0x10+NORM },	/* 3015  */
{     0,  3017, 2*16+ 2, 0x0 +NORM },	/* 3016  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 3017  */
{  3011,  3019, 4*16+ 4, 0x10+NORM },	/* 3018 dia 86 */
{     0,  3020, 3*16+ 4, 0x0 +NORM },	/* 3019  */
{     0,     0, 4*16+ 5, 0x10+NORM },	/* 3020  */
{  3016,  3022, 2*16+ 2, 0x10+NORM },	/* 3021 dia 87 */
{     0,  3023, 4*16+ 7, 0x0 +NORM },	/* 3022  */
{     0,  3024, 4*16+ 8, 0x10+NORM },	/* 3023  */
{     0,  3025, 4*16+ 6, 0x0 +NORM },	/* 3024  */
{     0,  3026, 3*16+ 9, 0x10+NORM },	/* 3025  */
{     0,  3027, 5*16+ 4, 0x0 +NORM },	/* 3026  */
{     0,  3028, 6*16+ 4, 0x10+NORM },	/* 3027  */
{     0,     0, 6*16+ 6, 0x0 +NORM },	/* 3028  */
{  3013,  3030, 2*16+ 3, 0x10+NORM },	/* 3029 dia 88 */
{     0,  3031, 5*16+ 3, 0x0 +NORM },	/* 3030  */
{     0,  3049, 3*16+ 4, 0x10+NORM },	/* 3031  */
{     0,  3039, 4*16+ 2, 0x0 +NORM },	/* 3032  */
{     0,  3034, 3*16+ 3, 0x10+NORM },	/* 3033  */
{     0,  3035, 5*16+ 5, 0x0 +NORM },	/* 3034  */
{     0,  3036, 4*16+ 6, 0x10+NORM },	/* 3035  */
{     0,  3037, 4*16+ 5, 0x0 +NORM },	/* 3036  */
{     0,  3038, 2*16+ 5, 0x10+NORM },	/* 3037  */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 3038  */
{  3033,  3040, 4*16+ 5, 0x10+NORM },	/* 3039 dia 90 */
{     0,  3041, 3*16+ 3, 0x0 +NORM },	/* 3040  */
{     0,  3042, 2*16+ 4, 0x10+NORM },	/* 3041  */
{     0,  3047, 5*16+ 4, 0x0 +NORM },	/* 3042  */
{     0,  3044, 5*16+ 5, 0x10+NORM },	/* 3043  */
{     0,  3045, 6*16+ 5, 0x0 +NORM },	/* 3044  */
{     0,  3046, 5*16+ 7, 0x10+NORM },	/* 3045  */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 3046  */
{  3043,  3048, 4*16+ 6, 0x10+NORM },	/* 3047 dia 90 */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 3048  */
{  3032,  3054, 2*16+ 4, 0x0 +BAD  },	/* 3049 dia 91 */
{     0,  3051, 3*16+ 3, 0x10+BAD  },	/* 3050  */
{     0,  3052, 5*16+ 5, 0x0 +NORM },	/* 3051  */
{     0,  3053, 5*16+ 2, 0x10+NORM },	/* 3052  */
{     0,     0, 6*16+ 3, 0x0 +NORM },	/* 3053  */
{  3050,  3055, 5*16+ 4, 0x10+NORM },	/* 3054 dia 92 */
{     0,  3056, 3*16+ 3, 0x0 +NORM },	/* 3055  */
{     0,  3059, 4*16+ 2, 0x10+NORM },	/* 3056  */
{     0,  3058, 4*16+ 5, 0x0 +NORM },	/* 3057  */
{     0,     0, 6*16+ 3, 0x10+NORM },	/* 3058  */
{  3057,  3060, 4*16+ 7, 0x0 +NORM },	/* 3059 dia 93 */
{     0,  3061, 5*16+ 5, 0x10+NORM },	/* 3060  */
{     0,  3062, 3*16+ 8, 0x0 +NORM },	/* 3061  */
{     0,  3063, 4*16+ 5, 0x10+NORM },	/* 3062  */
{     0,  3064, 3*16+ 6, 0x0 +NORM },	/* 3063  */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 3064  */
{  3012,  3066, 2*16+ 2, 0x0 +BAD  },	/* 3065 dia 94 */
{     0,  3067, 3*16+ 4, 0x10+NORM },	/* 3066  */
{     0,  3068, 2*16+ 3, 0x0 +NORM },	/* 3067  */
{     0,  3069, 4*16+ 4, 0x10+NORM },	/* 3068  */
{     0,  3070, 2*16+ 4, 0x0 +NORM },	/* 3069  */
{     0,  3071, 4*16+ 5, 0x10+NORM },	/* 3070  */
{     0,  3072, 3*16+ 6, 0x0 +NORM },	/* 3071  */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 3072  */
{  3010,  3097, 5*16+ 6, 0x0 +BAD  },	/* 3073 sec 6 knights move */
{     0,  3084, 6*16+ 4, 0x10+NORM },	/* 3074  */
{     0,  3076, 4*16+ 8, 0x0 +NORM },	/* 3075  */
{     0,  3077, 3*16+ 4, 0x10+NORM },	/* 3076  */
{     0,  3078, 4*16+ 5, 0x0 +NORM },	/* 3077  */
{     0,  3079, 8*16+ 4, 0x10+NORM },	/* 3078  */
{     0,  3080, 3*16+ 8, 0x0 +NORM },	/* 3079  */
{     0,  3081, 2*16+ 5, 0x10+NORM },	/* 3080  */
{     0,  3082, 2*16+ 6, 0x0 +NORM },	/* 3081  */
{     0,  3083, 2*16+ 4, 0x10+NORM },	/* 3082  */
{     0,     0, 2*16+ 7, 0x0 +NORM },	/* 3083  */
{  3075,  3093, 3*16+ 3, 0x0 +NORM },	/* 3084 dia 98 */
{     0,  3086, 4*16+ 4, 0x10+NORM },	/* 3085  */
{     0,  3087, 3*16+ 4, 0x0 +NORM },	/* 3086  */
{     0,  3091, 4*16+ 5, 0x10+NORM },	/* 3087  */
{     0,  3089, 4*16+ 6, 0x0 +NORM },	/* 3088  */
{     0,  3090, 3*16+ 6, 0x10+NORM },	/* 3089  */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 3090  */
{  3088,  3092,10*16+ 4, 0x0 +NORM },	/* 3091 dia 98 */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 3092  */
{  3085,  3094, 3*16+ 2, 0x10+BAD  },	/* 3093 dia 99 */
{     0,  3095, 4*16+ 4, 0x0 +NORM },	/* 3094  */
{     0,  3096, 5*16+ 3, 0x10+NORM },	/* 3095  */
{     0,     0, 4*16+ 8, 0x0 +NORM },	/* 3096  */
{  3074,  3098, 4*16+ 5, 0x10+NORM },	/* 3097 dia 100 */
{     0,  3099, 4*16+ 6, 0x0 +NORM },	/* 3098  */
{     0,  3100, 3*16+ 6, 0x10+NORM },	/* 3099  */
{     0,  3101, 4*16+ 4, 0x0 +NORM },	/* 3100  */
{     0,  3102, 5*16+ 5, 0x10+NORM },	/* 3101  */
{     0,  3103, 5*16+ 4, 0x0 +NORM },	/* 3102  */
{     0,  3104, 6*16+ 5, 0x10+NORM },	/* 3103  */
{     0,  3105, 3*16+ 4, 0x0 +NORM },	/* 3104  */
{     0,  3106, 6*16+ 4, 0x10+NORM },	/* 3105  */
{     0,  3107, 5*16+ 3, 0x0 +NORM },	/* 3106  */
{     0,  3110, 6*16+ 3, 0x10+NORM },	/* 3107  */
{     0,  3109, 5*16+ 2, 0x0 +NORM },	/* 3108  */
{     0,     0, 6*16+ 7, 0x10+NORM },	/* 3109  */
{  3108,  3111, 6*16+ 8, 0x0 +NORM },	/* 3110 dia 101 */
{     0,  3112, 5*16+ 2, 0x10+NORM },	/* 3111  */
{     0,  3113, 4*16+ 2, 0x0 +NORM },	/* 3112  */
{     0,  3114, 6*16+ 2, 0x10+NORM },	/* 3113  */
{     0,  3115, 3*16+ 2, 0x0 +NORM },	/* 3114  */
{     0,  3116, 4*16+ 9, 0x10+NORM },	/* 3115  */
{     0,  3117, 6*16+ 6, 0x0 +NORM },	/* 3116  */
{     0,     0,10*16+ 3, 0x10+NORM },	/* 3117  */
{  3073,  3156, 4*16+ 5, 0x10+NORM },	/* 3118 tenuki sec 7 */
{     0,  3126, 4*16+ 4, 0x0 +NORM },	/* 3119  */
{     0,  3121, 5*16+ 4, 0x10+NORM },	/* 3120  */
{     0,  3122, 3*16+ 4, 0x0 +NORM },	/* 3121  */
{     0,  3123, 4*16+ 6, 0x10+NORM },	/* 3122  */
{     0,  3124, 3*16+ 3, 0x0 +NORM },	/* 3123  */
{     0,  3131, 5*16+ 2, 0x10+NORM },	/* 3124  */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 3125  */
{  3120,  3127, 3*16+ 4, 0x10+NORM },	/* 3126 dia 104 */
{     0,  3128, 5*16+ 4, 0x0 +NORM },	/* 3127  */
{     0,  3129, 3*16+ 3, 0x10+NORM },	/* 3128  */
{     0,  3130, 4*16+ 6, 0x0 +NORM },	/* 3129  */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 3130  */
{  3125,  3132, 4*16+ 2, 0x0 +BAD  },	/* 3131 dia 105 */
{     0,  3133, 5*16+ 3, 0x10+NORM },	/* 3132  */
{     0,  3134, 3*16+ 2, 0x10+NORM },	/* 3133 dia 106 */
{     0,  3135, 2*16+ 2, 0x0 +NORM },	/* 3134  */
{     0,  3136, 4*16+ 1, 0x10+NORM },	/* 3135  */
{     0,  3137, 2*16+ 1, 0x0 +NORM },	/* 3136  */
{     0,  3138, 1*16+ 4, 0x10+NORM },	/* 3137  */
{     0,  3139, 2*16+ 6, 0x0 +NORM },	/* 3138  */
{     0,  3140, 2*16+ 7, 0x10+NORM },	/* 3139  */
{     0,     0, 1*16+ 5, 0x0 +NORM },	/* 3140  */
{  3119,  3142, 3*16+ 4, 0x0 +NORM },	/* 3141 dia 107 */
{     0,  3149, 4*16+ 4, 0x10+NORM },	/* 3142  */
{     0,  3144, 3*16+ 3, 0x0 +NORM },	/* 3143  */
{     0,  3145, 3*16+ 6, 0x10+NORM },	/* 3144  */
{     0,  3146, 4*16+ 2, 0x0 +NORM },	/* 3145  */
{     0,  3147, 5*16+ 2, 0x10+NORM },	/* 3146  */
{     0,  3148, 3*16+ 2, 0x0 +NORM },	/* 3147  */
{     0,     0, 6*16+ 3, 0x10+NORM },	/* 3148  */
{  3143,  3150, 3*16+ 6, 0x0 +BAD  },	/* 3149 dia 108 */
{     0,  3151, 4*16+ 6, 0x10+NORM },	/* 3150  */
{     0,  3152, 2*16+ 7, 0x0 +NORM },	/* 3151  */
{     0,  3153, 3*16+ 8, 0x10+NORM },	/* 3152  */
{     0,  3154, 2*16+ 8, 0x0 +NORM },	/* 3153  */
{     0,  3155, 3*16+ 9, 0x10+NORM },	/* 3154  */
{     0,     0, 3*16+ 3, 0x0 +NORM },	/* 3155  */
{  3141,  3157, 3*16+ 6, 0x10+NORM },	/* 3156 dia 109 */
{     0,  3158, 3*16+ 4, 0x0 +BAD  },	/* 3157  */
{     0,     0, 3*16+ 3, 0x10+NORM },	/* 3158  */
{  3118,  3160, 3*16+ 4, 0x10+NORM },	/* 3159 dia 110 */
{     0,  3161, 4*16+ 5, 0x0 +NORM },	/* 3160  */
{     0,  3162, 6*16+ 4, 0x10+NORM },	/* 3161  */
{     0,  3163, 5*16+ 6, 0x0 +NORM },	/* 3162  */
{     0,  3164, 8*16+ 4, 0x10+NORM },	/* 3163  */
{     0,  3165, 4*16+ 8, 0x0 +NORM },	/* 3164  */
{     0,  3166, 3*16+10, 0x10+NORM },	/* 3165  */
{     0,     0, 3*16+ 8, 0x0 +NORM },	/* 3166  */
{     0,  3168, 6*16+ 5, 0x0 +NORM },	/* 3167 Ish V 1 chapt V 2 space pincer */
{     0,  3171, 7*16+ 3, 0x10+NORM },	/* 3168  */
{     0,  3170, 3*16+11, 0x0 +NORM },	/* 3169  */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 3170  */
{  3169,  3172, 3*16+10, 0x0 +NORM },	/* 3171 dia 2 */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 3172  */
{  3167,  3174, 5*16+ 5, 0x0 +BAD  },	/* 3173 dia 5 */
{     0,  3175, 6*16+ 4, 0x10+NORM },	/* 3174  */
{     0,  3176, 3*16+11, 0x0 +NORM },	/* 3175  */
{     0,  3177, 5*16+ 8, 0x10+NORM },	/* 3176  */
{     0,  3178, 6*16+ 5, 0x0 +NORM },	/* 3177  */
{     0,     0, 7*16+ 4, 0x10+NORM },	/* 3178  */
{  3173,  3243, 5*16+ 4, 0x0 +NORM },	/* 3179 dia 6 */
{     0,  3181, 5*16+ 3, 0x10+NORM },	/* 3180  */
{     0,  3182, 6*16+ 4, 0x0 +NORM },	/* 3181  */
{     0,  3183, 7*16+ 3, 0x10+NORM },	/* 3182  */
{     0,  3184, 3*16+10, 0x0 +NORM },	/* 3183  */
{     0,  3218, 5*16+ 8, 0x10+NORM },	/* 3184  */
{     0,  3186, 7*16+ 4, 0x0 +NORM },	/* 3185  */
{     0,  3187, 8*16+ 3, 0x10+NORM },	/* 3186  */
{     0,  3188, 8*16+ 4, 0x0 +NORM },	/* 3187  */
{     0,  3189, 9*16+ 4, 0x10+NORM },	/* 3188  */
{     0,  3204, 9*16+ 5, 0x0 +NORM },	/* 3189  */
{     0,  3191,10*16+ 4, 0x10+NORM },	/* 3190  */
{     0,  3192, 6*16+ 3, 0x0 +NORM },	/* 3191  */
{     0,  3193, 6*16+ 2, 0x10+NORM },	/* 3192  */
{     0,  3198, 7*16+ 2, 0x0 +NORM },	/* 3193  */
{     0,  3195, 5*16+ 2, 0x10+NORM },	/* 3194  */
{     0,  3196, 5*16+10, 0x0 +NORM },	/* 3195  */
{     0,  3197, 7*16+ 8, 0x10+NORM },	/* 3196  */
{     0,     0, 8*16+ 6, 0x0 +NORM },	/* 3197  */
{  3194,  3199, 8*16+ 2, 0x10+NORM },	/* 3198 dia 7 */
{     0,  3200, 5*16+ 2, 0x0 +NORM },	/* 3199  */
{     0,  3201, 7*16+ 1, 0x10+NORM },	/* 3200  */
{     0,  3202, 3*16+ 3, 0x0 +NORM },	/* 3201  */
{     0,  3203, 4*16+ 2, 0x10+NORM },	/* 3202  */
{     0,     0, 5*16+10, 0x0 +NORM },	/* 3203  */
{  3190,  3205,10*16+ 5, 0x10+NORM },	/* 3204 dia 8 */
{     0,  3206, 6*16+ 3, 0x0 +NORM },	/* 3205  */
{     0,  3207, 6*16+ 2, 0x10+NORM },	/* 3206  */
{     0,  3208, 5*16+ 2, 0x0 +NORM },	/* 3207  */
{     0,  3209, 7*16+ 2, 0x10+NORM },	/* 3208  */
{     0,  3215, 3*16+ 3, 0x0 +NORM },	/* 3209  */
{     0,  3211, 4*16+ 2, 0x10+NORM },	/* 3210  */
{     0,  3212,10*16+ 4, 0x0 +NORM },	/* 3211  */
{     0,  3213, 9*16+ 3, 0x10+NORM },	/* 3212  */
{     0,  3214, 9*16+ 6, 0x0 +NORM },	/* 3213  */
{     0,     0, 5*16+10, 0x10+NORM },	/* 3214  */
{  3210,  3216, 9*16+ 6, 0x10+NORM },	/* 3215 dia 11 */
{     0,  3217, 4*16+ 2, 0x0 +NORM },	/* 3216  */
{     0,     0, 8*16+ 5, 0x10+NORM },	/* 3217  */
{  3185,  3221, 5*16+10, 0x0 +NORM },	/* 3218 dia 12 */
{     0,  3220, 7*16+ 8, 0x10+BAD  },	/* 3219  */
{     0,     0, 6*16+ 6, 0x0 +NORM },	/* 3220  */
{  3219,  3229, 5*16+ 6, 0x10+NORM },	/* 3221 dia 13 */
{     0,  3223, 6*16+ 3, 0x0 +NORM },	/* 3222  */
{     0,  3224, 6*16+ 2, 0x10+NORM },	/* 3223  */
{     0,  3236, 7*16+ 4, 0x0 +NORM },	/* 3224  */
{     0,  3226, 8*16+ 3, 0x10+NORM },	/* 3225  */
{     0,  3227, 4*16+ 4, 0x0 +NORM },	/* 3226  */
{     0,  3228, 3*16+ 3, 0x10+NORM },	/* 3227  */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 3228  */
{  3222,  3230, 4*16+ 4, 0x0 +NORM },	/* 3229 dia 14 */
{     0,  3231, 7*16+ 4, 0x10+NORM },	/* 3230  */
{     0,  3232, 6*16+ 3, 0x0 +NORM },	/* 3231  */
{     0,  3233, 6*16+ 2, 0x10+NORM },	/* 3232  */
{     0,  3234, 3*16+ 3, 0x0 +NORM },	/* 3233  */
{     0,  3235, 7*16+ 5, 0x10+NORM },	/* 3234  */
{     0,     0, 3*16+ 2, 0x0 +NORM },	/* 3235  */
{  3225,  3237, 4*16+ 4, 0x10+NORM },	/* 3236 dia 15 */
{     0,  3238, 4*16+ 5, 0x0 +NORM },	/* 3237  */
{     0,  3239, 5*16+ 5, 0x10+NORM },	/* 3238  */
{     0,  3240, 7*16+ 2, 0x0 +NORM },	/* 3239  */
{     0,  3241, 5*16+ 2, 0x10+NORM },	/* 3240  */
{     0,  3242, 8*16+ 3, 0x0 +NORM },	/* 3241  */
{     0,     0, 2*16+ 3, 0x10+NORM },	/* 3242  */
{  3180,  3244, 4*16+ 4, 0x0 +BAD  },	/* 3243 dia 16 */
{     0,  3245, 4*16+ 5, 0x0 +NORM },	/* 3244  */
{     0,  3247, 5*16+ 5, 0x10+NORM },	/* 3245  */
{     0,  3255, 6*16+ 4, 0x0 +NORM },	/* 3246  */
{  3246,  3248, 5*16+ 3, 0x0 +NORM },	/* 3247 dia 16 */
{     0,  3360, 5*16+ 2, 0x10+NORM },	/* 3248  */
{     0,  3250, 5*16+ 6, 0x10+BAD  },	/* 3249 dia 17 */
{     0,  3251, 2*16+ 4, 0x0 +NORM },	/* 3250  */
{     0,  3252, 6*16+ 2, 0x10+NORM },	/* 3251  */
{     0,  3253, 4*16+ 7, 0x0 +NORM },	/* 3252  */
{     0,  3254, 4*16+ 6, 0x10+NORM },	/* 3253  */
{     0,     0, 3*16+ 7, 0x0 +NORM },	/* 3254  */
{  3249,  3256, 6*16+ 5, 0x10+NORM },	/* 3255 dia 18 */
{     0,  3257, 7*16+ 4, 0x0 +NORM },	/* 3256  */
{     0,  3336, 7*16+ 5, 0x10+NORM },	/* 3257  */
{     0,  3309, 5*16+ 2, 0x0 +NORM },	/* 3258  */
{     0,  3288, 4*16+ 2, 0x10+NORM },	/* 3259  */
{     0,  3301, 3*16+ 4, 0x0 +NORM },	/* 3260  */
{     0,  3262, 3*16+ 3, 0x10+NORM },	/* 3261  */
{     0,  3263, 2*16+ 3, 0x0 +NORM },	/* 3262  */
{     0,  3264, 2*16+ 2, 0x10+NORM },	/* 3263  */
{     0,  3265, 1*16+ 4, 0x0 +NORM },	/* 3264  */
{     0,  3266, 6*16+ 2, 0x10+NORM },	/* 3265  */
{     0,  3267, 5*16+ 3, 0x0 +NORM },	/* 3266  */
{     0,  3268, 5*16+ 1, 0x10+NORM },	/* 3267  */
{     0,  3269, 8*16+ 4, 0x0 +NORM },	/* 3268  */
{     0,  3270, 4*16+ 6, 0x10+NORM },	/* 3269  */
{     0,  3271, 3*16+ 1, 0x0 +NORM },	/* 3270  */
{     0,  3272, 4*16+ 1, 0x10+NORM },	/* 3271  */
{     0,  3273, 1*16+ 2, 0x0 +NORM },	/* 3272  */
{     0,  3274, 2*16+ 1, 0x10+NORM },	/* 3273  */
{     0,  3275, 3*16+ 6, 0x0 +NORM },	/* 3274  */
{     0,  3276, 3*16+ 7, 0x10+NORM },	/* 3275  */
{     0,  3277, 7*16+ 2, 0x0 +NORM },	/* 3276  */
{     0,  3278, 2*16+ 6, 0x10+NORM },	/* 3277  */
{     0,  3279, 2*16+ 5, 0x0 +NORM },	/* 3278  */
{     0,  3280, 1*16+ 6, 0x10+NORM },	/* 3279  */
{     0,  3281, 1*16+ 5, 0x0 +NORM },	/* 3280  */
{     0,  3282, 2*16+ 8, 0x10+NORM },	/* 3281  */
{     0,  3283, 5*16+ 6, 0x0 +NORM },	/* 3282  */
{     0,  3284, 4*16+ 7, 0x10+NORM },	/* 3283  */
{     0,  3285, 6*16+ 3, 0x0 +NORM },	/* 3284  */
{     0,  3286, 6*16+ 7, 0x10+NORM },	/* 3285  */
{     0,  3287, 6*16+ 1, 0x0 +NORM },	/* 3286  */
{     0,     0, 3*16+ 2, 0x10+NORM },	/* 3287  */
{  3260,  3289, 2*16+ 3, 0x0 +BAD  },	/* 3288 dia 21 */
{     0,  3290, 8*16+ 4, 0x10+NORM },	/* 3289  */
{     0,  3291, 6*16+ 2, 0x0 +NORM },	/* 3290  */
{     0,  3292, 2*16+ 2, 0x10+NORM },	/* 3291  */
{     0,  3293, 3*16+ 3, 0x0 +NORM },	/* 3292  */
{     0,  3294, 3*16+ 2, 0x10+NORM },	/* 3293  */
{     0,  3295, 1*16+ 2, 0x0 +NORM },	/* 3294  */
{     0,  3298, 4*16+ 7, 0x10+NORM },	/* 3295  */
{     0,  3297, 3*16+ 4, 0x0 +NORM },	/* 3296  */
{     0,     0, 9*16+ 5, 0x10+NORM },	/* 3297  */
{  3296,  3299, 3*16+ 4, 0x10+NORM },	/* 3298 dia 21 */
{     0,  3300, 2*16+ 4, 0x0 +NORM },	/* 3299  */
{     0,     0, 2*16+ 5, 0x10+NORM },	/* 3300  */
{  3261,  3302, 2*16+ 3, 0x10+BAD  },	/* 3301 dia 22 */
{     0,  3303, 3*16+ 3, 0x0 +NORM },	/* 3302  */
{     0,  3304, 3*16+ 2, 0x10+NORM },	/* 3303  */
{     0,  3305, 2*16+ 2, 0x0 +NORM },	/* 3304  */
{     0,  3306, 2*16+ 1, 0x10+NORM },	/* 3305  */
{     0,  3307, 1*16+ 2, 0x0 +NORM },	/* 3306  */
{     0,  3308, 1*16+ 3, 0x10+NORM },	/* 3307  */
{     0,     0, 4*16+ 1, 0x0 +NORM },	/* 3308  */
{  3259,  3310, 3*16+ 4, 0x10+NORM },	/* 3309 dia 23 */
{     0,  3311, 4*16+ 7, 0x0 +NORM },	/* 3310  */
{     0,  3312, 8*16+ 4, 0x10+NORM },	/* 3311  */
{     0,  3313, 8*16+ 3, 0x0 +NORM },	/* 3312  */
{     0,  3314, 9*16+ 3, 0x10+NORM },	/* 3313  */
{     0,  3315, 4*16+ 2, 0x0 +NORM },	/* 3314  */
{     0,  3316, 2*16+ 5, 0x10+NORM },	/* 3315  */
{     0,  3317, 3*16+ 7, 0x0 +NORM },	/* 3316  */
{     0,  3318, 2*16+ 3, 0x10+NORM },	/* 3317  */
{     0,  3319, 8*16+ 2, 0x0 +NORM },	/* 3318  */
{     0,  3320, 9*16+ 2, 0x10+NORM },	/* 3319  */
{     0,  3321, 7*16+ 2, 0x0 +NORM },	/* 3320  */
{     0,  3322, 3*16+ 2, 0x10+NORM },	/* 3321  */
{     0,  3323, 2*16+ 6, 0x0 +NORM },	/* 3322  */
{     0,  3324, 1*16+ 5, 0x10+NORM },	/* 3323 dia 23 */
{     0,  3326, 2*16+ 1, 0x0 +NORM },	/* 3324  */
{     0,     0, 8*16+ 5, 0x10+NORM },	/* 3325  */
{  3325,  3327, 3*16+ 1, 0x10+BAD  },	/* 3326 dia 24 */
{     0,  3328, 6*16+ 7, 0x0 +NORM },	/* 3327  */
{     0,  3329, 8*16+ 5, 0x10+NORM },	/* 3328  */
{     0,  3330, 3*16+10, 0x0 +NORM },	/* 3329  */
{     0,  3331, 8*16+ 1, 0x10+BAD  },	/* 3330  */
{     0,  3332, 5*16+ 3, 0x0 +NORM },	/* 3331  */
{     0,  3333, 4*16+ 1, 0x10+NORM },	/* 3332  */
{     0,  3334, 5*16+ 1, 0x0 +NORM },	/* 3333  */
{     0,  3335, 2*16+ 2, 0x10+NORM },	/* 3334  */
{     0,     0, 7*16+ 1, 0x0 +NORM },	/* 3335  */
{  3258,  3337, 8*16+ 4, 0x0 +BAD  },	/* 3336 dia 26 */
{     0,  3338, 4*16+ 6, 0x10+NORM },	/* 3337  */
{     0,  3339, 3*16+ 6, 0x0 +NORM },	/* 3338  */
{     0,  3340, 3*16+ 7, 0x10+NORM },	/* 3339  */
{     0,  3341, 2*16+ 3, 0x0 +NORM },	/* 3340  */
{     0,  3342, 2*16+ 2, 0x10+NORM },	/* 3341  */
{     0,  3343, 5*16+ 6, 0x0 +NORM },	/* 3342  */
{     0,  3344, 4*16+ 7, 0x10+NORM },	/* 3343  */
{     0,  3345, 8*16+ 5, 0x0 +NORM },	/* 3344  */
{     0,  3346, 6*16+ 7, 0x10+NORM },	/* 3345  */
{     0,  3347, 1*16+ 2, 0x0 +NORM },	/* 3346  */
{     0,  3348, 2*16+ 4, 0x10+NORM },	/* 3347  */
{     0,  3349, 3*16+ 4, 0x0 +NORM },	/* 3348  */
{     0,  3350, 3*16+ 3, 0x10+NORM },	/* 3349  */
{     0,  3351, 1*16+ 4, 0x0 +NORM },	/* 3350  */
{     0,  3352, 1*16+ 3, 0x10+NORM },	/* 3351  */
{     0,  3353, 7*16+ 6, 0x0 +NORM },	/* 3352  */
{     0,  3354, 6*16+ 6, 0x10+NORM },	/* 3353  */
{     0,  3355, 2*16+ 3, 0x0 +NORM },	/* 3354  */
{     0,  3356, 2*16+ 5, 0x10+NORM },	/* 3355  */
{     0,  3357, 2*16+ 6, 0x0 +NORM },	/* 3356  */
{     0,     0, 1*16+ 3, 0x10+NORM },	/* 3357  */
{     0,  3359, 6*16+ 2, 0x0 +NORM },	/* 3358 dia 28 */
{     0,     0, 4*16+ 2, 0x10+NORM },	/* 3359  */
{  3358,  3361, 4*16+ 2, 0x0 +BAD  },	/* 3360 dia 29 */
{     0,  3362, 3*16+ 2, 0x10+NORM },	/* 3361  */
{     0,  3363, 3*16+ 3, 0x0 +NORM },	/* 3362  */
{     0,  3364, 4*16+ 1, 0x10+NORM },	/* 3363  */
{     0,  3365, 3*16+ 4, 0x0 +NORM },	/* 3364  */
{     0,  3366, 4*16+ 2, 0x10+NORM },	/* 3365  */
{     0,  3367, 7*16+ 4, 0x0 +NORM },	/* 3366  */
{     0,  3370, 5*16+ 6, 0x10+NORM },	/* 3367  */
{     0,  3369, 7*16+ 3, 0x0 +NORM },	/* 3368  */
{     0,  3374, 2*16+ 2, 0x10+NORM },	/* 3369  */
{  3368,  3371, 2*16+ 2, 0x0 +NORM },	/* 3370 dia 29 */
{     0,  3372, 7*16+ 3, 0x10+NORM },	/* 3371  */
{     0,  3373, 6*16+ 4, 0x0 +NORM },	/* 3372  */
{     0,     0, 8*16+ 4, 0x10+NORM },	/* 3373  */
{     0,  3375, 4*16+ 7, 0x0 +NORM },	/* 3374 dia 30 */
{     0,  3376, 5*16+ 7, 0x10+NORM },	/* 3375  */
{     0,  3377, 4*16+ 8, 0x0 +NORM },	/* 3376  */
{     0,  3378, 4*16+ 6, 0x10+NORM },	/* 3377  */
{     0,  3379, 3*16+ 7, 0x0 +NORM },	/* 3378  */
{     0,  3380, 3*16+ 6, 0x10+NORM },	/* 3379  */
{     0,  3381, 2*16+ 6, 0x0 +NORM },	/* 3380  */
{     0,  3386, 2*16+ 5, 0x10+NORM },	/* 3381  */
{     0,  3383, 2*16+ 8, 0x0 +NORM },	/* 3382  */
{     0,  3384, 1*16+ 6, 0x10+NORM },	/* 3383  */
{     0,  3385, 2*16+ 7, 0x0 +NORM },	/* 3384  */
{     0,     0, 2*16+ 4, 0x10+NORM },	/* 3385  */
{  3382,  3387, 2*16+ 4, 0x0 +NORM },	/* 3386 dia 31 */
{     0,  3388, 2*16+ 7, 0x10+NORM },	/* 3387  */
{     0,  3390, 1*16+ 5, 0x0 +NORM },	/* 3388  */
{     0,     0, 4*16+ 9, 0x10+NORM },	/* 3389  */
{  3389,  3391, 2*16+ 9, 0x10+NORM },	/* 3390 dia 31 */
{     0,  3392, 5*16+ 8, 0x0 +NORM },	/* 3391  */
{     0,  3393, 7*16+ 7, 0x10+NORM },	/* 3392  */
{     0,  3394, 1*16+ 3, 0x0 +NORM },	/* 3393  */
{     0,  3395, 3*16+10, 0x10+NORM },	/* 3394  */
{     0,  3396, 2*16+ 3, 0x0 +NORM },	/* 3395  */
{     0,     0, 7*16+ 2, 0x10+NORM },	/* 3396  */
{  3179,  3609, 3*16+ 3, 0x0 +NORM },	/* 3397 sec 3 3-3 attach */
{     0,  3399, 3*16+ 2, 0x10+NORM },	/* 3398  */
{     0,  3411, 4*16+ 4, 0x0 +NORM },	/* 3399  */
{     0,  3401, 5*16+ 3, 0x10+NORM },	/* 3400  */
{     0,  3402, 2*16+ 3, 0x0 +NORM },	/* 3401  */
{     0,  3407, 7*16+ 4, 0x10+NORM },	/* 3402  */
{     0,  3404, 2*16+ 2, 0x0 +NORM },	/* 3403  */
{     0,     0,11*16+ 3, 0x10+NORM },	/* 3404  */
{  3403,  3406, 2*16+ 2, 0x10+NORM },	/* 3405 dia 32 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 3406  */
{  3405,  3408, 4*16+ 6, 0x10+NORM },	/* 3407 dia 33 */
{     0,  3409, 5*16+ 4, 0x0 +NORM },	/* 3408  */
{     0,  3410, 6*16+ 4, 0x10+NORM },	/* 3409  */
{     0,     0, 2*16+ 2, 0x0 +NORM },	/* 3410  */
{  3400,  3428, 2*16+ 3, 0x10+NORM },	/* 3411 dia 34 */
{     0,  3413, 5*16+ 3, 0x0 +NORM },	/* 3412  */
{     0,  3414, 3*16+ 4, 0x10+NORM },	/* 3413  */
{     0,  3420, 4*16+ 2, 0x0 +NORM },	/* 3414  */
{     0,  3416, 3*16+ 3, 0x10+NORM },	/* 3415  */
{     0,  3417, 5*16+ 5, 0x0 +NORM },	/* 3416  */
{     0,  3418, 2*16+ 5, 0x10+BAD  },	/* 3417  */
{     0,  3419, 3*16+ 6, 0x0 +NORM },	/* 3418  */
{     0,     0, 2*16+ 6, 0x10+NORM },	/* 3419  */
{  3415,  3421, 4*16+ 5, 0x10+NORM },	/* 3420 dia 36 */
{     0,  3422, 3*16+ 3, 0x0 +NORM },	/* 3421  */
{     0,  3423, 2*16+ 4, 0x10+NORM },	/* 3422  */
{     0,  3424, 5*16+ 4, 0x0 +NORM },	/* 3423  */
{     0,  3425, 3*16+ 6, 0x10+NORM },	/* 3424  */
{     0,  3426, 5*16+ 5, 0x0 +NORM },	/* 3425  */
{     0,  3427, 4*16+ 6, 0x10+NORM },	/* 3426  */
{     0,     0, 9*16+ 3, 0x0 +NORM },	/* 3427  */
{  3412,  3429, 5*16+ 4, 0x0 +NORM },	/* 3428 dia 37 */
{     0,  3430, 3*16+ 4, 0x10+NORM },	/* 3429  */
{     0,  3431, 2*16+ 4, 0x0 +NORM },	/* 3430  */
{     0,  3432, 3*16+ 3, 0x10+NORM },	/* 3431  */
{     0,  3433, 6*16+ 3, 0x0 +NORM },	/* 3432  */
{     0,  3434, 3*16+11, 0x10+NORM },	/* 3433  */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 3434  */
{  3397,  3493, 7*16+ 3, 0x0 +NORM },	/* 3435 sec 4 counter pincer */
{     0,  3437, 5*16+ 4, 0x10+NORM },	/* 3436  */
{     0,  3486,10*16+ 3, 0x0 +NORM },	/* 3437  */
{     0,  3485, 4*16+ 6, 0x10+NORM },	/* 3438  */
{     0,  3473, 2*16+ 3, 0x0 +FOLL },	/* 3439  */
{     0,  3446, 3*16+ 6, 0x10+NORM },	/* 3440  */
{     0,  3444, 3*16+ 2, 0x0 +NORM },	/* 3441  */
{     0,  3443, 2*16+ 5, 0x10+NORM },	/* 3442  */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 3443  */
{  3442,  3445, 4*16+ 2, 0x10+BAD  },	/* 3444 dia 40 */
{     0,     0, 2*16+ 5, 0x0 +NORM },	/* 3445  */
{  3441,  3447, 2*16+ 5, 0x0 +NORM },	/* 3446 dia 41 */
{     0,  3448, 3*16+ 2, 0x10+NORM },	/* 3447  */
{     0,  3449, 2*16+ 2, 0x0 +NORM },	/* 3448  */
{     0,  3450, 2*16+ 1, 0x10+NORM },	/* 3449  */
{     0,  3451, 4*16+ 5, 0x0 +NORM },	/* 3450  */
{     0,  3452, 5*16+ 5, 0x10+NORM },	/* 3451  */
{     0,  3453, 2*16+ 6, 0x0 +NORM },	/* 3452  */
{     0,  3454, 2*16+ 7, 0x10+NORM },	/* 3453  */
{     0,  3455, 3*16+ 3, 0x0 +NORM },	/* 3454  */
{     0,  3456, 5*16+ 2, 0x10+NORM },	/* 3455  */
{     0,     0, 1*16+ 4, 0x0 +NORM },	/* 3456  */
{  3440,  3458, 2*16+ 2, 0x10+NORM },	/* 3457 dia 42 */
{     0,  3465, 3*16+ 2, 0x0 +NORM },	/* 3458  */
{     0,  3460, 2*16+ 4, 0x10+NORM },	/* 3459  */
{     0,  3461, 1*16+ 2, 0x0 +NORM },	/* 3460  */
{     0,  3462, 2*16+ 1, 0x10+NORM },	/* 3461  */
{     0,  3463, 3*16+ 1, 0x0 +NORM },	/* 3462  */
{     0,  3464, 3*16+ 4, 0x10+NORM },	/* 3463  */
{     0,     0, 5*16+ 2, 0x0 +NORM },	/* 3464  */
{  3459,  3466, 3*16+ 3, 0x10+BAD  },	/* 3465 dia 43 */
{     0,  3467, 1*16+ 2, 0x0 +NORM },	/* 3466  */
{     0,  3468, 2*16+ 1, 0x10+NORM },	/* 3467  */
{     0,  3469, 3*16+ 1, 0x0 +NORM },	/* 3468  */
{     0,  3470, 2*16+ 4, 0x10+NORM },	/* 3469  */
{     0,  3471, 1*16+ 1, 0x0 +NORM },	/* 3470  */
{     0,  3472, 3*16+ 4, 0x10+NORM },	/* 3471  */
{     0,     0, 6*16+ 2, 0x0 +NORM },	/* 3472  */
{  3457,  3474, 3*16+ 2, 0x10+BAD  },	/* 3473 dia 44 */
{     0,  3475, 2*16+ 7, 0x0 +NORM },	/* 3474  */
{     0,  3476, 2*16+ 8, 0x10+NORM },	/* 3475  */
{     0,  3477, 4*16+ 5, 0x0 +NORM },	/* 3476  */
{     0,  3478, 5*16+ 5, 0x10+NORM },	/* 3477  */
{     0,  3479, 2*16+ 2, 0x0 +NORM },	/* 3478  */
{     0,  3480, 2*16+ 1, 0x10+NORM },	/* 3479  */
{     0,  3481, 3*16+ 3, 0x0 +NORM },	/* 3480  */
{     0,  3482, 5*16+ 2, 0x10+NORM },	/* 3481  */
{     0,  3483, 3*16+ 6, 0x0 +NORM },	/* 3482  */
{     0,  3484, 3*16+ 7, 0x10+NORM },	/* 3483  */
{     0,     0, 1*16+ 7, 0x0 +NORM },	/* 3484  */
{  3439,     0, 4*16+ 2, 0x10+NORM },	/* 3485 dia 45 */
{  3438,  3489, 3*16+ 4, 0x10+NORM },	/* 3486 dia 46 */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 3487  */
{  3487,     0, 4*16+ 5, 0x0 +NORM },	/* 3488 dia 46 */
{  3488,  3490, 2*16+ 4, 0x0 +NORM },	/* 3489 dia 46 */
{     0,  3491, 2*16+ 3, 0x10+NORM },	/* 3490  */
{     0,  3492, 6*16+ 5, 0x0 +NORM },	/* 3491  */
{     0,     0, 4*16+ 6, 0x10+NORM },	/* 3492  */
{  3436,  3496, 4*16+ 5, 0x10+NORM },	/* 3493 dia 47 */
{     0,  3495,10*16+ 3, 0x0 +BAD  },	/* 3494  */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 3495  */
{  3494,  3501, 4*16+ 4, 0x0 +NORM },	/* 3496 dia 49 */
{     0,  3498, 3*16+ 4, 0x10+NORM },	/* 3497  */
{     0,  3499, 5*16+ 4, 0x0 +NORM },	/* 3498  */
{     0,  3500, 3*16+ 3, 0x10+NORM },	/* 3499  */
{     0,     0, 4*16+ 6, 0x0 +NORM },	/* 3500  */
{  3497,  3502, 5*16+ 4, 0x10+NORM },	/* 3501 dia 49 */
{     0,  3514, 3*16+ 4, 0x0 +NORM },	/* 3502  */
{     0,  3543, 5*16+ 2, 0x10+NORM },	/* 3503  */
{     0,  3531, 3*16+ 7, 0x0 +NORM },	/* 3504  */
{     0,  3518, 4*16+ 8, 0x10+NORM },	/* 3505  */
{     0,  3516, 5*16+ 7, 0x0 +NORM },	/* 3506  */
{     0,  3508, 3*16+ 3, 0x10+NORM },	/* 3507  */
{     0,  3509, 5*16+ 5, 0x0 +NORM },	/* 3508  */
{     0,  3510, 6*16+ 5, 0x10+NORM },	/* 3509  */
{     0,  3511, 4*16+ 6, 0x0 +NORM },	/* 3510  */
{     0,     0, 6*16+ 4, 0x10+NORM },	/* 3511  */
{  3503,  3513, 5*16+ 5, 0x10+BAD  },	/* 3512 dia 49 */
{     0,     0, 5*16+ 3, 0x0 +NORM },	/* 3513  */
{  3512,  3515, 3*16+ 3, 0x10+BAD  },	/* 3514 dia 49 */
{     0,     0, 5*16+ 3, 0x0 +NORM },	/* 3515  */
{  3507,  3517, 5*16+ 5, 0x10+BAD  },	/* 3516 dia 50 */
{     0,     0, 4*16+ 7, 0x0 +NORM },	/* 3517  */
{  3506,  3519, 5*16+ 5, 0x0 +NORM },	/* 3518 dia 51 */
{     0,  3520, 4*16+ 6, 0x10+NORM },	/* 3519  */
{     0,  3521, 4*16+ 7, 0x0 +NORM },	/* 3520  */
{     0,  3522, 5*16+ 6, 0x10+NORM },	/* 3521  */
{     0,  3523, 5*16+ 7, 0x0 +NORM },	/* 3522  */
{     0,  3529, 6*16+ 5, 0x10+NORM },	/* 3523  */
{     0,  3525, 4*16+10, 0x0 +NORM },	/* 3524  */
{     0,  3526, 6*16+ 7, 0x10+NORM },	/* 3525 dia 51 */
{     0,  3527, 5*16+ 8, 0x0 +NORM },	/* 3526  */
{     0,  3528, 6*16+ 8, 0x10+NORM },	/* 3527  */
{     0,     0, 5*16+ 9, 0x0 +NORM },	/* 3528  */
{  3524,  3530, 6*16+ 7, 0x0 +NORM },	/* 3529 dia 51 */
{     0,     0, 4*16+11, 0x10+NORM },	/* 3530  */
{  3505,  3532, 4*16+ 6, 0x10+NORM },	/* 3531 dia 53 */
{     0,  3533, 4*16+ 7, 0x0 +NORM },	/* 3532  */
{     0,  3534, 3*16+ 6, 0x10+NORM },	/* 3533  */
{     0,  3535, 2*16+ 6, 0x0 +NORM },	/* 3534  */
{     0,  3538, 2*16+ 5, 0x10+NORM },	/* 3535  */
{     0,  3537, 2*16+ 8, 0x0 +NORM },	/* 3536  */
{     0,  3542, 2*16+ 4, 0x10+NORM },	/* 3537  */
{  3536,  3539, 2*16+ 4, 0x0 +NORM },	/* 3538 dia 54 */
{     0,  3540, 2*16+ 7, 0x10+NORM },	/* 3539  */
{     0,  3541, 1*16+ 5, 0x0 +NORM },	/* 3540  */
{     0,     0, 5*16+ 7, 0x10+NORM },	/* 3541  */
{     0,     0, 6*16+ 7, 0x0 +NORM },	/* 3542 dia 54 */
{  3504,  3544, 4*16+ 6, 0x0 +NORM },	/* 3543 dia 53 */
{     0,  3545, 5*16+ 5, 0x10+NORM },	/* 3544  */
{     0,  3546, 5*16+ 6, 0x0 +NORM },	/* 3545  */
{     0,  3547, 6*16+ 6, 0x10+NORM },	/* 3546  */
{     0,  3548, 4*16+ 8, 0x0 +NORM },	/* 3547  */
{     0,  3549, 4*16+ 9, 0x10+NORM },	/* 3548  */
{     0,  3550, 5*16+ 8, 0x0 +NORM },	/* 3549  */
{     0,  3551, 3*16+ 7, 0x10+NORM },	/* 3550  */
{     0,  3552, 3*16+ 6, 0x0 +NORM },	/* 3551  */
{     0,  3553, 2*16+ 3, 0x10+NORM },	/* 3552  */
{     0,     0, 4*16+11, 0x0 +NORM },	/* 3553  */
{  3435,  3563, 4*16+ 6, 0x0 +NORM },	/* 3554 sec 5 fiagonal play */
{     0,  3556, 6*16+ 4, 0x10+NORM },	/* 3555  */
{     0,  3557, 2*16+ 3, 0x0 +NORM },	/* 3556  */
{     0,  3558,10*16+ 3, 0x10+NORM },	/* 3557  */
{     0,  3559, 5*16+ 7, 0x10+NORM },	/* 3558 dia 56 */
{     0,  3560, 5*16+ 6, 0x0 +NORM },	/* 3559  */
{     0,  3561, 6*16+ 7, 0x10+NORM },	/* 3560  */
{     0,  3562, 6*16+ 6, 0x0 +NORM },	/* 3561  */
{     0,     0, 7*16+ 7, 0x10+NORM },	/* 3562  */
{  3555,  3564, 3*16+ 4, 0x10+NORM },	/* 3563 dia 57 */
{     0,  3565, 2*16+ 4, 0x0 +NORM },	/* 3564  */
{     0,  3566, 2*16+ 3, 0x10+NORM },	/* 3565  */
{     0,  3567, 4*16+ 4, 0x0 +NORM },	/* 3566  */
{     0,  3568, 3*16+ 3, 0x10+NORM },	/* 3567  */
{     0,  3569, 5*16+ 4, 0x0 +NORM },	/* 3568  */
{     0,  3570, 6*16+ 3, 0x10+NORM },	/* 3569  */
{     0,     0, 4*16+11, 0x0 +NORM },	/* 3570  */
{  3554,  3593, 5*16+ 7, 0x0 +NORM },	/* 3571 sec 6 diagonal jump */
{     0,  3573, 4*16+ 6, 0x10+BAD  },	/* 3572  */
{     0,  3574, 5*16+ 4, 0x0 +NORM },	/* 3573  */
{     0,  3575, 5*16+ 3, 0x10+NORM },	/* 3574  */
{     0,  3576, 6*16+ 4, 0x0 +NORM },	/* 3575 dia 59 */
{     0,  3577, 7*16+ 3, 0x10+NORM },	/* 3576  */
{     0,     0, 5*16+ 6, 0x0 +NORM },	/* 3577  */
{  3572,  3582, 4*16+ 5, 0x10+NORM },	/* 3578 dia 60 */
{     0,  3580, 3*16+ 7, 0x0 +NORM },	/* 3579  */
{     0,  3581, 3*16+ 4, 0x10+NORM },	/* 3580  */
{     0,     0, 4*16+ 8, 0x0 +NORM },	/* 3581  */
{  3579,  3583, 4*16+ 6, 0x0 +BAD  },	/* 3582 dia 61 */
{     0,  3584, 5*16+ 5, 0x10+NORM },	/* 3583  */
{     0,  3585, 3*16+ 4, 0x0 +NORM },	/* 3584  */
{     0,  3586, 3*16+ 3, 0x10+NORM },	/* 3585  */
{     0,     0, 4*16+ 8, 0x0 +FOLL },	/* 3586  */
{  3578,  3588, 5*16+ 4, 0x10+BAD  },	/* 3587 dia 62 */
{     0,  3589, 3*16+ 7, 0x0 +NORM },	/* 3588  */
{     0,  3590, 4*16+ 8, 0x10+NORM },	/* 3589  */
{     0,  3591, 4*16+ 7, 0x0 +NORM },	/* 3590  */
{     0,  3592, 3*16+12, 0x10+NORM },	/* 3591  */
{     0,     0, 2*16+ 3, 0x0 +NORM },	/* 3592  */
{  3587,  3594, 5*16+ 8, 0x10+NORM },	/* 3593 dia 63 */
{     0,  3595, 6*16+ 8, 0x0 +NORM },	/* 3594  */
{     0,  3596, 5*16+ 9, 0x10+NORM },	/* 3595  */
{     0,  3597, 4*16+ 7, 0x0 +NORM },	/* 3596  */
{     0,  3598, 7*16+ 3, 0x10+NORM },	/* 3597  */
{     0,  3599, 4*16+ 8, 0x0 +NORM },	/* 3598  */
{     0,  3600, 4*16+ 9, 0x10+NORM },	/* 3599  */
{     0,  3601, 3*16+ 9, 0x0 +NORM },	/* 3600  */
{     0,  3602, 3*16+10, 0x10+NORM },	/* 3601  */
{     0,  3603, 2*16+ 9, 0x0 +NORM },	/* 3602  */
{     0,  3604, 2*16+10, 0x10+NORM },	/* 3603  */
{     0,  3605, 2*16+ 8, 0x0 +NORM },	/* 3604  */
{     0,     0, 5*16+11, 0x10+NORM },	/* 3605  */
{    42,  3608, 5*16+ 5, 0x10+BAD  },	/* 3606 5-5 for 9 line games */
{     0,     0, 3*16+ 5, 0x0 +NORM },	/* 3607  */
{  3607,  3615, 5*16+ 3, 0x10+NORM },	/* 3608 another 5-5 respons */
{  3398,  3610, 4*16+ 4, 0x10+BAD  },	/* 3609 bad move from dragon */
{     0,  3611, 3*16+ 4, 0x0 +NORM },	/* 3610  */
{     0,  3612, 4*16+ 5, 0x10+BAD  },	/* 3611  */
{     0,     0, 2*16+ 7, 0x0 +NORM },	/* 3612  */
{  3606,  3614, 3*16+ 6, 0x10+IGNR },	/* 3613  For 9 line games - corners */
{     0,     0, 3*16+ 4, 0x10+NORM },	/* 3614 make large knights */
{     0,     0, 3*16+ 5, 0x0 +NORM },  /* 3615 respond */
};
